

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGBORDERFILLER_H
#define ADORE_IMGPROCESSINGBORDERFILLER_H

#include <adrcore/imgprocessing/channelprocessor.h>

namespace adore
{

	namespace core
	{

		/**
		 * A Chanelprocessor that fills borders with a constant value.
		 * @tparam img_in_typeT the image type.
		 *  
		*/
		template <class img_in_typeT>
		class BorderFiller : public ChannelProcessor<1, 1, img_in_typeT, img_in_typeT>
		{
			public:
				typedef ChannelProcessor<1, 1, img_in_typeT, img_in_typeT> BaseT;
				typedef img_in_typeT img_in_type;
				typedef img_in_type img_out_type;

			protected:

				img_in_type v;	///the value to fill the border with
				uint w;		///the border width

			public:
				/**
				 * Constructor
				 * @param width the desired border width
				 * @param val the value to paint onto the border pixel
				 */
				BorderFiller ( uint width = 0, img_in_type val = img_in_type ( 0 ) ) :
						ChannelProcessor<1, 1, img_in_type, img_in_type>(),
						v ( val ),
						w ( width )
				{
				}
				
				/** 
				 * paints the image border
				 * @param in the input channel 
				 * @param out the output channel, currently only out == in is supported
				 */
				void process ( Channel<img_in_type> *in, Channel<img_out_type> *out )
				{
//					printf( "processing\n" );
					if ( in == out ) //only in place processing is supported
					{
						for ( uint pos = 0; pos < w; ++pos )
						{
							//top rows
							typename Channel<img_in_type>::Iterator1D itH = in->getXIterator ( pos, pos );//(row, col)
							typename Channel<img_in_type>::Iterator1D stopH = in->getXIterator ( pos, in->getWidth() - pos );
							for ( ; itH != stopH; ++itH )
							{
								*itH = v;
							}

							//bottom rows
							itH = in->getXIterator ( in->getHeight() - pos - 1, pos );
							stopH = in->getXIterator ( in->getHeight() - pos - 1, in->getWidth() - pos );
							for ( ; itH != stopH; ++itH )
							{
								*itH = v;
							}

							//left cols
							typename Channel<img_in_type>::Iterator1D itV = in->getYIterator ( pos + 1, pos );
							typename Channel<img_in_type>::Iterator1D stopV = in->getYIterator ( in->getHeight() - pos - 1, pos );
							for ( ; itV != stopV; ++itV )
							{
								*itV = v;
							}

							//right cols
							itV = in->getYIterator ( pos + 1, in->getWidth() - pos - 1 );
							stopV = in->getYIterator ( in->getHeight() - pos - 1, in->getWidth() - pos - 1 );
							for ( ; itV != stopV; ++itV )
							{
								*itV = v;
							}
						}
					}
					else
					{
						error ( "yet to implement\n" );
					}
				}

				/**
				 * @param value the border color of choice
				 */ 
				void setColor ( const img_in_type value )
				{
					v = value;
				}

				/**
				 * @return the border color of choice
				 */ 
				img_in_type getColor() const
				{
					return v;
				}

				/**
				 * @param value the border width of choice
				 */ 
				void setWidth ( const uint& theValue )
				{
					w = theValue;
				}

				/**
				 * @return the border width of choice
				 */ 
				uint getWidth() const
				{
					return w;
				}


		};

	}

}

#endif
