

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGCONVOLVER1D3_H
#define ADORE_IMGPROCESSINGCONVOLVER1D3_H

#include <adrcore/imgprocessing/channelprocessor.h>
#include <adrcore/imgprocessing/iteratorgetter.h>

namespace adore
{

	namespace core
	{

		/**
		 * A speciallized Convolver that takes 1D-Kernels of 3.
		 * Such a Kernel is applied horizontally or vetically, depending on the template parameter dirT
		 * @param dirT The direction in which the Kernel is shifted over the image, either DIR_X or DIR_Y,
		 *        wich means horizontally or vertically. The Kernel is oriented perpendicular to this direction.
		 *  
		 */
		template<class ConvKernT, int dirT>
		class Convolver1D3 : public ChannelProcessor<1, 1, typename ConvKernT::img_in_type, typename ConvKernT::img_out_type>
		{
			public:
				typedef typename ConvKernT::img_in_type img_in_type;
				typedef typename ConvKernT::img_out_type img_out_type;

			private:
				ConvKernT kernel;
				IteratorGetter<dirT> itGet;

			public:

				Convolver1D3() : ChannelProcessor<1, 1, img_in_type, img_out_type>()
				{
				}

				void process( Channel<img_in_type> *in, Channel<img_out_type> *out )
				{
					//left border
					typename Channel<img_in_type>::Iterator1D srcB =  itGet.get1DSwap( *in, 0, 0 );//(row, col)
					typename Channel<img_in_type>::Iterator1D srcC =  itGet.get1DSwap( *in, 1, 0 );//(row, col)
					typename Channel<img_out_type>::Iterator1D dst1  = itGet.get1DSwap( *out, 0, 0 );//(row, col)
					typename Channel<img_out_type>::Iterator1D stop1 = itGet.get1DSwap( *out, 0, itGet.getMajorDim( *out ) );
					for ( ; dst1 != stop1; ++dst1, ++srcB, ++srcC )
					{
// 						uint xb,yb, xc, yc;
// 						in->getCoors(srcB.getPointer(), xb, yb);
// 						in->getCoors(srcC.getPointer(), xc, yc);
// 						printf("(%d, %d):(%d, %d)<->(%d, %d)\n", in->getWidth(), in->getHeight(), xb, yb, xc, yc);
						*dst1 = kernel.convolveBorder( *srcC, *srcB );
					}

					//right border
					typename Channel<img_in_type>::Iterator1D srcA =  itGet.get1DSwap( *in, itGet.getMinorDim( *out ) - 2, 0 );
					srcB =  itGet.get1DSwap( *in, itGet.getMinorDim( *out ) - 1, 0 );
					dst1  = itGet.get1DSwap( *out, itGet.getMinorDim( *out ) - 1, 0 );
					stop1 = itGet.get1DSwap( *out, itGet.getMinorDim( *out ) - 1, itGet.getMajorDim( *out ) );
					for ( ; dst1 != stop1; ++dst1, ++srcA, ++srcB )
					{
						*dst1 = kernel.convolveBorder( *srcA, *srcB );
					}

					//the rest
					typename Channel<img_in_type>::Iterator2D src1 =  itGet.get2DSwap( *in, 0, 0, itGet.getMajorDim( *out ) );
					typename Channel<img_in_type>::Iterator2D src2 =  itGet.get2DSwap( *in, 1, 0, itGet.getMajorDim( *out ) );
					typename Channel<img_in_type>::Iterator2D src3 =  itGet.get2DSwap( *in, 2, 0, itGet.getMajorDim( *out ) );
					typename Channel<img_out_type>::Iterator2D dst  = itGet.get2DSwap( *out, 1, 0, itGet.getMajorDim( *out ) );
					typename Channel<img_out_type>::Iterator2D stop =
					        itGet.get2DSwap( *out, itGet.getMinorDim( *out ) - 1, 0, itGet.getMajorDim( *out ) );
					for ( ;dst != stop; ++dst, ++src1, ++src2, ++src3 )
					{
						*dst = kernel.convolve( *src1, *src2, *src3 );
					}
				}

		};

	}
}

#endif
