

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREIMGARRAYOWN_H
#define ADORE_COREIMGARRAYOWN_H

#include <adrcore/util/debug.h>
#include <adrcore/imgprocessing/roi.h>

namespace adore
{

	namespace core
	{

		struct ImgArrayOwn
		{
			char *imgData;
			uint width;
			uint height;
			uint widthStep;//always in bytes
			uint nChans;
		};

		typedef ImgArrayOwn ImgArray;

		template<class T>
		inline ImgArray *createImgArray( const int width, const int height, const int nChans )
		{
			const int align = 4;//8 maybe better for 64 Bit machines
			// 8 is also better for Pentium 2 - Pentium 4 which are 32 bit machines
			// but read data in 8-byte chunks from RAM

			ImgArray *arr = new ImgArray;
			arr->width = width;
			arr->height = height;
			arr->nChans = nChans;
			arr->widthStep = ( ( ( arr->width * arr->nChans * 8 * sizeof( T ) + 7 )/8 )+ align - 1 ) & ( ~( align - 1 ) );

			DBGOUT( DBGALL, "new ImgArray - w: %d, h: %d, c: %d\n", arr->width, arr->height, arr->nChans );
			DBGOUT( DBGALL, "allocating %d*%d bytes for image\n", arr->widthStep, height );
			arr->imgData = new char[height * arr->widthStep];
			return arr;
		}

		inline void releaseImgArray( ImgArray *arr )
		{
			if ( !arr ) return;
			delete [] arr->imgData;
			delete arr;
		}

		template<class T>
		inline T *getImgArrayData( ImgArray *arr )
		{
			return ( T* ) arr->imgData;
		}

		inline uint getImgArrayWidth( const ImgArray *arr )
		{
			if ( !arr ) return 0;
			return arr->width;
		}

		inline uint getImgArrayHeight( const ImgArray *arr )
		{
			if ( !arr ) return 0;
			return arr->height;
		}

		//for visual clipping
		inline void setImgArrayHeight( ImgArray *arr, const int h )
		{
			arr->height = h;
		}

		inline uint getImgArrayWidthStep( const ImgArray *arr )
		{
			return arr->widthStep;
		}

		inline uint getImgArrayNChannels( const ImgArray *arr )
		{
			return arr->nChans;
		}

	}
}

#endif
