

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREKERNEL2D_H
#define ADORE_COREKERNEL2D_H

#include <adrcore/imgprocessing/kerneloperatorbase.h>
#include <adrcore/types/aarray.h>

namespace adore
{

	namespace core
	{

		/**
			 
		*/
		template <class TypeT, class inT = TypeT, class ouT = TypeT>
		class Kernel2D : public KernelOperatorBase
		{
			public:
				typedef TypeT value_type;
				typedef TypeT T_numtype;
				typedef typename signedT< typename calcT<TypeT>::type >::type calcT;
				typedef AArray<TypeT, 2> ArrayT;
				typedef typename AArray<TypeT, 2>::iterator iterator;

				typedef inT img_in_type;
				typedef ouT img_out_type;

			public:
				Kernel2D( int width = 1, int height = 1, int centerX = -1, int centerY = -1 ):
						kernel( width, height ),
						kernelSum( 0 )
				{
					assert( width > 0 );
					assert( height > 0 );
					initSize( width, height, centerX, centerY );
				}

				/**
				 * Convolves.
				 * The iterator has to iterate through iterators that iterate lines :)
				 * That means:
				 * LineIterator = *IteratorT
				 * value = *LineIterator
				 */
				template<class IteratorT>
				TypeT convolve( IteratorT it )
				{
					typedef typename IteratorT::value_type lineIterator;
					calcT result = 0;
					iterator kit = begin();
					for ( int row = 0; row < (int)kernel.rows(); ++row, ++it )
					{
						lineIterator lit = *it;
						for ( int col = 0; col < (int)kernel.cols(); ++col, ++lit, ++kit )
						{
							result += *kit * *lit;
						}
					}

					if ( kernelSum == 0 ) return result;
					return img_out_type( result / kernelSum );
				}

				iterator begin()
				{
					return kernel.begin();
				}

				iterator end()
				{
					return kernel.end();
				}

				TypeT &operator()( int row, int col )
				{
					return kernel( row, col );
				}

				void resize( int width = 1, int height = 1, int centerX = -1, int centerY = -1 )
				{
					assert( width > 0 );
					assert( height > 0 );

					kernel.resize( width, height );
					initSize( width, height, centerX, centerY );
				}

				void init()
				{
					kernelSum = 0;
					for ( iterator kit = begin(); kit != end(); ++kit )
					{
						kernelSum += *kit;
					}
				}

			private:

				ArrayT kernel;
				calcT kernelSum;

				void initSize( int width = 1, int height = 1, int centerX = -1, int centerY = -1 )
				{
					if ( centerX < 0 ) centerX = ( width - 1 ) >> 1;
					if ( centerY < 0 ) centerY = ( height - 1 ) >> 1;

					setTopHood( centerY );
					setLeftHood( centerX );
					setBottomHood( height - centerY - 1 );
					setRightHood( width - centerX - 1 );
				}

		};

	}

}

#endif
