

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGMAXREGIONS_H
#define ADORE_IMGPROCESSINGMAXREGIONS_H

#include <adrcore/util/printimage.h>
#include <adrcore/imgprocessing/borderfiller.h>
#include <adrcore/imgprocessing/imageplane.h>
#include <adrcore/imgprocessing/channelprocessor.h>
#include <adrcore/imgprocessing/coronaindices.h>

#include <set>
#include <list>

namespace adore
{

	namespace core
	{

		template<class imgT>
		class MaxRegions
		{
			public:

				typedef uint16_t compoundT;

				class Compound
				{
					public:
						Compound( compoundT idx, imgT value ):val( value ), id( idx ), max( true )
						{
							;
						}
						imgT getVal() const
						{
							return val;
						}
						compoundT getId() const
						{
							return id;
						}
						ROI &getBox()
						{
							return boundingbox;
						}
						const ROI &getBox() const
						{
							return boundingbox;
						}

						void include( uint x, uint y )
						{
							boundingbox.include( x, y );
						}

						bool isMax() const
						{
							return max;
						}
						void setMax( const bool m )
						{
							max = m;
						}

					private:
						imgT val;
						compoundT id;
						ROI boundingbox;
						bool max;
				};

//				template<class imgT>
				class CompoundComparator
				{
					public:
						bool operator()( const Compound &a, const Compound &b ) const
						{
							bool ab = compVal( a.getVal(), b.getVal() );
							bool ba = compVal( b.getVal(), a.getVal() );
							if ( ab == ba )
							{
								return compID( a.getId(), b.getId() );
							}
							else
							{
								return ab;
							}
						}

					private:
						greater<imgT> compVal;
						greater<compoundT> compID;
				};


			public:
				typedef std::set<Compound, CompoundComparator> CCollT;
				typedef typename CCollT::iterator iterator;
				typedef typename CCollT::const_iterator const_iterator;

			private:
				CONNECTIVITY con;
				ImagePlane<uint16_t> marker;
				CCollT compounds;
				int numMax;

			public:

				MaxRegions( CONNECTIVITY c = EIGHT ):
						con( c ), numMax( 0 )
				{
				}

				/**
				 * Destructor
				 */
				~MaxRegions()
				{
					;
				}

				/**
				 */
				void process( Channel<imgT> *in )
				{
					typedef CoronaIndices::iterator CiT;
					CoronaIndices pixel2Corona( con ); //help at image boundary

					ROI roi = in->getROI();
					uint offX = roi.getX();
					uint offY = roi.getY();

					marker.init( roi.getWidth(), roi.getHeight(), 1 );
					marker.fill( 0 );

					uint x = 0, y = 0;
					compoundT compoundid = 1;//0 means not yet visited
					numMax = 0;
					compounds.clear();

					for ( typename Channel<imgT>::iterator it = in->getIteratorBegin();
					                it != in->getIteratorEnd(); ++it )
					{
						imgT *pix = it.getPointer();
						in->getCoors( pix, x, y );
//						printf( "x: %d, y: %d\n", x, y );

						if ( visited( x, y ) ) continue;

						Compound comp( compoundid, *pix );
						comp.setMax( true );

						list<imgT *> pixel;

						pixel.push_back( pix );
						mark( x-offX, y-offY, compoundid );
						comp.getBox().set( x, y, 1, 1 );

						while ( pixel.size() )
						{
							imgT *pix = pixel.front();
							imgT value = *pix;
//							printf( "value : %d\n", value );
							in->getCoors( pix, x, y );

							pixel2Corona.init( x, y, roi );
							for ( CiT it = pixel2Corona.begin(); it != pixel2Corona.end(); ++it )
							{
//								printf( "x: %d, y: %d, co: %d\n", x, y, *it );
								uint cx = x + CORONA[*it][0];
								uint cy = y - CORONA[*it][1];

								imgT *cand = in->getPointer( cx, cy );
//								printf( "candvalue : %d\n", *cand );
								if ( !visited( cx-offX, cy-offY ) && *cand == value )
								{
//									printf( "accepted\n" );
									pixel.push_back( cand );
									mark( cx-offX, cy-offY, compoundid );

//									printf( "x: %d, y: %d, width: %d, height: %d\n", comp.getBox().getX(), comp.getBox().getY(), comp.getBox().getWidth(), comp.getBox().getHeight() );
									comp.include( cx, cy );
//									printf( "cx: %d, cy: %d\n", cx, cy );

//									printf( "x: %d, y: %d, width: %d, height: %d\n", comp.getBox().getX(), comp.getBox().getY(), comp.getBox().getWidth(), comp.getBox().getHeight() );
//									pressKey();
								}
								else if ( *cand  > value )
								{
									comp.setMax( false );
								}
							}
//							pressKey();
							pixel.pop_front();
						}

//						cout << "adding compound" << endl;
						++compoundid;
						compounds.insert( comp );
						if ( comp.isMax() ) ++numMax;
//						cout << "compound added" << endl;
					}
//					marker.save( "maxregionsmarker.png" );
				}

				iterator begin()
				{
					return compounds.begin();
				}

				iterator end()
				{
					return compounds.end();
				}

				const_iterator begin() const
				{
					return compounds.begin();
				}

				const_iterator end() const
				{
					return compounds.end();
				}

				uint numMaxRegions()
				{
					return numMax;
				}

				const Compound getLargest() const
				{
					uint largestArea = 0;
					Compound largestCompound( -1, 0 );
					for ( const_iterator it = begin(); it != end(); ++it )
					{
						const Compound &c = *it;
						if ( c.isMax() )
						{
							uint area = c.getBox().getArea();
							if ( area > largestArea )
							{
								largestArea = area;
								largestCompound = c;
							}
						}
					}
					return largestCompound;
				}

			private:

				void mark( uint x, uint y, compoundT id )
				{
					marker( x, y ) = id;
				}

				bool visited( uint x, uint y )
				{
					return marker( x, y ) != 0;
				}

		};
	}
}//namespace


#endif
