

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGORIENTATIONANDMAGNITUDE_H
#define ADORE_IMGPROCESSINGORIENTATIONANDMAGNITUDE_H

#include <adrcore/imgprocessing/channel.h>
#include <adrcore/imgprocessing/dizenzocommon.h>

/**
 * The sectors are numerated as follows
 * 3 2 1
 * 0 X 0
 * 1 2 3
 */
namespace adore
{

	namespace core
	{

		/**
		 
		*/

		template<class gT, class magT>
		class Magnitude
		{
			public:
				Magnitude( int lowThresh = 0 ):low( lowThresh )
				{
					;
				}

				void setThresh( const magT &val )
				{
					low = val;
				}

				magT getThresh() const
				{
					return low;
				}

				/**
				 * calculates magnitude
				*/
				void m( const Channel<gT> * const gxx,
				        const Channel<gT> * const gyy,
				        const Channel<gT> * const gxy,
				        Channel<magT> * const mag ) const;

				/**
				 * calculates magnitude
				*/
				void m( const Channel<gT> * const gx,
				        const Channel<gT> * const gy,
				        Channel<magT> * const mag ) const;

				/**
				 * calculates magnitudeX/magnitudeY
				*/
				void mm( const Channel<gT> * const gxx,
				         const Channel<gT> * const gyy,
				         const Channel<gT> * const gxy,
				         Channel<magT> * const magX,
				         Channel<magT> * const magY ) const;
			protected:

				magT low;
		};

		template<class gT, class oT>
		class Orientation
		{
			public:

				typedef uint8_t secT;

				Orientation()
				{
					;
				}

				~Orientation()
				{
					;
				}

				/**
				 * calculates orientation
				*/
				void o( const Channel<gT> * const gxx,
				        const Channel<gT> * const gyy,
				        const Channel<gT> * const gxy,
				        Channel<oT> * const orient ) const;

				/**
				 * calculates sector
				*/
				void s( const Channel<gT> * const gxx,
				        const Channel<gT> * const gyy,
				        const Channel<gT> * const gxy,
				        Channel<secT> * const orient ) const;

			protected:

		};


		template<class gT, class oT, class magT>
		class OrientationAndMagnitude:public Magnitude<gT, magT>, public Orientation<gT, oT>
		{
			public:
				typedef typename Orientation<gT, oT>::secT secT;

				OrientationAndMagnitude( int lowThresh = 0 ):
						Magnitude<gT, magT>( lowThresh )
				{
					;
				}

				~OrientationAndMagnitude()
				{
					;
				}

				/**
				 * calculates orientation and magnitude
				 */
				void om( const Channel<gT> * const gxx,
				         const Channel<gT> * const gyy,
				         const Channel<gT> * const gxy,
				         Channel<oT> * const orient,
				         Channel<magT> * const mag ) const;

				/**
				 * calculates orientation and magnitudeX/magnitudeY
				 */
				void omm( const Channel<gT> * const gxx,
				          const Channel<gT> * const gyy,
				          const Channel<gT> * const gxy,
				          Channel<oT> * const orient,
				          Channel<magT> * const magX,
				          Channel<magT> * const magY ) const;

				/**
				 * calculates sector and magnitude
				 */
				void sm( const Channel<gT> * const gxx,
				         const Channel<gT> * const gyy,
				         const Channel<gT> * const gxy,
				         Channel<secT> * const orient,
				         Channel<magT> * const mag ) const;

				void sm( const Channel<gT> * const gx,
				         const Channel<gT> * const gy,
				         Channel<secT> * const orient,
				         Channel<magT> * const mag ) const;

				/**
				 * calculates sector and magnitudeX/magnitudeY
				 */
				void smm( const Channel<gT> * const gxx,
				          const Channel<gT> * const gyy,
				          const Channel<gT> * const gxy,
				          Channel<secT> * const orient,
				          Channel<magT> * const magX,
				          Channel<magT> * const magY ) const;

		};

		template <class gT, class oT, class magT>
		void OrientationAndMagnitude<gT, oT, magT>::om(
		        const Channel<gT> * const gxx,
		        const Channel<gT> * const gyy,
		        const Channel<gT> * const gxy,
		        Channel<oT> * const orient,
		        Channel<magT> * const mag ) const
		{
			const float halfPi = 0.5 * M_PI;

			typename Channel<gT>::Iterator2D itGxx = gxx->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGyy = gyy->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGxy = gxy->getIteratorBegin();

			typename Channel<oT>::Iterator2D itO = orient->getIteratorBegin();
			typename Channel<magT>::Iterator2D itM = mag->getIteratorBegin();

			for ( ; itGxx != gxx->getIteratorEnd(); ++itGxx, ++itGyy, ++itGxy, ++itO, ++itM )
			{
				gT &gxx = *itGxx;
				gT &gyy = *itGyy;
				gT &gxy = *itGxy;

				double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
				magT mag = magT( getMagnitude( gxx, gyy, gxy, winkelMal2 ) );

				if ( mag > this->getThresh() )
				{
					double orient = 0.5 * winkelMal2;
					if ( orient == -halfPi ) orient = halfPi;
					*itO = orient;

					*itM = mag;
				}
				else
				{
					*itO = 0;
					*itM = 0;
				}
			}
		}

		template <class gT, class oT, class magT>
		void OrientationAndMagnitude<gT, oT, magT>::omm(
		        const Channel<gT> * const gxx,
		        const Channel<gT> * const gyy,
		        const Channel<gT> * const gxy,
		        Channel<oT> * const orient,
		        Channel<magT> * const magX,
		        Channel<magT> * const magY ) const
		{
			const float halfPi = 0.5 * M_PI;

			typename Channel<gT>::Iterator2D itGxx = gxx->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGyy = gyy->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGxy = gxy->getIteratorBegin();

			typename Channel<oT>::Iterator2D itO = orient->getIteratorBegin();
			typename Channel<magT>::Iterator2D itMX = magX->getIteratorBegin();
			typename Channel<magT>::Iterator2D itMY = magY->getIteratorBegin();

			for ( ; itGxx != gxx->getIteratorEnd(); ++itGxx, ++itGyy, ++itGxy, ++itO, ++itMX, ++itMY )
			{
				gT &gxx = *itGxx;
				gT &gyy = *itGyy;
				gT &gxy = *itGxy;

				double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
				magT mag = magT( getMagnitude( gxx, gyy, gxy, winkelMal2 ) );

				if ( mag > this->getThresh() )
				{
					double orient = 0.5 * winkelMal2;
					if ( orient == -halfPi ) orient = halfPi;
					*itO = orient;

					*itMX = mag * cos( orient );
					*itMY = mag * sin( orient );
				}
				else
				{
					*itO = 0;
					*itMX = 0;
					*itMY = 0;
				}
			}
		}

		template <class gT, class oT, class magT>
		void OrientationAndMagnitude<gT, oT, magT>::sm(
		        const Channel<gT> * const gxx,
		        const Channel<gT> * const gyy,
		        const Channel<gT> * const gxy,
		        Channel<secT> * const sector,
		        Channel<magT> * const mag ) const
		{
			assert( gxx->getROI().getWidth() == sector->getROI().getWidth() );
			assert( mag->getROI().getWidth() == sector->getROI().getWidth() );

			typename Channel<gT>::Iterator2D itGxx = gxx->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGyy = gyy->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGxy = gxy->getIteratorBegin();

			typename Channel<secT>::Iterator2D itS = sector->getIteratorBegin();
			typename Channel<magT>::Iterator2D itM = mag->getIteratorBegin();

			for ( ; itGxx != gxx->getIteratorEnd(); ++itGxx, ++itGyy, ++itGxy, ++itS, ++itM )
			{
//				gxx->printCoors( itGxx.getPointer() );
//				gyy->printCoors( itGyy.getPointer() );
//				gxy->printCoors( itGxy.getPointer() );
//				sector->printCoors( itS.getPointer() );
//				mag->printCoors( itM.getPointer() );
//				pressKey();

				gT &gxx = *itGxx;
				gT &gyy = *itGyy;
				gT &gxy = *itGxy;

				double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
				magT mag = magT( getMagnitude( gxx, gyy, gxy, winkelMal2 ) );

				if ( mag > this->getThresh() )
				{
//					if ( fabs( winkelMal2 ) > 3 * M_PI / 4 ) *itS = 2;
//					else if ( winkelMal2 > M_PI / 4 ) *itS = 1;
//					else if ( winkelMal2 > -M_PI / 4 ) *itS = 0;
//					else *itS = 3;
					*itS = getSector( winkelMal2 );

//					printf( "sector is %d\n", *itS );

					*itM = mag;
				}
				else
				{
					*itS = 0;
					*itM = 0;
				}
			}
		}

		template <class gT, class oT, class magT>
		void OrientationAndMagnitude<gT, oT, magT>::sm(
		        const Channel<gT> * const gx,
		        const Channel<gT> * const gy,
		        Channel<secT> * const sector,
		        Channel<magT> * const mag ) const
		{
			assert( gx->getROI().getWidth() == mag->getROI().getWidth() );

			typename Channel<gT>::Iterator2D itGx = gx->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGy = gy->getIteratorBegin();

			typename Channel<secT>::Iterator2D itS = sector->getIteratorBegin();
			typename Channel<magT>::Iterator2D itM = mag->getIteratorBegin();

			for ( ; itGx != gx->getIteratorEnd(); ++itGx, ++itGy, ++itM, ++itS )
			{
				gT &gx = *itGx;
				gT &gy = *itGy;

				magT mag = magT( sqrtf( gx * gx + gy * gy ) );

				if ( mag > this->getThresh() )
				{
					*itM = mag;
					*itS = this->getSector( atan2f( float( gy ), float( gx ) ) );
				}
				else
				{
					*itM = 0;
					*itS = 0;
				}
			}
		}


		template <class gT, class oT, class magT>
		void OrientationAndMagnitude<gT, oT, magT>::smm(
		        const Channel<gT> * const gxx,
		        const Channel<gT> * const gyy,
		        const Channel<gT> * const gxy,
		        Channel<secT> * const sector,
		        Channel<magT> * const magX,
		        Channel<magT> * const magY ) const
		{
			const float halfPi = 0.5 * M_PI;

			typename Channel<gT>::Iterator2D itGxx = gxx->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGyy = gyy->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGxy = gxy->getIteratorBegin();

			typename Channel<secT>::Iterator2D itS = sector->getIteratorBegin();
			typename Channel<magT>::Iterator2D itMX = magX->getIteratorBegin();
			typename Channel<magT>::Iterator2D itMY = magY->getIteratorBegin();

			for ( ; itGxx != gxx->getIteratorEnd(); ++itGxx, ++itGyy, ++itGxy, ++itS, ++itMX, ++itMY )
			{
				gT &gxx = *itGxx;
				gT &gyy = *itGyy;
				gT &gxy = *itGxy;

				double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
				magT mag = magT( getMagnitude( gxx, gyy, gxy, winkelMal2 ) );

				if ( mag > this->getThresh() )
				{
//					if ( fabs( winkelMal2 ) > 3 * M_PI / 4 ) *itS = 2;
//					else if ( winkelMal2 > M_PI / 4 ) *itS = 3;
//					else if ( winkelMal2 > -M_PI / 4 ) *itS = 0;
//					else *itS = 1;

					*itS = this->getSector( winkelMal2 );

					double orient = 0.5 * winkelMal2;
					if ( orient == -halfPi ) orient = halfPi;

					*itMX = mag * cos( orient );
					*itMY = mag * sin( orient );
				}
				else
				{
					*itS = 0;
					*itMX = 0;
					*itMY = 0;
				}
			}
		}


		template <class gT, class magT>
		void Magnitude<gT, magT>::m(
		        const Channel<gT> * const gxx,
		        const Channel<gT> * const gyy,
		        const Channel<gT> * const gxy,
		        Channel<magT> * const mag ) const
		{
			assert( gxx->getROI().getWidth() == mag->getROI().getWidth() );

			typename Channel<gT>::Iterator2D itGxx = gxx->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGyy = gyy->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGxy = gxy->getIteratorBegin();

			typename Channel<magT>::Iterator2D itM = mag->getIteratorBegin();

			for ( ; itGxx != gxx->getIteratorEnd(); ++itGxx, ++itGyy, ++itGxy, ++itM )
			{
				gT &gxx = *itGxx;
				gT &gyy = *itGyy;
				gT &gxy = *itGxy;

				magT mag = magT( getMagnitude( gxx, gyy, gxy ) );

				if ( mag > low )
				{
					*itM = mag;
				}
				else
				{
					*itM = 0;
				}
			}
		}

		template <class gT, class magT>
		void Magnitude<gT, magT>::m(
		        const Channel<gT> * const gx,
		        const Channel<gT> * const gy,
		        Channel<magT> * const mag ) const
		{
			assert( gx->getROI().getWidth() == mag->getROI().getWidth() );

			typename Channel<gT>::Iterator2D itGx = gx->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGy = gy->getIteratorBegin();

			typename Channel<magT>::Iterator2D itM = mag->getIteratorBegin();

			for ( ; itGx != gx->getIteratorEnd(); ++itGx, ++itGy, ++itM )
			{
				gT &gx = *itGx;
				gT &gy = *itGy;

				magT mag = magT( sqrtf( gx * gx + gy * gy ) );

				if ( mag > low )
				{
					*itM = mag;
				}
				else
				{
					*itM = 0;
				}
			}
		}


		template <class gT, class magT>
		void Magnitude<gT, magT>::mm(
		        const Channel<gT> * const gxx,
		        const Channel<gT> * const gyy,
		        const Channel<gT> * const gxy,
		        Channel<magT> * const magX,
		        Channel<magT> * const magY ) const
		{
			const float halfPi = 0.5 * M_PI;

			typename Channel<gT>::Iterator2D itGxx = gxx->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGyy = gyy->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGxy = gxy->getIteratorBegin();

			typename Channel<magT>::Iterator2D itMX = magX->getIteratorBegin();
			typename Channel<magT>::Iterator2D itMY = magY->getIteratorBegin();

			for ( ; itGxx != gxx->getIteratorEnd(); ++itGxx, ++itGyy, ++itGxy, ++itMX, ++itMY )
			{
				gT &gxx = *itGxx;
				gT &gyy = *itGyy;
				gT &gxy = *itGxy;

				float winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
				magT mag = magT( getMagnitude( gxx, gyy, gxy, winkelMal2 ) );

				if ( mag > low )
				{
					float orient = 0.5 * winkelMal2;
					if ( orient == -halfPi ) orient = halfPi;

					*itMX = magT( mag * cos( orient ) );
					*itMY = magT( mag * sin( orient ) );
				}
				else
				{
					*itMX = 0;
					*itMY = 0;
				}
			}
		}

		template <class gT, class oT>
		void Orientation<gT, oT>::s(
		        const Channel<gT> * const gxx,
		        const Channel<gT> * const gyy,
		        const Channel<gT> * const gxy,
		        Channel<secT> * const sector ) const
		{
			assert( gxx->getROI().getWidth() == sector->getROI().getWidth() );

			typename Channel<gT>::Iterator2D itGxx = gxx->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGyy = gyy->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGxy = gxy->getIteratorBegin();

			typename Channel<secT>::Iterator2D itS = sector->getIteratorBegin();

			for ( ; itGxx != gxx->getIteratorEnd(); ++itGxx, ++itGyy, ++itGxy, ++itS )
			{
				gT &gxx = *itGxx;
				gT &gyy = *itGyy;
				gT &gxy = *itGxy;

				double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
				*itS = getSector( winkelMal2 );
			}
		}

		template <class gT, class oT>
		void Orientation<gT, oT>::o(
		        const Channel<gT> * const gxx,
		        const Channel<gT> * const gyy,
		        const Channel<gT> * const gxy,
		        Channel<oT> * const orient ) const
		{
			typename Channel<gT>::Iterator2D itGxx = gxx->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGyy = gyy->getIteratorBegin();
			typename Channel<gT>::Iterator2D itGxy = gxy->getIteratorBegin();

			typename Channel<oT>::Iterator2D itO = orient->getIteratorBegin();

			for ( ; itGxx != gxx->getIteratorEnd(); ++itGxx, ++itGyy, ++itGxy, ++itO )
			{
				gT &gxx = *itGxx;
				gT &gyy = *itGyy;
				gT &gxy = *itGxy;

				double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
				double orient = 0.5 * winkelMal2;
				if ( orient == -halfPI ) orient = halfPI;
				*itO = orient;
			}
		}

	}
}

#endif
