

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGRGBTOSCALAROPERATOR_H
#define ADORE_IMGPROCESSINGRGBTOSCALAROPERATOR_H

#include <adrcore/imgprocessing/channelprocessor.h>
#include <adrcore/imgprocessing/channel.h>
#include <adrcore/imgprocessing/imageplane.h>
#include <adrcore/util/calctype.h>

namespace adore
{

	namespace core
	{
		
		inline float tunedDiv(float pro, float contra, float intens)
		{
			float divisor = max( 0.0F, float( contra - intens ));
			if(divisor < FLT_EPSILON)//no fabs, indeed!
			{
				divisor = 1.0F;
			}
			float dividend = float( pro - intens );
			return dividend / divisor;
		}

		template<class _typeIn, class _typeOut> class RGB2GrayOp
		{
			public:
				typedef _typeIn typeIn;
				typedef _typeOut typeOut;

				typeOut operator()( typeIn r, typeIn g, typeIn b )
				{
					return typeOut( 0.2125F*r+0.7154F*g+0.0721F*b );
				}
		};

		template<class _typeIn, class _typeOut> class RGB2IntensityOp
		{
			public:
				typedef _typeIn typeIn;
				typedef _typeOut typeOut;
				typedef typename calcT<typename signedT<typeIn>::type>::type calcT;

				typeOut operator()( calcT r, calcT g, calcT b )
				{
					return typeOut( float( r+g+b ) / 3.0F );
				}
		};

		template<class _typeIn, class _typeOut> class RGB2TunedRedOp
		{
			public:
				typedef _typeIn typeIn;
				typedef _typeOut typeOut;
				typedef typename calcT<typename signedT<typeIn>::type>::type calcT;

				typeOut operator()( calcT r, calcT g, calcT b )
				{
					return typeOut( max( 0.0F, float( r ) - 0.5F * ( g + b ) ) );
				}
		};
		
		template<class _typeIn, class _typeOut> class RGB2TunedRedSubGreenOp
		{
			public:
				typedef _typeIn typeIn;
				typedef _typeOut typeOut;
				typedef typename calcT<typename signedT<typeIn>::type>::type calcT;

				typeOut operator()( calcT r, calcT g, calcT b )
				{
					return typeOut( max( 0.0F, float( r - g ) ));
				}
		};
		
		template<class _typeIn, class _typeOut> class RGB2TunedRedDivGreenOp
		{
			public:
				typedef _typeIn typeIn;
				typedef _typeOut typeOut;
				typedef typename calcT<typename signedT<typeIn>::type>::type calcT;

				typeOut operator()( calcT r, calcT g, calcT b )
				{
					return tunedDiv(r, g, b);
				}
		};

		template<class _typeIn, class _typeOut> class RGB2TunedGreenOp
		{
			public:
				typedef _typeIn typeIn;
				typedef _typeOut typeOut;
				typedef typename calcT<typename signedT<typeIn>::type>::type calcT;

				typeOut operator()( calcT r, calcT g, calcT b )
				{
					return typeOut( max( 0.0F, float( g ) - 0.5F * ( r + b ) ) );
				}
		};
		
		template<class _typeIn, class _typeOut> class RGB2TunedGreenSubRedOp
		{
			public:
				typedef _typeIn typeIn;
				typedef _typeOut typeOut;
				typedef typename calcT<typename signedT<typeIn>::type>::type calcT;

				typeOut operator()( calcT r, calcT g, calcT b )
				{
					return typeOut( max( 0.0F, float( g - r ) ));
				}
		};
		
		template<class _typeIn, class _typeOut> class RGB2TunedGreenDivRedOp
		{
			public:
				typedef _typeIn typeIn;
				typedef _typeOut typeOut;
				typedef typename calcT<typename signedT<typeIn>::type>::type calcT;

				typeOut operator()( calcT r, calcT g, calcT b )
				{
					return tunedDiv(g, r, b);
				}
		};


		template<class _typeIn, class _typeOut> class RGB2TunedBlueOp
		{
			public:
				typedef _typeIn typeIn;
				typedef _typeOut typeOut;
				typedef typename calcT<typename signedT<typeIn>::type>::type calcT;

				typeOut operator()( calcT r, calcT g, calcT b )
				{
					return typeOut( max( 0.0F, float( b ) - 0.5F * ( g + r ) ) );
				}
		};

		template<class _typeIn, class _typeOut> class RGB2TunedYellowOp
		{
			public:
				typedef _typeIn typeIn;
				typedef _typeOut typeOut;
				typedef typename calcT<typename signedT<typeIn>::type>::type calcT;

				typeOut operator()( calcT r, calcT g, calcT b )
				{
					return typeOut( max( 0.0F, 0.5F * float( ( r + g ) - tabs<calcT>( r - g ) ) - b ) );
				}
		};

		template<class Converter>
		class RGBtoScalar : public ChannelProcessor< 3, 1, typename Converter::typeIn, typename Converter::typeOut >
		{
				typedef typename Converter::typeIn typeIn;
				typedef typename Converter::typeOut typeOut;

			public:
				void process( typename Channel<typeIn>::Set &in, Channel<typeOut> *out )
				{
					assert( in.size() == 3 );
					Image2DIterator<typeOut> outit = out->getIteratorBegin();
					Image2DIterator<typeIn> inrit = in[0]->getIteratorBegin();
					Image2DIterator<typeIn> ingit = in[1]->getIteratorBegin();
					Image2DIterator<typeIn> inbit = in[2]->getIteratorBegin();

					for ( ;outit!=out->getIteratorEnd();++outit,++inrit,++ingit,++inbit )
					{
						*outit = convert( *inrit, *ingit, *inbit );
					};
				}

				void process( ImagePlane<typeIn> *img, Channel<typeOut> *out )
				{
					ChannelProcessor< 3, 1, typeIn, typeOut >::process( img, out );
				}

				/**
					in case you need to modify the converter
				*/
				Converter &getConverter()
				{
					return convert;
				}

			private:
				Converter convert;
		};
	}
}//ns
#endif
