

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGTYPECONVERTER_H
#define ADORE_IMGPROCESSINGTYPECONVERTER_H

#include <adrcore/imgprocessing/channelprocessor.h>
#include <adrcore/imgprocessing/channelfunctions.h>

namespace adore
{

	namespace core
	{

		/**
		 
		*/
		template<class typeIn, class typeOut>
		class TypeConverter : public ChannelProcessor< 1, 1, typeIn, typeOut >
		{
			public:
				TypeConverter():ChannelProcessor< 1, 1, typeIn, typeOut >()
				{
					;
				}

				~TypeConverter()
				{
					;
				}

				void process( Channel<typeIn> *in, Channel<typeOut> *out )
				{
					error( "type conversion not yet supported\n" );
				}

			private:

				template<class T>
				void processIntUchar( Channel<T> *in, Channel<uint8_t> *out )
				{
					T min, max, *minL, *maxL;

					minMaxLoc( *in, min, max, minL, maxL );
					if ( max == min )
					{
						out->fill( 0 );
						return;
					}

					int delta = max - min;

					typename Channel<T>::Iterator2D srcIt = in->getIteratorBegin();
					typename Channel<uint8_t>::Iterator2D dstIt = out->getIteratorBegin();
					typename Channel<T>::Iterator2D srcEnd = in->getIteratorEnd();
//					typename Channel<uint8_t>::Iterator2D dstEnd = out->getIteratorEnd();
					for ( ; srcIt != srcEnd; ++srcIt, ++dstIt )
					{
						*dstIt = uint8_t( int( *srcIt - min ) * 255.0F / delta );
					}
				}

		};

		template<> inline void TypeConverter<uint8_t, float>::process( Channel<uint8_t> *in, Channel<float> *out )
		{
			Channel<uint8_t>::Iterator2D srcIt = in->getIteratorBegin();
			Channel<float>::Iterator2D dstIt = out->getIteratorBegin();
			Channel<uint8_t>::Iterator2D srcEnd = in->getIteratorEnd();
//			Channel<float>::Iterator2D dstEnd = out->getIteratorEnd();
			for ( ; srcIt != srcEnd; ++srcIt, ++dstIt )
			{
				*dstIt = float( *srcIt ) / 255.0f;
			}
		}

		template<> inline void TypeConverter<float, uint8_t>::process( Channel<float> *in, Channel<uint8_t> *out )
		{
			Channel<float>::Iterator2D srcIt = in->getIteratorBegin();
			Channel<uint8_t>::Iterator2D dstIt = out->getIteratorBegin();
			Channel<float>::Iterator2D srcEnd = in->getIteratorEnd();
//			Channel<uint8_t>::Iterator2D dstEnd = out->getIteratorEnd();
			for ( ; srcIt != srcEnd; ++srcIt, ++dstIt )
			{
				*dstIt = lrintf( *srcIt * 255.0f );
			}
		}

		template<> inline void TypeConverter<int8_t, uint8_t>::process( Channel<int8_t> *in, Channel<uint8_t> *out )
		{
			processIntUchar( in, out );
		}

		template<> inline void TypeConverter<int16_t, uint8_t>::process( Channel<int16_t> *in, Channel<uint8_t> *out )
		{
			processIntUchar( in, out );
		}

		template<> inline void TypeConverter<uint16_t, uint8_t>::process( Channel<uint16_t> *in, Channel<uint8_t> *out )
		{
			processIntUchar( in, out );
		}

		template<> inline void TypeConverter<int32_t, uint8_t>::process( Channel<int32_t> *in, Channel<uint8_t> *out )
		{
			processIntUchar( in, out );
		}

		template<> inline void TypeConverter<uint32_t, uint8_t>::process( Channel<uint32_t> *in, Channel<uint8_t> *out )
		{
			processIntUchar( in, out );
		}

	}
}

#endif
