

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming
/// Author: Sergey Cheremukhin


#ifndef ADORE_ERRORFUNCTORS3D_H
#define ADORE_ERRORFUNCTORS3D_H

#include <adrcore/types/aarray.h>
#include <adrcore/util/matmul2.h>
#include <adrcore/util/printmat.h>

#include <cfloat>
#include <cassert>
#include <algorithm>
#include <vector>
using std::vector;

#include <iostream>
using std::cout;
using std::endl;

namespace adore
{
	namespace core
	{
		/**
		 *  
		 *  
		 */
		class CPErrorsClassic2
		{
			AVector testpt;
			public:

				CPErrorsClassic2() : testpt(4) {}

				double operator() ( const AMatrix& H, double threshold, vector<int> & candInliers, const vector<AVector*>& pts1, const vector<AVector*>& pts2 )
				{
					assert(pts1.size() == pts2.size());

					double err = 0.0;

					for ( int i = 0; i < (int)pts1.size(); ++i )
					{
						AVector * p1 = pts1[i];
						AVector * p2 = pts2[i];
						matMul2(H, *p2, testpt);

						double dist = 0;
						for(int j = 0; j < 3; ++j)
						{
							dist +=intPow((*p1)(j) * testpt(3) - testpt(j) * (*p1)(3), 2);
						}
						if(dist < threshold)
						{
							candInliers.push_back(i);
							err += dist;
						} else
						{
							err += threshold;
						}
					}
					return err;
				}
		};

		class CPErrorsSimpleDist
		{
                        AVector temp1, temp2;
			public:

                CPErrorsSimpleDist() : temp1(4), temp2(4) {}

                double operator() ( const AMatrix& H, double threshold, vector<int> & candInliers, const vector<std::pair<AVector*, AVector*> >& pts )
				{
                    double err = 0.0;

                    for ( int i = 0; i < (int)pts.size(); ++i )
                    {
                        const AVector & p1 = *(pts[i].first);
                        const AVector & p2 = *(pts[i].second);
                        assert(p1(3) == 1.0);
                        assert(p2(3) == 1.0);

                        matMul2(H, p2, temp1);

                        temp2 = p1 - temp1;
                        double dist = sqrt(dot(temp2, temp2));
                        if(dist < threshold)
                        {
                            candInliers.push_back(i);
                            err += dist;
                        } else
                        {
                            err += threshold;
                        }
                    }
                    return err;
				}
		};


		class CPErrorsClassic
		{
			AVector testpt;
			public:

				CPErrorsClassic() : testpt(4) {}

				double operator() ( const AMatrix& H, double threshold, vector<int> & candInliers, const vector<AVector*>& pts1, const vector<AVector*>& pts2 )
				{
					assert(pts1.size() == pts2.size());

					double err = 0.0;

					vector<double> medDist;
					medDist.reserve( pts1.size() );

					vector<int> prefiltered;
					prefiltered.reserve( pts1.size() );

					for ( int i = 0; i < (int)pts1.size(); ++i )
					{
						AVector * p = pts2[i];

						double w1 = ( *p )( 3 );
						if ( fabs( w1 ) < 8 * FLT_EPSILON )
						{
							cout << "CPErrorsH: fabs(w1) < 8 * FLT_EPSILON" << endl;
							continue;
						}

						matMul2( H, *p, testpt );

						double w2 = -testpt( 3 );
						if ( fabs( w2 ) < 8 * FLT_EPSILON )
						{
							cout << "RANSACClientMergerCP: fabs(w2) < 8 * FLT_EPSILON; H - pt2 | pt22:" << endl;
							printMat( H );
							printMats( *p, testpt );
							cout << endl;
							continue;
						}

						double x1 = ( *p )( 0 ) / w1;
						double y1 = ( *p )( 1 ) / w1;
						double z1 = ( *p )( 2 ) / w1;
						double x2 = testpt( 0 ) / w2;
						double y2 = testpt( 1 ) / w2;
						double z2 = testpt( 2 ) / w2;
						double dist = ( x1 - x2 ) * ( x1 - x2 ) + ( y1 - y2 ) * ( y1 - y2 ) + ( z1 - z2 ) * ( z1 - z2 );

						prefiltered.push_back( i );
						medDist.push_back( dist );
					}

					if ( prefiltered.size() > 0 )
					{
						//median berechnen
						double median = 0.0;
						std::sort( medDist.begin(), medDist.end() );
						uint half = prefiltered.size() >> 1;
						if ( half << 1 == prefiltered.size() )
						{
							//gerade
							median = 0.5 *( medDist[half] + medDist[half - 1] );
						}
						else
						{
							median = medDist[half];
						}

						//all distances inside (1.0 + fractionThresh) * median mark inliers
						double thresh = median * ( 1.0 + threshold );
						for ( uint i = 0; i < prefiltered.size(); ++i )
						{
							if ( medDist[i] > thresh )
							{
								err += thresh;
							}
							else {
								err += medDist[i];
								candInliers.push_back(prefiltered[i]);
							}
						}
					}

					return err;
				}
		};

	}
}

#endif // ADORE_ERRORFUNCTORS3D_H
