

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef CORE_FILTERUTIL_H
#define CORE_FILTERUTIL_H

#include <adrcore/types/nview.h>
#include <adrcore/random/ransacparams.h>
#include <adrcore/mestimate/filterransac.h>
#include <adrcore/viewmatch/viewknitter.h>
#include <adrcore/random/ransac2.h>
#include <adrcore/random/prosac.h>

namespace adore
{

	namespace core
	{

//		typedef RANSAC2 ADRSAC;
		typedef PROSAC ADRSAC;

		inline float getThreshFactor() {return 2.0F;}

		inline float getOptThreshFM()
		{
			return 1.5;
		}

		inline float getOptThreshRDFM()
		{
			return 2.0;
		}

		inline float getOptThreshTT()
		{
//			return 12.0;
			return 8.0;
		}

		inline NVIRANSACClient *getDefaultFilter( int numViews, ViewBase *v = NULL )
		{
			float thresh = 0;
			switch ( numViews )
			{
				case 2:
					thresh = getNormalizedPixel( getOptThreshFM(), v );
					return new FilterFMat( thresh );
					break;
				case 3:
					thresh = getNormalizedPixel( getOptThreshTT(), v );
					return new FilterTTensor( thresh );
					break;
				default:
					return NULL;
					break;
			}
			return NULL;
		}

		/** gric BEGIN */
		NVIRANSACClient * assessRansacClients( NView &nview, const ClientList& data );
		void filter( NView &nview, NVIRANSACClient *ranser );
		/** gric END */

		/**
		 * applies an _initialized_ NVIRANSACClient.
		 * Errorneous correspondences are removed form NView
		 * The NViewInfo ist _not_ set
		 */
		void filter( NView &nview, NVIRANSACClient *ranser, const RansacParams &rp );

		/**
		 * applies a _not_ initialized NVIRANSACClient.
		 * The client get initialized using the views provided by the knitter
		 * Errorneous correspondences are removed form NView
		 * The NViewInfo ist _not_ set
		 * This one is currently for a tuple size of 2 only.
		 */
		void filter( NView &nview, NVIRANSACClient *ranser, const RansacParams &rp, ViewKnitter &knitter );

		/**
			deletes all Points from a View v that are not member in any coor of nv
		*/
		void filterNonCorr( ViewBase *v, NView *nv );

		/**
			deltes all features from the views that are not compatible with
			an estimated epipolar geometry
			returns true on success;
		*/
		bool filter2KDTreeRansac( ViewBase *lView, ViewBase *rView, const RansacParams rp, int countThresh = INT_MAX );

		bool filter2EpiAlone( AMatrix FMatrix, ViewBase *lView, ViewBase *rView, float distThresh = 2.0, int countThresh = INT_MAX );

		/**
		 * create Korrespondences using KDTreeMatcher and then dismiss all corrs not agreeeing to epi geom
		 */
		bool filter2KDTreeEpi( AMatrix FMatrix, ViewBase *lView, ViewBase *rView, float distThresh = 2.0, int countThresh = INT_MAX );

	}
}
#endif

