

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_MINIMIZATIONCOVARSQUARER_H
#define ADORE_MINIMIZATIONCOVARSQUARER_H

#include <adrcore/types/aarray.h>
#include <adrcore/minimization/lmconstants.h>

namespace adore
{

	namespace core
	{

		/**
			 
		*/
		template <class InvCovarT>
		class CovarSquarer
		{
			typedef typename InvCovarT::scalar_type number_t;

			public:

				const AArray<number_t, 2> covarProduct( const AArray<number_t, 2> &A,
				                                        const InvCovarT &invCovar ) const
				{
					return covarProduct( A, invCovar, A );
				}

				const AArray<number_t, 2> covarProduct( const AArray<number_t, 2> &A,
				                                        const InvCovarT &invCovar,
				                                        const AArray<number_t, 2> &B ) const
				{
					ADORE_UNUSED( invCovar );

//					std::cout << "Covar Squarer: covar not used" << endl;

					AMatrix result(A.cols(), B.cols());
					matMul2_AT( A, B, result );
					return result;
				}

				const AArray<number_t, 1> covarProduct( const AArray<number_t, 2> &A,
				                                        const InvCovarT &invCovar,
				                                        const AArray<number_t, 1> &B ) const
				{
					ADORE_UNUSED( invCovar );

					AVector result(B.rows());
					matMul2_AT( A, B, result );
					return result;
				}
		};

		template <class number_t>
		class CovarSquarer<AArray<number_t, 2> >
		{
			public:

				const AArray<number_t, 2> covarProduct( const AArray<number_t, 2> &A,
				                                        const AArray<number_t, 2> &invCovar ) const
				{
					return covarProduct( A, invCovar, A );
				}

				const AArray<number_t, 2> covarProduct( const AArray<number_t, 2> &A,
				                                        const AArray<number_t, 2> &invCovar,
				                                        const AArray<number_t, 2> &B ) const
				{
					AArray<number_t, 2> covar = matMul( invCovar, B );
					return matMul_AT( A, covar );
				}

				const AArray<number_t, 1> covarProduct( const AArray<number_t, 2> &A,
				                                        const AArray<number_t, 2> &invCovar,
				                                        const AArray<number_t, 1> &B ) const
				{
					AArray<number_t, 1> covar = matMul( invCovar, B );
					return matMul_AT( A, covar );
				}
		};

		template <class number_t>
		class CovarSquarer<AArray<number_t, 1> >
		{
			public:
				const AArray<number_t, 2> covarProduct( const AArray<number_t, 2> &A,
				                                        const AArray<number_t, 1> &invCovar ) const
				{
					return covarProduct( A, invCovar, A );
				}

				const AArray<number_t, 2> covarProduct( const AArray<number_t, 2> &A,
				                                        const AArray<number_t, 1> &invCovar,
				                                        const AArray<number_t, 2> &B ) const
				{
					AArray<number_t, 2> covar( B.rows(), invCovar.cols() );
					covar = B;
					//multiply diagonal elements
					for ( int row = 0; row < invCovar.rows(); ++row )
						covar( row, row ) = covar( row, row ) * invCovar( row );

					return matMul_AT( A, covar );
				}

				const AArray<number_t, 1> covarProduct( const AArray<number_t, 2> &A,
				                                        const AArray<number_t, 1> &invCovar,
				                                        const AArray<number_t, 1> &B ) const
				{
					AArray<number_t, 1> covar( B.rows() );
					covar = B;
					//multiply diagonal elements
					for ( int row = 0; row < invCovar.rows(); ++row )
						covar( row ) = covar( row ) * invCovar( row );

					return matMul_AT( A, covar );
				}
		};

	}

}

#endif
