

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_MINIMIZATIONSPARSEDEFAULTERRORFUNC_H
#define ADORE_MINIMIZATIONSPARSEDEFAULTERRORFUNC_H

#include <adrcore/types/aarray.h>
#include <adrcore/util/printmat.h>
#include <adrcore/minimization/attenuation.h>
#include <vector>

namespace adore
{

	namespace core
	{

		using namespace std;

		/**
			 
		*/
		class SparseDefaultErrorFunc
		{
			public:
				SparseDefaultErrorFunc( vector<AVector> &goal ):g( goal )
				{
				}

				void operator()( vector<AVector> &val, vector<AVector> &err ) const
				{
					assert( val.size() == err.size() );
					for ( uint i = 0; i < val.size(); ++i )
					{
						if ( isNullArray( g[i] ) )
						{
							err[i] = 0;
						}
						else
						{
							err[i] = g[i] - val[i];
						}
					}
				}

			private:
				vector<AVector> &g;
		};

		class SparseZeroErrorFunc
		{
			public:
				SparseZeroErrorFunc()
				{
				}

				void operator()( vector<AVector> &val, vector<AVector> &err ) const
				{
					for ( uint i = 0; i < val.size(); ++i ) err[i] = val[i];
				}

			private:

		};

		class SparseHuberVectorErrorFunc
		{
			public:
				SparseHuberVectorErrorFunc( vector<AVector> &goal, double attThresh ):g( goal ),thresh( attThresh )
				{
				}

				void operator()( vector<AVector> &val, vector<AVector> &err ) const
				{
					assert( val.size() == err.size() );
					assert( val.size() == g.size() );

					for ( uint i = 0; i < val.size(); ++i )
					{
						if ( isNullArray( g[i] ) )
						{
							err[i] = 0;
						}
						else
						{
							err[i] = g[i] - val[i];
							huberVector( err[i], thresh );
						}
					}
				}

			private:
				vector<AVector> &g;
				double thresh;
		};

		class SparseHuberVectorErrorFuncSimple
		{
			public:
				SparseHuberVectorErrorFuncSimple( vector<AVector> &goal, double attThresh ):g( goal ),thresh( attThresh )
				{
				}

				void operator()( vector<AVector> &val, vector<AVector> &err, vector<AVector> &cam, vector<AVector> &pt ) const
				{
					assert( val.size() == err.size() );
					assert( val.size() == g.size() );

					uint i = 0;
					for ( uint b = 0; b < pt.size(); ++b )
					{
						for ( uint a = 0; a < cam.size(); ++a, ++i )
						{
							if ( isNullArray( g[i] ) )
							{
								err[i] = 0;
							}
							else
							{
								err[i] = g[i] - val[i];
								huberVector( err[i], thresh );
//								double depthpenalty = 0.00001 * depth( cam[a], pt[b] );//BULLSHIT
//								printf( "depthpenalty: %f\n", depthpenalty );
//								err[i] += depthpenalty;
							}
						}
					}
				}

				double depth( const AVector &cam, const AVector &pt3D ) const
				{
					return fabs( 1.0f/( pt3D( 2 ) * sqrt( cam( 2 ) * cam( 2 ) + cam( 5 ) * cam( 5 ) + cam( 8 ) * cam( 8 ) ) ) );
				}

			private:
				vector<AVector> &g;
				double thresh;
		};

		class SparseBlakeZissVectorErrorFunc
		{
			public:
				SparseBlakeZissVectorErrorFunc( vector<AVector> &goal, double attThresh ) : g( goal ),thresh( attThresh )
				{
				}

				void operator()( vector<AVector> &val, vector<AVector> &err ) const
				{
					assert( val.size() == err.size() );
					assert( val.size() == g.size() );

					for ( uint i = 0; i < val.size(); ++i )
					{
						if ( isNullArray( g[i] ) )
						{
							err[i] = 0;
						}
						else
						{
							err[i] = g[i] - val[i];
							blakeZissVector( err[i], thresh );
						}
					}
				}

			private:
				vector<AVector> & g;
				double thresh;
		};

	}

}

#endif
