

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_AUTOCALIBRATIONAUTOCALIBRATIONMINFUNC2_H
#define ADORE_AUTOCALIBRATIONAUTOCALIBRATIONMINFUNC2_H

#include <vector>
#include <adrcore/types/camera.h>
#include <adrcore/util/mathutil.h>
#include <adrcore/util/printmat.h>

namespace adore
{

	namespace core
	{

		using namespace std;

		/**
		 
		*/
		template< class CalibrationParamT >
		class AutocalibrationMinFunc2
		{
			public:
				typedef double T_numtype;
			public:
				AutocalibrationMinFunc2( CalibrationParamT* p = NULL )
						: Q( 4, 4 ), tempMat1( 4, 3 ), tempMat2( 3, 3 ), c( p )
				{ }


				~AutocalibrationMinFunc2()
				{
					;
				}

				void setCameras( NView::CameraIterator camIt, const int numCams )
				{
					cams.resize( numCams );
					for ( int i = 0; i < numCams; ++i, ++camIt ) cams[i] = *camIt;
					resizeWMatrices( numCams );
				}

				void operator()( const AVector &quadric, AVector &err ) const;

				void setCalibrationParam( CalibrationParamT* theValue )
				{
					c = theValue;
//					if ( cams ) setDimensions( c->getNumParams(), 1, 9 * cams->size() + c->getNumSoftConstraints(), 1 );
				}

				CalibrationParamT* getCalibrationParam() const
				{
					return c;
				}

				int getRangeDim() const
				{
					assert( c );
					if ( c ) return 9 * cams.size() + c->getNumSoftConstraints();
					return -1;
				}

			private:
				vector<Camera *> cams;
				mutable AMatrix Q;
				mutable AMatrix tempMat1;
				mutable AMatrix tempMat2;

				CalibrationParamT *c;
				mutable vector<AMatrix> Ws;

				void resizeWMatrices( uint num );
		};

		template< class CalibrationParamT >
		void AutocalibrationMinFunc2< CalibrationParamT >::operator()( const AVector &quadric, AVector &err ) const
		{
			assert( c );

			c->calculateQuadricAndConics( quadric, 0, Q, Ws );


			/*    printf("values in MinFunc:\n");
			    printf("Q:\n");
			    printMat(Q);
			    for(uint i = 0; i < Ws.size(); ++i)
			    {
			        printf("W[%d]\n", i);
			        printMat(*Ws[i]);
			    }*/

			err = 0;

//			pressKey();

			int errRow = 0;
			for ( uint i = 0; i < cams.size(); ++i )
			{
				AMatrix &cam = cams[i]->getProjMat();

				matMul2_BT( Q, cam, tempMat1 );
				matMul2( cam, tempMat1, tempMat2 );

//				printf( "tmpMat2\n" );
//				printMat( tmpMat2 );
				normalize( tempMat2, 1.0 );
//				printf( "tmpMat2 - n\n" );
//				printMat( tmpMat2 );

				AMatrix &w = Ws[i];
//				printf( "w[%d]:\n", i );
//				printMat( w );
				normalize( w, 1.0 );
//				printf( "w[%d] - n:\n", i );
//				printMat( w );

				err( errRow++ ) = w( 0, 0 ) - tempMat2( 0, 0 );
				err( errRow++ ) = w( 0, 1 ) - tempMat2( 0, 1 );
				err( errRow++ ) = w( 0, 2 ) - tempMat2( 0, 2 );
				err( errRow++ ) = w( 1, 0 ) - tempMat2( 1, 0 );
				err( errRow++ ) = w( 1, 1 ) - tempMat2( 1, 1 );
				err( errRow++ ) = w( 1, 2 ) - tempMat2( 1, 2 );
				err( errRow++ ) = w( 2, 0 ) - tempMat2( 2, 0 );
				err( errRow++ ) = w( 2, 1 ) - tempMat2( 2, 1 );
				err( errRow++ ) = w( 2, 2 ) - tempMat2( 2, 2 );
			}

//			pressKey();

			c->evaluateSoftConstraints( err, errRow++ );

//			pressKey();
		}


		template< class CalibrationParamT >
		void AutocalibrationMinFunc2< CalibrationParamT >::resizeWMatrices( uint num )
		{
			if ( num > 0 && Ws.size() != num )
			{
				Ws.resize( num );
				for ( uint i = 0; i < num; ++i )
				{
					Ws[i].resize( 3, 3 );
				}
			}
		}


	}

}

#endif
