

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_AUTOCALIBRATIONAUTOCALIBRATORLINEAR_H
#define ADORE_AUTOCALIBRATIONAUTOCALIBRATORLINEAR_H

#include <adrcore/types/nview.h>
#include <adrcore/nview/nviewoperator.h>
#include <adrcore/nview/autocalibratorcommon.h>

namespace adore
{
	namespace core
	{
		/**
		 
		*/
		template<class LinearQCalculator>
		class AutocalibratorLinear : public NViewOperator
		{
			public:

				AutocalibratorLinear() {}

				AutocalibratorLinear ( const AutocalibratorLinear &ac ) {}

				NViewOperator *clone()
				{
					return new AutocalibratorLinear ( *this );
				}

				void modify ( NView &nview )
				{
					state_ok = false;

					if ( nview.getNumCameras() <= 1 )
					{
						DBGOUT ( DBGCRITICAL, "Not enough Cameras to calibrate\n" );
						return;
					}

					//got Cameras and 3D-points
					DBGOUT ( DBGNORMAL, "autocalibrating %d cams, %u points\n", nview.getNumCameras(), nview.getNumPoints3D() );

					AMatrix Q( 4, 4 );
					bool have_good_q = calculateQ<LinearQCalculator>( nview, Q, ac );
					if(!have_good_q)
					{
						DBGOUT ( DBGCRITICAL, "Could not calculate good Q\n" );
						return;
					}

					state_ok = true;

					ac.computeMetricReconstruction ( Q, nview.getCamerasBegin(), nview.getNumCameras(), nview.getPoints3DBegin(), nview.getNumPoints3D() );
				}

			private:

				AutocalibratorCommon ac;

		};

	}
}

#endif
