

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "featurepointfilter2.h"
#include <adrcore/mestimate/filterutil.h>

namespace adore
{

	namespace core
	{

		void FeaturePointFilter2::modify ( NView &nview )
		{
			//delte 2-Corrs if 3-View-Filtering is done? TODO: make parameter
			const bool remove2Corrs = true;

			DBGOUT( DBGNORMAL, "FeaturePointFilter2::modify entered\n" );
			assert( nview.isNormalized() );

#ifdef TAKE_FPF_TIME
			timeval t1, t2;
			gettimeofday( &t1, NULL );
#endif

			if ( nview.getNumViews() == 2 && ranser2 )
			{
				ranser2->init( nview.getPointCorrsBegin(), nview.getNumPointCorrs(), nview.getViewsBegin() );
				DBGOUT( DBGNORMAL, "reanser2 numCandidates: %d\n", ranser2->getNumCandidates() );

				filter( nview, ranser2, RansacParams( ranser2->getNumCandidates() ) );
				nview.setInfo( ranser2->createNViewInfo() );
			}
			if ( nview.getNumViews() == 3 )
			{
				if ( ranser2 )
				{
					DBGOUT( DBGNORMAL, "\n\n" );
					ranser2->init( nview.getPointCorrsBegin(), nview.getNumPointCorrs(), nview.getViewsBegin() );
					DBGOUT( DBGNORMAL, "reanser2a numCandidates/numPointCorrs: %d/%d\n", ranser2->getNumCandidates(), nview.getNumPointCorrs() );
					filter( nview, ranser2, RansacParams( ranser2->getNumCandidates() ) );
					DBGOUT( DBGNORMAL, "\n\n" );
					ranser2->init( nview.getPointCorrsBegin(), nview.getNumPointCorrs(), ++nview.getViewsBegin() );
					DBGOUT( DBGNORMAL, "reanser2b numCandidates/numPointCorrs: %d/%d\n", ranser2->getNumCandidates(), nview.getNumPointCorrs() );
					filter( nview, ranser2, RansacParams( ranser2->getNumCandidates() ) );
				}

				if ( ranser3 )
				{
					if ( remove2Corrs )
					{
						NView::CorrIterator cit = nview.getPointCorrsBegin();
						while ( cit != nview.getPointCorrsEnd() )
						{
							PointCorr *c = *cit;
							assert( c );
							++cit;//increment before removal
							if ( c->getSize() < 3 ) nview.removePointCorrAnd3D ( c );
						}
					}

					DBGOUT( DBGNORMAL, "FeaturePointFilter2: 3 views and ranser3\n" );
					DBGOUT( DBGNORMAL, "\n\n" );
					ranser3->init( nview.getPointCorrsBegin(), nview.getNumPointCorrs(), nview.getViewsBegin() );
					DBGOUT( DBGNORMAL, "reanser3 numCandidates/numPointCorrs: %d/%d\n", ranser3->getNumCandidates(), nview.getNumPointCorrs() );
					filter( nview, ranser3, RansacParams( ranser3->getNumCandidates() ) );
					nview.setInfo( ranser3->createNViewInfo() );
					DBGOUT( DBGNORMAL, "\n\n" );
				}
				else
				{
					DBGOUT( DBGNORMAL, "FeaturePointFilter2: 3 views but no ranser3\n" );
				}
			}

#ifdef TAKE_FPF_TIME
			int gain = nview.getNumPointCorrs();
			gettimeofday( &t2, NULL );
			long diff = 1000000 * t2.tv_sec + t2.tv_usec - 1000000 * t1.tv_sec - t1.tv_usec;
//			printf( "numnviews = %d, difftime = %ld\n", nview.getNumViews(), diff );
			if ( fpfile.length() > 0 && diff > 100 )
			{
				ostringstream oss;
				NView::ViewIterator vit = nview.getViewsBegin();
				oss << fpfile << "_" << ( *vit )->getId(); ++vit;
				oss << "_" << ( *vit )->getId();
				if ( nview.getNumViews() == 3 )
				{
					++vit;
					oss << "_" << ( *vit )->getId();
				}

				FILE *fpf1 = fopen( string( oss.str() + ".time" ).c_str(), "a" );
				fprintf( fpf1, "%ld\n", diff );
				fclose( fpf1 );
				FILE *fpf2 = fopen( string( oss.str() + ".prevail" ).c_str(), "a" );
				fprintf( fpf2, "%d\n", gain );
				fclose( fpf2 );
			}
#endif
		}
	}//ns
}
