

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWFEATUREPOINTFILTER2_H
#define ADORE_NVIEWFEATUREPOINTFILTER2_H

#include <adrcore/nview/nviewoperator.h>
#include <adrcore/random/ransacclient.h>
#include <adrcore/random/nviransacclient.h>
#include <adrcore/random/ransacparams.h>

//#define TAKE_FPF_TIME 1

namespace adore
{

	namespace core
	{

		/**
		 * Whether or not a normalization is required gets determined by
		 * the corresponding property of the nview.
			 
		*/
		class FeaturePointFilter2 : public NViewOperator
		{
			public:
#ifdef TAKE_FPF_TIME
				string fpfile;
#endif

			private:
				NVIRANSACClient *ranser2;
				NVIRANSACClient *ranser3;
				bool ownsR2;
				bool ownsR3;

			public:

				FeaturePointFilter2 ( NVIRANSACClient *ri2, NVIRANSACClient *ri3 ) :
						ranser2 ( ri2 ), ranser3 ( ri3 ), ownsR2( false ), ownsR3( false )
				{
					;
				}

				FeaturePointFilter2 ( const FeaturePointFilter2 &fpf ): ranser2( NULL ), ranser3( NULL ), ownsR2( true ), ownsR3( true )
				{
					if ( fpf.ranser2 )
					{
						ranser2 = fpf.ranser2->clone();
					}
					if ( fpf.ranser3 )
					{
						ranser3 = fpf.ranser3->clone();
					}
				}

				~FeaturePointFilter2()
				{
					if ( ownsR2 ) delete ranser2;
					if ( ownsR3 ) delete ranser3;
				}

				NViewOperator *clone()
				{
					return new FeaturePointFilter2( *this );
				}

				void modify ( NView &nview );

				void setRanser2 ( NVIRANSACClient* theValue )
				{
					ranser2 = theValue;
				}

				NVIRANSACClient* getRanser2() const
				{
					return ranser2;
				}

				void setRanser3 ( NVIRANSACClient* theValue )
				{
					ranser3 = theValue;
				}

				NVIRANSACClient* getRanser3() const
				{
					return ranser3;
				}
		};



	}
}

#endif
