

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "featurepointfilterfmat.h"
#include <adrcore/features/linedistfilter.h>

namespace adore
{

	namespace core
	{

		void FeaturePointFilterFMat::modify2( NView &nview )
		{
			DBGOUT( DBGNORMAL, "FeaturePointFilterFMat: filtering features (2 views), dist thresh is %f\n", thresh );

			if ( nview.isNViewSet() )
			{
				NViewInfo *nvi = nview.getInfo();
				if ( !nvi->isOkay() )
				{
					DBGOUT( DBGNORMAL, "FeaturePointFilterFMat: No fmatrix -> not filtering" );
					return;
				}
				assert( nvi->getLambda() == 0 );

				const AMatrix & fmat = nvi->getFMatrix();
				AVector line(3);

				DBGOUT( DBGNORMAL, "numPointCorrs = %d\n", nview.getNumPointCorrs() );
				NView::CorrIterator it = nview.getPointCorrsBegin();
				while ( it != nview.getPointCorrsEnd() )
				{
					bool remove = true;
					PointCorr *corr = *it;	++it;

					assert( corr );
					if ( corr->getSize() == 2 )
					{
						PointCorr::Iterator fpit = corr->getBegin();
						FeaturePoint *fp1 = *fpit;
						++fpit;
						FeaturePoint *fp2 = *fpit;
						if ( fp1 && fp2 )
						{
							matMul2( fmat, *fp1, line );
							LineDistance ldist( line( 0 ), line( 1 ), line( 2 ) );
							if ( !ldist.isOkay() )
							{
								DBGOUT( DBGNORMAL, "FeaturePointFilterFMat: insane line occurred - STOP" );
								printMat( DBGNORMAL, fmat );
								printMat( DBGNORMAL, *fp1 );
								return;
							}
							assert( ( *fp2 )( 2 ) == 1 );
							float dist = fabsf( ldist.getDistance( ( *fp2 )( 0 ), ( *fp2 )( 1 ) ) );
							// 					printf("dist: %f\n", dist);
							if ( dist < thresh )
							{
								remove = false;
							}
						}
					}

					if ( remove )
					{
						nview.removePointCorrAnd3D( corr );
					}
				}
				DBGOUT( DBGNORMAL, "numPointCorrs = %d\n", nview.getNumPointCorrs() );
			}
			else
			{
				DBGOUT( DBGNORMAL, "FeaturePointFilterFMat: Not an NViewSet -> no fmatrix -> not filtering" );
			}
		}


	}

}
