

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWLINEARTRIANGULATOR_H
#define ADORE_NVIEWLINEARTRIANGULATOR_H

#include <vector>
#include <adrcore/types/camera.h>
#include <adrcore/types/pointcorr.h>
#include <cfloat>
#include <adrcore/util/debug.h>
#include <adrcore/util/solvesvd.h>

namespace adore
{
	namespace core
	{

		using namespace std;

		/**
		 * This class computes 3D-points from 2D-points and their camera matrices.
		 * It utilizes the DLT algorithm.
		 *  
		*/
		class LinearTriangulator
		{
			public:
				/**
				 * Constructor.
				 */
				LinearTriangulator();

				/**
				 * Destructor.
				 */
				~LinearTriangulator();

				/**
				 * calculates a 3D-point from a set of cameras and points; one point for each camera.
				 * This is the homogenous method of section 12.2 in the HZ-Book
				 * @param cams a vector of n cameras
				 * @param points a vector of n points. Though only the first two coordinates are used, the points may be homogenous.
				 * @param result The resulting 3D point as a homogenous 4-vector
				 */
				bool operator() ( const vector <Camera *> &cams,
				                  const vector < AVector* > &points,
				                  AVector &result )
				{
					vector <AMatrix *> mats( cams.size() );
					for ( int i = cams.size() - 1; i >= 0; --i ) mats[i] = &cams[i]->getProjMat();
					return operator()( mats, points, result );
				}

				bool operator() ( const vector <AMatrix *> &cams,
				                  const vector < AVector* > &points,
				                  AVector &result );

				template<class CamIterator, class ViewIterator>
				bool operator() ( CamIterator it, PointCorr *corr, ViewIterator vit, int size, AVector &result )
				{
					vector<Camera  *> cams ( size );
					vector<AVector *> points ( size );
					for ( int i = 0; i < size; ++i, ++it, ++vit )
					{
						cams[i] = *it;
						points[i] = corr->getFeature ( *vit );
					}
					return operator() ( cams, points, result );
				}

			private:
				uint numCams;
				AMatrix A;
				SVDStoraged store4;

				void fillA ( const vector <AMatrix *> &cams, const vector < AVector* > &points );

		};

	}

}

#endif
