

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWMERGERHMINFUNC_H
#define ADORE_NVIEWMERGERHMINFUNC_H

#include <adrcore/types/aarray.h>
#include <vector>

namespace adore
{
	namespace core
	{

		using namespace std;

		/**
		 * Used internally.
		 * Output dimension: 2 * feats->size() * (*feats)[0].size()
		 *  
		*/
		class MergerHMinFunc
		{
			private:
//				typedef AVector ( MergerHMinFunc::* reproErr ) ( AMatrix &H ) const;
			public:
				typedef double T_numtype;
				enum HMIN_METHOD
				{
					HMIN_SINGLE = 0,
					HMIN_BOTH
				};

			private:
//				reproErr reproFunc;
				HMIN_METHOD method;

				vector<AVector3D *> *pts3D1;
				vector<AVector3D *> *pts3D2;
				vector<AMatrix *> *cams1;
				vector<AMatrix *> *cams2;
				vector< vector<AVector2D *> > *feats;

				void reprojection ( const AMatrix &H, AVector &result ) const;
				void reprojectionBoth ( const AMatrix &H, AVector &result ) const;

				mutable AMatrix H44;
				mutable AMatrix invH44;
			public:
				/**
				 * Constructor.
				 */
				MergerHMinFunc ( vector< vector< AVector2D* > > *p2D );

				/**
				 */
				AVector operator() ( const AVector &H ) const;
				void operator() ( const AVector &H, AVector &result ) const;

				void set3DPoints ( vector< AVector3D* > *p3D1, vector< AVector3D* > *p3D2 )
				{
					pts3D1 = p3D1;
					pts3D2 = p3D2;
				}

				void setCameras ( vector<AMatrix *> *cams1, vector<AMatrix *> *cams2 )
				{
					this->cams1 = cams1;
					this->cams2 = cams2;
				}

				void setFeats ( vector< vector< AVector2D* > > *p2D )
				{
					feats = p2D;
				}

				void useBoth ( bool yes )
				{
//					if ( yes ) reproFunc = &MergerHMinFunc::reprojectionBoth;
//					else reproFunc = &MergerHMinFunc::reprojection;
					if ( yes ) method = HMIN_BOTH;
					else method = HMIN_SINGLE;
				}
		};

	}

}

#endif
