

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREPARALLELAUTOMERGER_H
#define ADORE_COREPARALLELAUTOMERGER_H

#include "merger.h"
#include "badpointremover.h"
#include <adrcore/nview/baprojective.h>
#include <adrcore/util/thread.h>

namespace adore
{

	namespace core
	{

		/**
			 
		*/
		class ParallelAutoMerger
		{
			private:

				class MJob : public Job
				{
					private:
						typedef ViewKnitter::VTuple VTuple;
						typedef ViewKnitter::Tuple Tuple;

						Merger merger;
						BadPointRemover2 bpr2;//repro err
						BadPointRemover3 bpr3;//in front of cams
						BAProjectiveSimple ba;
						Point3DCalculator p3dCalc;

						NView *nv1;
						NView *nv2;
						NView *result;
						Tuple tup;

					public:
						MJob():Job(), nv1( NULL ), nv2( NULL ), result( NULL )
						{
							p3dCalc.setMinimizeError( true );
							merger.setMinimizeReproErr( true );
							merger.setUseCameras ( true );
							merger.setUsePoints3D ( false );
						}

						void setBPR2ReproThresh( const double val )
						{
							bpr2.setThresh( val );
						}

						void set( NView *n1, NView *n2, NView *r, Tuple t )
						{
							nv1 = n1;
							nv2 = n2;
							result = r;
							tup = t;
						}

					protected:
						//overloaded
						void execute()
						{
							assert( nv1 );
							assert( nv2 );
							assert( result );

							if ( !merger.merge( *nv1, *nv2, *result, tup ) )
							{
								DBGOUT( DBGALL, "merging failed\n" );
								error( "blub\n" );
								return;
							}

							if ( nv1->isNormalized() && nv2->isNormalized() )
							{
								result->setNormalized( true );
							}
							else if ( !nv1->isNormalized() && !nv2->isNormalized() )
							{
								result->setNormalized( false );
							}
							else
							{
								error( "Merger found mixed normalizations\n" );
							}

							//if one of those has 3d points already, we assume a recalculation
							if ( nv1->getNumPoints3D() || nv2->getNumPoints3D() )
							{
								p3dCalc.modify( *result );
							}

							bpr2.modify( *result );

//							if ( bundleAdjust )
//							{
								ba.modify( *result );
								DBGOUT( DBGNORMAL, "Merger: BA after merging %d views needs %d iterations\nresidual(before - after): %.30f - %.30f\n", result->getNumViews(), ba.getLastIterations(), ba.getInitialError(), ba.getLastError() );
//							}

							DBGOUT( DBGNORMAL, "Merger: (%d + %d)->%d\n", nv1->getNumViews(), nv2->getNumViews(), result->getNumViews() );
						}
				};

				ThreadPool pool;
				vector<MJob> jobs;

			public:
				ParallelAutoMerger( double reproThresh, uint nThreads ):pool( nThreads ), jobs( nThreads )
				{
					for ( uint i = 0; i < nThreads; ++i )
					{
						jobs[i].setBPR2ReproThresh( reproThresh );
						pool.getThread( i )->setJob( &jobs[i] );
					}
				}

				/**
					@param delSourceNViews if true, the nviews that were merged
					get deleted and only the result prevails
				*/
				bool merge ( NViewMap &nviewmap, ViewKnitter &knit, const bool delSourceNViews = false );
		};

	}

}

#endif
