

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "point3dcalculator.h"

namespace adore
{

	namespace core
	{

		Point3DCalculator::Point3DCalculator():
				minimize( true )
		{
		}

		Point3DCalculator::Point3DCalculator( const Point3DCalculator &pc ):
				minimize( pc.minimize )
		{
		}

		Point3DCalculator::~Point3DCalculator()
		{
		}

		void Point3DCalculator::modify( NView &nview )
		{
			int numViews = nview.getNumViews();
			if ( numViews < 2 ) return;
			if ( nview.getNumCameras() < numViews ) return;
			DBGOUT( DBGNORMAL, "Point3DCalculator triangulates points (numViews: %d, numCams: %d)\n", numViews, nview.getNumCameras() );

			for ( uint i = 0; i < feats.size(); ++i )
			{
				feats[i].clear();
			}
			feats.resize( numViews );

			vector<Camera *> cams( nview.getNumCameras() );
			nview.getCameras( cams.begin() );

			vector<Camera *> triCams; //a subset of cams for triangulation. Those cameras are missing whose 2D points are NULL
			triCams.reserve( numViews );

			nview.getPointCorrs( feats );

			vector<AVector2D *> points( numViews );
			AVector3D *result;

			vector< vector<FeaturePoint *>::iterator > it( numViews );
			for ( int i = 0; i < numViews; ++i )
			{
				if ( feats[i].empty() )
				{
					DBGOUT( DBGNORMAL, "Point3DCalculator has found no feature points in some view(%d)\n", i );
				}
				it[i] = feats[i].begin();
			}

			//check, if the fmatrix can be used instead of iterative correction
			AMatrix fmatrix( 3, 3 );
			bool hasFMat = false;
			if ( minimize && numViews == 2 )
			{
				if ( nview.isNViewSet() && nview.getInfo()->isOkay() )
				{
					fmatrix = nview.getInfo()->getFMatrix();
					hasFMat = true;
				}
			}

//			int iii = 0;
			while ( it[0] != feats[0].end() )
			{
				points.clear();
				triCams.clear();

				assert( ( int )it.size() == numViews );
				for ( int i = 0; i < numViews; ++i )
				{
					if ( *( it[i] ) )
					{
						AVector2D *tpt = new AVector2D( *( *( it[i] ) )/*->copy()*/ );
						points.push_back( tpt );
						triCams.push_back( cams[i] );
//						printf( "cam %d:\n", i );
//						cout << cams[i]->getProjMat();
//						printf( "2d pt %d:\n", i );
//						cout << **it[i] << endl;
					}
				}

				//at least two 2D points are needed to compute a 3D point
				if ( points.size() < 2 )
				{
					for ( int i = 0; i < numViews; ++i ) ++it[i];
					continue;
				}

				result = NULL;
				FeaturePoint *indexPoint = NULL;
				for ( int i = 0; i < numViews; ++i )
				{
					if ( *it[i] )
					{
						indexPoint = *it[i];
						result = nview.getPoint3D( indexPoint );
						break;// one it is sufficient, because the others have the same Correpondence
					}
				}
				if ( !result )
				{
					result = new AVector3D( 4 );
					nview.setPoint3D( result, indexPoint );
				}

				if ( hasFMat )
				{
//					DBGOUT( DBGALL, "Point3DCalculator - correcting correspondences for fmatrix\n" );
					correctFM( *points[0], *points[1], fmatrix );
//					DBGOUT( DBGALL, "Point3DCalculator - correcting done\n" );
				}

//				printf( "Triangel in Point3DCalc, cams: %d\n", triCams.size() );
				if ( triangulator( triCams, points, *result ) )
				{
//					printf( "Correction in Point3DCalc, cams: %d\n", cams.size() );
//					printf( "3d point before iterative correction: (%f, %f, %f, %f)\n", ( *result )( 0 )/( *result )( 2 ), ( *result )( 1 )/( *result )( 2 ), 1.0, ( *result )( 3 )/( *result )( 2 ) );
//					cout << *result << endl;
//					cout << iii << endl;
//					++iii;
//					printMat(*result, 45); cout << endl;
					if ( fabs( ( *result )( 2 ) ) >= FLT_EPSILON && minimize && !hasFMat )
					{
						Point3DCorrector correct( points.size() ); //false: inhomogen minimization
						correct( triCams, points, *result );
//						printMat(*result, 25); cout << endl;
					}
//					printf( "3d point after iterative correction: (%f, %f, %f, %f)\n", ( *result )( 0 ), ( *result )( 1 ), ( *result )( 2 ), ( *result )( 3 ) );
//					printMat(*result, 45); cout << endl;
				}
				else
				{
					DBGOUT( DBGALL, "Point3DCalculator: triangulation failed\n" );
				}

				int idx = 0;
				for ( int i = 0; i < numViews; ++i )
				{
					if ( *it[i] )
					{
						delete points[idx++];
					}
					++it[i];
				}
			}

//			printf( "leaving\n" );
		}

	}
}//namespace

