

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_VIEWMATCHPOINT3DCORRFUNC_H
#define ADORE_VIEWMATCHPOINT3DCORRFUNC_H

#include <vector>
#include <adrcore/types/camera.h>

namespace adore
{
	namespace core
	{

		using namespace std;

		/**
		 * This is a LMFunction used by the Point3DCorrector.
		 * Essentially, it projects a 3D-point via a set of Cameras to a set of 2D-points.
		 *  
		*/
		class Point3DCorrFunc
		{
			public:

				typedef double T_numtype;

			public:
				/**
				 * Constructor.
				 * (homogen removed - always inhomogeneous 2D points)
				 * @param cams the cameras.
				 * initialization of domain and range dimensions.
				 */
				Point3DCorrFunc( vector<Camera *> *cams = NULL );

				/**
				 * Destructor.
				 */
				~Point3DCorrFunc();

				/**
				 * Carries out the projection.
				 * @param pt3D a 3-Vector representing a 3DPoint (x, y, 1, w)
				 * @param result all 2D-points (one for each cam)
				 */
				void operator()( const AVector & pt3D, AVector & result ) const;

				/**
				 * Tell this functor, what cams it should use.
				 * @param theValue the cameras
				 */
				void setCams( vector< Camera * >* theValue )
				{
					camvec = theValue;
				}

			protected:
				vector<Camera *> *camvec;
		};

	}

}

#endif
