

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWQUASIAFFINATOR_H
#define ADORE_NVIEWQUASIAFFINATOR_H

#include <adrcore/types/nview.h>
#include <adrcore/nview/nviewoperator.h>
#include <adrcore/nview/fmatrixcalculator.h>
#include <adrcore/util/mathutil.h>

namespace adore
{
	namespace core
	{

		/**
		 
		*/
		class QuasiAffinator : public NViewOperator
		{
			public:
				QuasiAffinator();

				~QuasiAffinator();

				void modify( NView &nview );

				template<class CorrIterator, class ViewIterator>
				static int calcOrientationScore( const AMatrix &fmat, const AVector & epi2, CorrIterator cit, int size, ViewIterator vit )
				{
					int score = 0;
					ViewBase *v1 = *vit; ++vit;
					ViewBase *v2 = *vit;
					for ( int i = 0; i < size; ++i )
					{
						PointCorr *corr = *cit;
						if ( !corr ) continue;
						AVector *fp1 = corr->getFeature( v1 );
						if ( !fp1 ) continue;
						AVector *fp2= corr->getFeature( v2 );
						if ( !fp2 ) continue;
						bool o = FMatrixCalculator::isOriented ( fmat, epi2, *fp1, *fp2 );
						score += o ? 1 : -1;
					}
					return score;
				}

				///epi2 will hold the computed epipole
				///returns false if orientation has failed
				template<class CorrIterator, class ViewIterator>
				static bool calcOrientedEpipole( AMatrix &fmat, CorrIterator cit, int size, ViewIterator vit, AVector & epi2 )
				{
					SVDStoraged store3( 3, 3 );//TODO
					transInPlace( fmat );
					FMatrixCalculator::calculateEpipolarPoint( fmat, epi2, store3 );
					transInPlace( fmat );
					int score = calcOrientationScore( fmat, epi2, cit, size, vit );
					if ( score == 0 ) return false;
					if ( score < 0 ) epi2 *= -1;
					return true;
				}

				/**
					count the number of points not visible by their cameras (which is not desired)
				*/
				static int countInvisible ( NView &nview );

				/**
					count the number of points not visible by their cameras, after those cameras get transformed by H and the points by H^-1
				 * Side effect: H gets inverted
				 */
				static int countInvisible ( NView &nview, AMatrix &H );
		};

	}

}

#endif
