

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWTENSORUPDATER_H
#define ADORE_NVIEWTENSORUPDATER_H

#include <adrcore/types/nview.h>
#include <adrcore/nview/fmatrixcalculatorpvp.h>
#include <adrcore/nview/ttensorcalculatorppp.h>
#include <adrcore/nview/nviewoperator.h>

namespace adore
{
	namespace core
	{

		/**
		 * This class calculates the tensors from point correspondences only.
		 *  
		*/
		class TensorUpdater : public NViewOperator
		{
			public:
				/**
				 * Constructor.
				 * @param normalize you can switch on normalization.
				 * In most cases, it is a better idea to normalize the points before using the mechanism providet
				 * by View
				 */
				TensorUpdater();

				TensorUpdater( const TensorUpdater &tu );

				/**
				 * Destructor.
				 */
				~TensorUpdater();

				NViewOperator *clone()
				{
					return new TensorUpdater( *this );
				}

				/**
				 * computes the Tensor for the given NViewSet. Currently, only NViewSets with 2 or 3 Views are supported.
				 * @param nviewset the NViewSet
				 */
				void modify( NView &nview );

				void modify2( NView &nview );

				void modify3( NView &nview );

				void setMinAlgebraic( bool theValue )
				{
					minAlgebraic = theValue;
				}

				bool getMinAlgebraic() const
				{
					return minAlgebraic;
				}

			private:
				bool minAlgebraic;

				FMatrixCalculatorPVP fcalc;
				TTensorCalculatorPPP tcalc;
//				vector< vector<FeaturePoint *> > feats;
		};

	}

}

#endif
