

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "ttensorcalculator.h"
#include <adrcore/util/printmat.h>
#include <adrcore/util/mathutil.h>
#include <adrcore/util/matmul2.h>
#include <adrcore/util/storageorder.h>
#include <adrcore/util/floatshift.h>
#include <adrcore/minimization/defaulterrorfunc2.h>
#include <adrcore/util/numericalanalysis.h>

#ifndef NDEBUG
#include <adrcore/nview/merger.h>
#include <adrcore/nview/point3dcorrector.h>
#endif

//just in case it turns out to be easily changed to n views
#define NVIEWS 3
#define RANK_E 15

	namespace adore
	{
		namespace core
		{

			TTensorCalculator::TTensorCalculator():
			pts6Stable( 3 ),
			pts6StableEqRow( 0 ),
			solution1( 3, 3 ),
			solution2( 3, 3 ),
			result1( 3, 3 ),
			result2( 3, 3 ),
			result3( 3, 3 ),
			X1( 4 ),
			X2( 4 ),
			X3( 4 ),
			camSelections( 4, 4 ),
			tensor1( 3, 3, 3 ),
			tensor2( 3, 3, 3 ),
			tensor3( 3, 3, 3 ),
			//Matrices for the linear solution
			//varible sized matrices
			linearEq(),
			lineTensor( 27 ),
			store27( 0, 27, SVD_VT ),
			E( 27, 18 ),
			//algebraic constraint enforcement
			store27_18( 27, 18, SVD_VT | SVD_U ),
			store15( 0, RANK_E, SVD_VT ),
			candT15( RANK_E ),
			matsize_15(),//variable sized
			epipole1( 3 ),
			epipole2( 3 ),
			epipoles( 6 ),
			lmGoal( 27 ),
			camRemains( 18 ),
			store3( 3, 3, SVD_VT ),
			vec3( 3 ),
			mat33( 3,3 )
			{
				T[0].resize( 3,3 );
				T[1].resize( 3,3 );
				T[2].resize( 3,3 );
				E = 0;

				for ( int i = 0; i < NVIEWS; ++i )
				{
					pts6Stable[i].resize( 6 );
					for ( int j = 0; j < 6; ++j )
					{
						pts6Stable[i][j].resize( 3 );
						pts6Stable[i][j]( 2 ) = 1.0;
					}
				}
			}


			TTensorCalculator::~TTensorCalculator()
			{
			}

			void TTensorCalculator::init( int numPPP, int numLLL, int numPLL, int numPPL )
			{
				//init class members used by createVarMatrices
				int eqSize = numPPP * 4 + numLLL * 2 + numPLL + numPPL * 2;

				linearEq.resizeAndPreserve( eqSize, 27 );
				matsize_15.resizeAndPreserve( eqSize, RANK_E );
				store15.changeRowsWorkspaceOnly( eqSize );
				store15.repairD();
				store27.changeRowsWorkspaceOnly( eqSize );
				store27.repairD();
			}

//			the linear steps

//			adds 4 equations corresponding to the three points
			void TTensorCalculator::setPPP( int corrNum, double p11, double p12, double p13, double p21, double p22, double p31, double p32 )
			{
				unsigned row = corrNum << 2;//four rows for each correspondence
				//first row
				assert( row < linearEq.rows() );
				linearEq( row,  0 ) =  p11;
				linearEq( row,  1 ) = 0;
				linearEq( row,  2 ) =  -p11*p31;
				linearEq( row,  3 ) = 0;
				linearEq( row,  4 ) = 0;
				linearEq( row,  5 ) = 0;
				linearEq( row,  6 ) =  -p11*p21;
				linearEq( row,  7 ) = 0;
				linearEq( row,  8 ) =  p11*p21*p31;

				linearEq( row,  9 ) =  p12;
				linearEq( row, 10 ) = 0;
				linearEq( row, 11 ) =  -p12*p31;
				linearEq( row, 12 ) = 0;
				linearEq( row, 13 ) = 0;
				linearEq( row, 14 ) = 0;
				linearEq( row, 15 ) =  -p12*p21;
				linearEq( row, 16 ) = 0;
				linearEq( row, 17 ) =  p12*p21*p31;

				linearEq( row, 18 ) =  p13;
				linearEq( row, 19 ) = 0;
				linearEq( row, 20 ) =  -p13*p31;
				linearEq( row, 21 ) = 0;
				linearEq( row, 22 ) = 0;
				linearEq( row, 23 ) = 0;
				linearEq( row, 24 ) =  -p13*p21;
				linearEq( row, 25 ) = 0;
				linearEq( row, 26 ) =  p13*p21*p31;
				++row;

				//second row
				assert( row < linearEq.rows() );
				linearEq( row,  0 ) = 0;
				linearEq( row,  1 ) =  p11;
				linearEq( row,  2 ) =  -p11*p32;
				linearEq( row,  3 ) = 0;
				linearEq( row,  4 ) = 0;
				linearEq( row,  5 ) = 0;
				linearEq( row,  6 ) = 0;
				linearEq( row,  7 ) =  -p11*p21;
				linearEq( row,  8 ) =  p11*p21*p32;

				linearEq( row,  9 ) = 0;
				linearEq( row, 10 ) =  p12;
				linearEq( row, 11 ) =  -p12*p32;
				linearEq( row, 12 ) = 0;
				linearEq( row, 13 ) = 0;
				linearEq( row, 14 ) = 0;
				linearEq( row, 15 ) = 0;
				linearEq( row, 16 ) =  -p12*p21;
				linearEq( row, 17 ) =  p12*p21*p32;

				linearEq( row, 18 ) = 0;
				linearEq( row, 19 ) =  p13;
				linearEq( row, 20 ) =  -p13*p32;
				linearEq( row, 21 ) = 0;
				linearEq( row, 22 ) = 0;
				linearEq( row, 23 ) = 0;
				linearEq( row, 24 ) = 0;
				linearEq( row, 25 ) =  -p13*p21;
				linearEq( row, 26 ) =  p13*p21*p32;
				++row;

				//third row
				assert( row < linearEq.rows() );
				linearEq( row,  0 ) = 0;
				linearEq( row,  1 ) = 0;
				linearEq( row,  2 ) = 0;
				linearEq( row,  3 ) =  p11;
				linearEq( row,  4 ) = 0;
				linearEq( row,  5 ) =  -p11*p31;
				linearEq( row,  6 ) =  -p11*p22;
				linearEq( row,  7 ) = 0;
				linearEq( row,  8 ) =  p11*p22*p31;

				linearEq( row,  9 ) = 0;
				linearEq( row, 10 ) = 0;
				linearEq( row, 11 ) = 0;
				linearEq( row, 12 ) =  p12;
				linearEq( row, 13 ) = 0;
				linearEq( row, 14 ) =  -p12*p31;
				linearEq( row, 15 ) =  -p12*p22;
				linearEq( row, 16 ) = 0;
				linearEq( row, 17 ) =  p12*p22*p31;

				linearEq( row, 18 ) = 0;
				linearEq( row, 19 ) = 0;
				linearEq( row, 20 ) = 0;
				linearEq( row, 21 ) =  p13;
				linearEq( row, 22 ) = 0;
				linearEq( row, 23 ) =  -p13*p31;
				linearEq( row, 24 ) =  -p13*p22;
				linearEq( row, 25 ) = 0;
				linearEq( row, 26 ) =  p13*p22*p31;
				++row;

				//fourth row
				assert( row < linearEq.rows() );
				linearEq( row,  0 ) = 0;
				linearEq( row,  1 ) = 0;
				linearEq( row,  2 ) = 0;
				linearEq( row,  3 ) = 0;
				linearEq( row,  4 ) =  p11;
				linearEq( row,  5 ) =  -p11*p32;
				linearEq( row,  6 ) = 0;
				linearEq( row,  7 ) =  -p11*p22;
				linearEq( row,  8 ) =  p11*p22*p32;

				linearEq( row,  9 ) = 0;
				linearEq( row, 10 ) = 0;
				linearEq( row, 11 ) = 0;
				linearEq( row, 12 ) = 0;
				linearEq( row, 13 ) =  p12;
				linearEq( row, 14 ) =  -p12*p32;
				linearEq( row, 15 ) = 0;
				linearEq( row, 16 ) =  -p12*p22;
				linearEq( row, 17 ) =  p12*p22*p32;

				linearEq( row, 18 ) = 0;
				linearEq( row, 19 ) = 0;
				linearEq( row, 20 ) = 0;
				linearEq( row, 21 ) = 0;
				linearEq( row, 22 ) =  p13;
				linearEq( row, 23 ) =  -p13*p32;
				linearEq( row, 24 ) = 0;
				linearEq( row, 25 ) =  -p13*p22;
				linearEq( row, 26 ) =  p13*p22*p32;

			}

//			the result is in linetensor
			int TTensorCalculator::calculateTrifocalTensorBaseLinear()
			{
//				assert_msg( linearEqRow == eqSize, "TTensor: Linear Equation not filled yet (%d < %d) \n", linearEqRow, eqSize );
				AVector &singulars27 = store27.D;
				AMatrix &V_T27 = store27.VT;
				svdVT( linearEq, store27 );

				lineTensor = V_T27( 26, Range::all() );

				if ( singulars27( 25 ) < FLT_EPSILON )
				{
					DBGOUT( DBGNORMAL, "TTensor: Linear Solution underdetermined\n" );
					DBGOUT( DBGNORMAL, "TTensor: The last 3 singular values are: %.10f   %.10f   %.10f\n", singulars27( 24 ), singulars27( 25 ), singulars27( 26 ) );
					return -1;
				}
				DBGOUT( DBGNORMAL, "smallest singular value after calculateTrifocalTensorBaseLinear: %.20f\n", singulars27( 26 ) );

				//normalizeExponents(lineTensor);//OPT

				return 1;
			}

			ATensor3 TTensorCalculator::buildTensorFromLineTensor( const AVector &colVector )
			{
				ATensor3 tensor( 3,3,3 );
				buildTensorFromLineTensor( colVector, tensor );
				return tensor;
			}

			void TTensorCalculator::buildTensorFromLineTensor( const AVector &colVector, ATensor3 &tensor )
			{
				tensor( 0,0,0 ) = colVector(  0 );
				tensor( 0,0,1 ) = colVector(  1 );
				tensor( 0,0,2 ) = colVector(  2 );
				tensor( 0,1,0 ) = colVector(  3 );
				tensor( 0,1,1 ) = colVector(  4 );
				tensor( 0,1,2 ) = colVector(  5 );
				tensor( 0,2,0 ) = colVector(  6 );
				tensor( 0,2,1 ) = colVector(  7 );
				tensor( 0,2,2 ) = colVector(  8 );
				tensor( 1,0,0 ) = colVector(  9 );
				tensor( 1,0,1 ) = colVector( 10 );
				tensor( 1,0,2 ) = colVector( 11 );
				tensor( 1,1,0 ) = colVector( 12 );
				tensor( 1,1,1 ) = colVector( 13 );
				tensor( 1,1,2 ) = colVector( 14 );
				tensor( 1,2,0 ) = colVector( 15 );
				tensor( 1,2,1 ) = colVector( 16 );
				tensor( 1,2,2 ) = colVector( 17 );
				tensor( 2,0,0 ) = colVector( 18 );
				tensor( 2,0,1 ) = colVector( 19 );
				tensor( 2,0,2 ) = colVector( 20 );
				tensor( 2,1,0 ) = colVector( 21 );
				tensor( 2,1,1 ) = colVector( 22 );
				tensor( 2,1,2 ) = colVector( 23 );
				tensor( 2,2,0 ) = colVector( 24 );
				tensor( 2,2,1 ) = colVector( 25 );
				tensor( 2,2,2 ) = colVector( 26 );
			}

			AVector TTensorCalculator::buildLineTensorFromTensor( const ATensor3 &tensor )
			{
				AVector colVector( 27 );
				buildLineTensorFromTensor( tensor, colVector );
				return colVector;
			}

			void TTensorCalculator::buildLineTensorFromTensor( const ATensor3 &tensor, AVector &colVector )
			{
				colVector(  0 ) = tensor( 0,0,0 );
				colVector(  1 ) = tensor( 0,0,1 );
				colVector(  2 ) = tensor( 0,0,2 );
				colVector(  3 ) = tensor( 0,1,0 );
				colVector(  4 ) = tensor( 0,1,1 );
				colVector(  5 ) = tensor( 0,1,2 );
				colVector(  6 ) = tensor( 0,2,0 );
				colVector(  7 ) = tensor( 0,2,1 );
				colVector(  8 ) = tensor( 0,2,2 );
				colVector(  9 ) = tensor( 1,0,0 );
				colVector( 10 ) = tensor( 1,0,1 );
				colVector( 11 ) = tensor( 1,0,2 );
				colVector( 12 ) = tensor( 1,1,0 );
				colVector( 13 ) = tensor( 1,1,1 );
				colVector( 14 ) = tensor( 1,1,2 );
				colVector( 15 ) = tensor( 1,2,0 );
				colVector( 16 ) = tensor( 1,2,1 );
				colVector( 17 ) = tensor( 1,2,2 );
				colVector( 18 ) = tensor( 2,0,0 );
				colVector( 19 ) = tensor( 2,0,1 );
				colVector( 20 ) = tensor( 2,0,2 );
				colVector( 21 ) = tensor( 2,1,0 );
				colVector( 22 ) = tensor( 2,1,1 );
				colVector( 23 ) = tensor( 2,1,2 );
				colVector( 24 ) = tensor( 2,2,0 );
				colVector( 25 ) = tensor( 2,2,1 );
				colVector( 26 ) = tensor( 2,2,2 );
			}

//			calculation of trifocal tensor minimizing algebraic error p. 396 - BEGIN

//			linearEq is assumed to be already filled
			int TTensorCalculator::calculateTrifocalTensorBaseMinAlgebraicError( bool doIterate, AMatrix *cam1, AMatrix *cam2 )
			{
				calculateTrifocalTensorBaseLinear(); //initial linear estimate
				//the result is in linetensor

//				cout.precision(25);
//				cout << lineTensor << endl;

				epipole1 = calculateEpipole( lineTensor, true );
				epipole2 = calculateEpipole( lineTensor, false );
//				cout << epipole1 << endl;
//				cout << epipole2 << endl;

//				printf( "before\n" );
//				std::cout << lineTensor << endl;
//				printMats( epipole1, epipole2 );
//				lmGoal = lineTensor;//copy is essential
				if ( doIterate )
				{
					lmGoal = lineTensor;//copy is essential

//					error27 for LM = linearEq * lineTensor
					epipoles( 0 ) = epipole1( 0 );
					epipoles( 1 ) = epipole1( 1 );
					epipoles( 2 ) = epipole1( 2 );
					epipoles( 3 ) = epipole2( 0 );
					epipoles( 4 ) = epipole2( 1 );
					epipoles( 5 ) = epipole2( 2 );

//					printf( "goal ---\n" );
//					printMat( lmGoal );
//					printf( "goal ---\n" );

					//6->27
					TTAlgebraicFunc algebraicFunc( this );
					LMFunctionJacobi2<TTAlgebraicFunc> algebraicJacobi( algebraicFunc, 6, 27 );
					DefaultErrorFunc2<double> errFunc( lmGoal );
					typedef LM2<TTAlgebraicFunc, LMFunctionJacobi2<TTAlgebraicFunc>, DefaultErrorFunc2<double>, InvCovarUnused<double>::type > lmT;
					lmT lm;

					lm.useInitialLambda( true );
					lm.setInitialLambda( 0.001 );
					lm.solve( epipoles, algebraicFunc, lmGoal.size(), algebraicJacobi,
					          errFunc,
					          lmT::DefaultInvCovar,
					          DBL_EPSILON,
					          1000 );
					DBGOUT( DBGNORMAL, "Algebraic minimization takes %d iterations (residual: %.20f - %.20f)\n", lm.getLastIterations(), lm.getInitialError(), lm.getLastError() );

					epipole1( 0 ) = epipoles( 0 );
					epipole1( 1 ) = epipoles( 1 );
					epipole1( 2 ) = epipoles( 2 );
					epipole2( 0 ) = epipoles( 3 );
					epipole2( 1 ) = epipoles( 4 );
					epipole2( 2 ) = epipoles( 5 );
//					cout << epipole1 << endl;
//					cout << epipole2 << endl;
				}

//				printf( "iteration done\n" );
//				pressKey();

				fillMatE( E, epipole1, epipole2 );
//				printMat( E );
				if ( cam1 || cam2 )
				{
					enforceEpipolesConstraint( linearEq, E, lineTensor, &camRemains );
					//minimize(linearEq, E, lineTensor, 15, camRemains);
				}
				else
				{
					enforceEpipolesConstraint( linearEq, E, lineTensor );
					//minimize(linearEq, E, lineTensor, 15);
				}

//				AVector checkTensor = matMul( E, camRemains );
//				printMats( lineTensor, checkTensor );
//				checkTensor = lineTensor - checkTensor;
//				printf( "residual (tensor difference):\n" );
//				printMat( checkTensor );
//				pressKey();
//				printf( "taucheck\n" );
//				tauCheck( lineTensor );

				if ( cam1 )
				{
//					printf( "getting cam1\n" );
					AMatrix &camMat = *cam1;
					camMat( 0, 0 ) = camRemains( 0 );
					camMat( 1, 0 ) = camRemains( 1 );
					camMat( 2, 0 ) = camRemains( 2 );
					camMat( 0, 1 ) = camRemains( 3 );
					camMat( 1, 1 ) = camRemains( 4 );
					camMat( 2, 1 ) = camRemains( 5 );
					camMat( 0, 2 ) = camRemains( 6 );
					camMat( 1, 2 ) = camRemains( 7 );
					camMat( 2, 2 ) = camRemains( 8 );
					camMat( Range::all(), 3 ) = epipole1;
					//normalizeExponents(camMat);//OPT
				}
				if ( cam2 )
				{
//					printf( "getting cam2\n" );
					AMatrix &camMat = *cam2;
					camMat( 0, 0 ) = camRemains( 9 );
					camMat( 1, 0 ) = camRemains( 10 );
					camMat( 2, 0 ) = camRemains( 11 );
					camMat( 0, 1 ) = camRemains( 12 );
					camMat( 1, 1 ) = camRemains( 13 );
					camMat( 2, 1 ) = camRemains( 14 );
					camMat( 0, 2 ) = camRemains( 15 );
					camMat( 1, 2 ) = camRemains( 16 );
					camMat( 2, 2 ) = camRemains( 17 );
					camMat( Range::all(), 3 ) = epipole2;
					//normalizeExponents(camMat);//OPT
				}
//				pressKey();

				//normalizeExponents(lineTensor);//OPT

//				double scale = 0;
//				double variance = 0;
//				scaleEqualityTest( lineTensor, lmGoal, scale, variance );
//				printf( "scale-eq-test: scale=%f, var=%f\n", scale, variance );
//				printMats( lineTensor, lmGoal );

				return 0;
			}

//			p.375
			AVector2D TTensorCalculator::calculateEpipole( const ATensor3 &tensor, bool trans )
			{
				AVector epipole( 3 );
				calculateEpipole( tensor, trans, epipole );
				return epipole;
			}

			void TTensorCalculator::calculateEpipole( const ATensor3 & tensor, bool trans, AVector & epipole )
			{
				//temporary vars
				AMatrix &V_T3 = store3.VT;
				AMatrix temp( 3, 3 );
				for ( int i = 0; i < 3; ++i )
				{
					T[i] = tensor( i, Range::all(), Range::all() );
					if ( trans )
					{
						AMatrix t = core::trans( T[i] );
						svdVT( t, store3 );
					}
					else
					{
						svdVT( T[i], store3 );
					}
					temp( i, Range::all() ) = V_T3( 2, Range::all() );//Jep, the row has to be copied to a row
				}
				svdVT( temp, store3 );
				epipole = V_T3( 2, Range::all() );
			}

//			p.375
			AVector2D TTensorCalculator::calculateEpipole( const AVector &lineTensor, bool trans )
			{
				//temporary vars
				AMatrix &V_T3 = store3.VT;
				AMatrix temp( 3, 3 );

				temp = 0;
				for ( int i = 0; i < 3; ++i )
				{
					int row = i * 9;
					T[i]( 0, 0 ) = lineTensor( row++ );
					T[i]( 0, 1 ) = lineTensor( row++ );
					T[i]( 0, 2 ) = lineTensor( row++ );
					T[i]( 1, 0 ) = lineTensor( row++ );
					T[i]( 1, 1 ) = lineTensor( row++ );
					T[i]( 1, 2 ) = lineTensor( row++ );
					T[i]( 2, 0 ) = lineTensor( row++ );
					T[i]( 2, 1 ) = lineTensor( row++ );
					T[i]( 2, 2 ) = lineTensor( row++ );

					if ( trans )
					{
						AMatrix t = core::trans( T[i] );
						svdVT( t, store3 );
					}
					else
					{
						svdVT( T[i], store3 );
					}
					temp( i, Range::all() ) = V_T3( 2, Range::all() );//Jep, the row has to be copied to a row
				}
				svdVT( temp, store3 );

				AVector2D epipole( 3 );
				epipole = V_T3( 2, Range::all() );

				return epipole;
			}


//			p.395
			void TTensorCalculator::fillMatE( AMatrix &E, const AVector2D &epi1, const AVector2D &epi2 )
			{
				double e11 = epi1( 0 );
				double e12 = epi1( 1 );
				double e13 = epi1( 2 );
				double e21 = epi2( 0 );
				double e22 = epi2( 1 );
				double e23 = epi2( 2 );

				//first 9 colums
				int row = 0;
				for ( int j = 0; j < 9; ++j )
				{
					E( row++, j ) =  e21;
					E( row++, j ) =  e22;
					E( row++, j ) =  e23;
				}

				//last 9 columns
				E(  0,  9 ) =  -e11;
				E(  1, 10 ) =  -e11;
				E(  2, 11 ) =  -e11;
				E(  3,  9 ) =  -e12;
				E(  4, 10 ) =  -e12;
				E(  5, 11 ) =  -e12;
				E(  6,  9 ) =  -e13;
				E(  7, 10 ) =  -e13;
				E(  8, 11 ) =  -e13;

				E(  9, 12 ) =  -e11;
				E( 10, 13 ) =  -e11;
				E( 11, 14 ) =  -e11;
				E( 12, 12 ) =  -e12;
				E( 13, 13 ) =  -e12;
				E( 14, 14 ) =  -e12;
				E( 15, 12 ) =  -e13;
				E( 16, 13 ) =  -e13;
				E( 17, 14 ) =  -e13;

				E( 18, 15 ) =  -e11;
				E( 19, 16 ) =  -e11;
				E( 20, 17 ) =  -e11;
				E( 21, 15 ) =  -e12;
				E( 22, 16 ) =  -e12;
				E( 23, 17 ) =  -e12;
				E( 24, 15 ) =  -e13;
				E( 25, 16 ) =  -e13;
				E( 26, 17 ) =  -e13;
			}


			void TTensorCalculator::enforceEpipolesConstraint( AMatrix &linearEq, AMatrix &E, AVector &lineTensor, AVector *camRemains )
			{
				const int rankE = RANK_E;
				const int subURows = 27;
				const int subUCols = rankE;

				AMatrix &U27_18 = store27_18.U;
				AMatrix &V_T18 = store27_18.VT;
				AMatrix &V_T15 = store15.VT;

				if ( camRemains )
				{
					//printf("enforceEpipolesConstraint - svd1\n");
					svdUVT( E, store27_18 );
//					printf( "5\n" );
					checkStorageOrder( V_T18 );
				}
				else
				{
					//in this case V_T is not needed, so let's speed it up a little.
					//printf("enforceEpipolesConstraint - svd2\n");
					svdU( E, store27_18 );
				}

//				don't use singulars18 from now on, because it is D and D'
//				same with V_T18

//				AMatrix subU( subURows, subUCols );//WORKAROUND
				AMatrix subU = U27_18( Range( 0, subURows - 1 ), Range( 0, subUCols - 1 ) );

//				printMat( subU );
//				pressKey();

//				abusing mat as a temp-variable
				matMul2(linearEq, subU, matsize_15);

//				printMat( matsize_15 );
//				pressKey();

				svdVT( matsize_15, store15 );
//				printMat( singulars15 );
//				pressKey();
//				printMat( V_T15 );
//				pressKey();

				candT15 = V_T15( rankE - 1, Range::all() );

//				printMat( candT15 );
//				pressKey();

				matMul2( subU, candT15, lineTensor );

//				printMat( lineTensor );
//				pressKey();

//				printf( "1\n" );
				//printf("enforceEpipolesConstraint - order checks\n");
				checkStorageOrder( linearEq );
//				printf( "2\n" );
				checkStorageOrder( E );
//				printf( "3\n" );
				checkStorageOrder( U27_18 );
//				printf( "4\n" );
				checkStorageOrder( matsize_15 );

				if ( camRemains )
				{
					//         AMatrix subV_T(RANK_E, V_T18.cols()); //WORKAROUND
					//printf("enforceEpipolesConstraint - svd3\n");
					AMatrix subV_T = V_T18( Range( 0, RANK_E - 1 ), Range::all() );

//					this for loop is Dinv * candT18, that is a simple division
					for ( int i = 0; i < rankE; ++i )
					{
						candT15( i ) = candT15( i ) / store27_18.D( i );
					}
					//printf("enforceEpipolesConstraint - final mul (%d, %d)(%d, %d)=(%d, %d)\n", subV_T.cols(), subV_T.rows(), candT15.rows(), candT15.cols(), camRemains->rows(), camRemains->cols());
					matMul2_AT( subV_T, candT15, *camRemains );
				}
			}




//			calculation of trifacol tensor minimizing algebraic error p. 396 - END


			/*

			Minimal Case: 6-Point algo --- BEGIN -------------------------------------------------

			*/
			void TTensorCalculator::setPPP( int row, double p11, double p12, double p21, double p22, double p31, double p32 )
			{
				assert_msg( row < 6, "You tried to use the 6 point algo with more than 6 points,\n" );

				pts6Stable[0][row]( 0 ) = p11;
				pts6Stable[0][row]( 1 ) = p12;
				pts6Stable[1][row]( 0 ) = p21;
				pts6Stable[1][row]( 1 ) = p22;
				pts6Stable[2][row]( 0 ) = p31;
				pts6Stable[2][row]( 1 ) = p32;
			}

			int TTensorCalculator::calculateTrifocalTensorBase6Point()
			{
				TTCamSolution sol[3];
				int num = calculateTrifocalTensorBase6Point( sol );
				if ( num > 0 )
				{
					calculateTrifocalTensorBase3CamsT( sol[0].getMat( 0 ), sol[0].getMat( 1 ), sol[0].getMat( 2 ), tensor1 );
//					tauCheck( tensor1 );
				}
				if ( num == 3 )
				{
					calculateTrifocalTensorBase3CamsT( sol[1].getMat( 0 ), sol[1].getMat( 1 ), sol[1].getMat( 2 ), tensor2 );
//					tauCheck( tensor2 );
					calculateTrifocalTensorBase3CamsT( sol[2].getMat( 0 ), sol[2].getMat( 1 ), sol[2].getMat( 2 ), tensor3 );
//					tauCheck( tensor3 );
				}

				return num;
			}

			int TTensorCalculator::calculateTrifocalTensorBase6Point( TTCamSolution sol[3] )
			{
				int num = dual.reconstructFrom6PointCorrespondences( pts6Stable, X1, X2, X3 );
//				COUT( DBGNORMAL, "num candidates: " << num << ", which are " << X1 << "\n" << X2 << "\n" << X3 << "\n" );
				if ( num > 0 )
				{
					if ( calculateTrifocalTensorBaseSixthPoint( X1, sol[0] ) < 0 ) return -1;
				}
				if ( num == 3 )
				{
					if ( calculateTrifocalTensorBaseSixthPoint( X2, sol[1] ) < 0 ) return -1;
					if ( calculateTrifocalTensorBaseSixthPoint( X3, sol[2] ) < 0 ) return -1;
				}

				return num;
			}

			int TTensorCalculator::calculateTrifocalTensorBaseSixthPoint( const AVector &X6, TTCamSolution &sol )
			{
				vector<AVector3D> pts3D( 6 );
				vector<AVector2D> pts2D( 6 );
				for ( uint i = 0; i < 6; ++i )
				{
//					pts2D[i] = new AVector2D( 3 );
//					pts3D[i] = new AVector3D( 4 );
					pts2D[i].resize( 3 );
					pts3D[i].resize( 4 );
				}
				AMatrix* cams[3] = { &sol.getMat( 0 ), &sol.getMat( 1 ), &sol.getMat( 2 ) };

				pts3D[0] = dual.getCanonicalX1();
				pts3D[1] = dual.getCanonicalX2();
				pts3D[2] = dual.getCanonicalX3();
				pts3D[3] = dual.getCanonicalX4();
				pts3D[4] = dual.getCanonicalX5();
				pts3D[5] = X6;

				for ( int i = 0; i < 3; ++i )
				{
					for ( int j = 0; j < 6; ++j )
					{
						pts2D[j]( 0 ) =  pts6Stable[i][j]( 0 );
						pts2D[j]( 1 ) =  pts6Stable[i][j]( 1 );
						pts2D[j]( 2 ) =  1.0;
					}
					if ( cam.calculateCamera( pts3D, pts2D, *cams[i] ) < 0 )
					{
						DBGOUT( DBGCRITICAL, "camera calculation failed, sixth point was:\n" );
						printMat( DBGCRITICAL, X6 );
						return -1;
					}
					else
					{
						DBGOUT( DBGCOND, "Condition of Camera %d:\n", i );
						printCondition( DBGCOND, *cams[i] );
						pressKey( DBGCOND );

						//should be superfluous
//						cam.minimizeError( pts3D, pts2D, cams[i] );
						normalizeExponents( cams[i]->begin(), cams[i]->end() );

						DBGOUT( DBGCOND, "Condition of Camera %d after opt and norm:\n", i );
						printCondition( DBGCOND, *cams[i] );
						pressKey( DBGCOND );
					}
				}

//				DBGOUT( DBGNORMAL, "Cameras after 6Point-Algo:\n" );
//				printMat( DBGNORMAL, cams[0], 6 );
//				DBGOUT( DBGNORMAL, "\n" );
//				printMat( DBGNORMAL, cams[1], 6 );
//				DBGOUT( DBGNORMAL, "\n" );
//				printMat( DBGNORMAL, cams[2], 6 );
//				DBGOUT( DBGNORMAL, "\n" );

				//test whether cams project the canonical points correctly
#ifndef NDEBUG
//				AVector dummyPt2D( 3 );
//				AVector imagePt2D( 3 );
//				double scale, var;
//				for ( int i = 0; i < 3; ++i )
//				{
//					for ( int j = 0; j < 6; ++j )
//					{
//						imagePt2D( 0 ) =  pts6Stable[i][j]( 0 );
//						imagePt2D( 1 ) =  pts6Stable[i][j]( 1 );
//						imagePt2D( 2 ) =  1.0;
//
//						matMul2( cams[i], pts3D[j], dummyPt2D );
//						scaleEqualityTest( dummyPt2D, imagePt2D, scale, var );
//						printf( "2d point and its reprojection\n" );
//						printMats( dummyPt2D, imagePt2D );
//						printf( "scale: %f, var: %f\n", scale, var );
//					}
//				}
//				printf( "\n\n" );
#endif

#ifndef NDEBUG
//				Camera daCam( cams[0] );
//				AVector2D p( 4 );
//				daCam.decompose();
//				p = daCam.getLocation();
//				printf( "6point - cam1: (%f, %f, %f)\n", p( 0 ), p( 1 ), p( 2 ) );
//				printMats( daCam.getCalibrationMatrix(), daCam.getRotationMatrix() );
//
//				daCam.getProjMat() = cams[1];
//				daCam.decompose();
//				p = daCam.getLocation();
//				printf( "6point - cam2: (%f, %f, %f)\n", p( 0 ), p( 1 ), p( 2 ) );
//				printMats( daCam.getCalibrationMatrix(), daCam.getRotationMatrix() );

//				daCam.getProjMat() = cams[2];
//				daCam.decompose();
//				p = daCam.getLocation();
//				printf( "6point - cam3: (%f, %f, %f)\n", p( 0 ), p( 1 ), p( 2 ) );
//				printMats( daCam.getCalibrationMatrix(), daCam.getRotationMatrix() );
#endif

#ifndef NDEBUG
				//test projective relationship between cams after converting to TT and back
//				ATensor3 tensor( 3, 3, 3 );
//				AMatrix tCam[3]; tCam[0].resize( 3, 4 ); tCam[1].resize( 3, 4 ); tCam[2].resize( 3, 4 );
//				vector< AMatrix * > tCams( 3 ); tCams[0] = &tCam[0]; tCams[1] = &tCam[1]; tCams[2] = &tCam[2];
//				vector< AMatrix * > oCams( 3 ); oCams[0] = &cams[0]; oCams[1] = &cams[1]; oCams[2] = &cams[2];
//				setIdentity( tCam[0] );
//				calculateTrifocalTensorBase3CamsT( cams[0], cams[1], cams[2], tensor );
//				cout << tensor << endl;
//				AVector2D epi1 = calculateEpipole( tensor, true );
//				AVector2D epi2 = calculateEpipole( tensor, false );
//				tCam[1] = calculateCameraMatrix( tensor, epi1, epi2, CAMERA_P1 );
//				tCam[2] = calculateCameraMatrix( tensor, epi1, epi2, CAMERA_P2 );
//				AMatrix homo = Merger::calculateHomographyFromCameras( tCams, oCams );
//				Merger::testHomography( tCams, oCams, homo );

				////test reprojection error
//				printMats( tCam[1], tCam[2] );
//				vector<AVector *> points( 3 );
//				AVector point3D( 4 );
//				AVector2D optPt( 3 );
//				LinearTriangulator triangle;
//				Point3DCorrector correct( 3 );
//				for ( int j = 0; j < 6; ++j )
//				{
//					printf( "corr %d: ", j );
//					for ( int i = 0; i < 3; ++i )
//					{
//						points[i] =  &pts6Stable[i][j];
//						cout << pts6Stable[i][j] << endl;
//					}
//
//					triangle ( tCams, points, point3D );
//					//         	correct ( tCams, points, point3D );
//
//					matMul2( *tCams[0], point3D, optPt );
//					inhomogenize<2> ( optPt );
//					double accudist = optPt.sqrDist ( *points[0] );
//
//					matMul2( *tCams[1], point3D, optPt );
//					inhomogenize<2> ( optPt );
//					accudist += optPt.sqrDist ( *points[1] );
//
//					matMul2( *tCams[2], point3D, optPt );
//					inhomogenize<2> ( optPt );
//					accudist += optPt.sqrDist ( *points[2] );
//
//					printf( "repro err: %f\n", accudist );
//				}

#endif

				return 0;
			}

			/*



			Minimal Case: 6-Point algo --- END -------------------------------------------------



			*/

//			P. 415 in HZ
			void TTensorCalculator::calculateTrifocalTensorBase3CamsT( const AMatrix &cam1, const AMatrix &cam2, const AMatrix &cam3, ATensor3 &tensor )
			{
				int sign = 1;
				for ( int i = 0; i < 3; ++i )
				{

					//i is the omitted row
					if ( i == 0 )
						camSelections( 0, Range::all() ) = cam1( 1, Range::all() );
					else
						camSelections( 0, Range::all() ) = cam1( 0, Range::all() );

					if ( i == 2 )
						camSelections( 1, Range::all() ) = cam1( 1, Range::all() );
					else
						camSelections( 1, Range::all() ) = cam1( 2, Range::all() );


					for ( int q = 0; q < 3; ++q )
					{
						camSelections( 2, Range::all() ) = cam2( q, Range::all() ); //q is chosen
						for ( int r = 0; r < 3; ++r )
						{
							camSelections( 3, Range::all() ) = cam3( r, Range::all() );//r is chosen
							//						printf("camSelections of [%d][%d][%d]\n", i, q, r);
							//						printMat(camSelections);
							//						printf("det: %f\n", detN(camSelections));
							//						pressKey();
							tensor( i, q, r ) = sign * detN( camSelections );
						}
					}

					sign *= -1;
				}

				normalizeExponents( tensor.begin(), tensor.end() );
			}

//			P. 415 in HZ
			AVector TTensorCalculator::calculateTrifocalTensorBase3CamsV( const AMatrix &cam1, const AMatrix &cam2, const AMatrix &cam3 )
			{
				AVector lineTensor( 27 );

//				printf( "calculateTrifocalTensorBase3CamsV:\n" );
//				std::cout << cam1;
//				std::cout << cam2;
//				std::cout << cam3;

				int sign = 1;
				int idx = 0;
				for ( int i = 0; i < 3; ++i )
				{

					//i is the omitted row
					if ( i == 0 )
						camSelections( 0, Range::all() ) = cam1( 1, Range::all() );
					else
						camSelections( 0, Range::all() ) = cam1( 0, Range::all() );

					if ( i == 2 )
						camSelections( 1, Range::all() ) = cam1( 1, Range::all() );
					else
						camSelections( 1, Range::all() ) = cam1( 2, Range::all() );


					for ( int q = 0; q < 3; ++q )
					{
						camSelections( 2, Range::all() ) = cam2( q, Range::all() ); //q is chosen
						for ( int r = 0; r < 3; ++r )
						{
							camSelections( 3, Range::all() ) = cam3( r, Range::all() ); //r is chosen
							lineTensor( idx++ ) = sign * detN( camSelections );
						}
					}

					sign *= -1;
				}

				//normalizeExponents(lineTensor);//OPT

				return lineTensor;
			}

//			p.377 HZ
			ATensor3 TTensorCalculator::calculateTrifocalTensorBase2CamsT( const AMatrix &cam1, const AMatrix &cam2 )
			{
				ATensor3 tensor( 3, 3, 3 );
				calculateTrifocalTensorBase2CamsT( cam1, cam2, tensor );
				return tensor;
			}

			void TTensorCalculator::calculateTrifocalTensorBase2CamsT( const AMatrix &cam1, const AMatrix &cam2, ATensor3 &tensor )
			{
				for ( int i = 0; i < 3; ++i )
				{
					for ( int j = 0; j < 3; ++j )
					{
						for ( int k = 0; k < 3; ++k )
						{
							tensor( i, j, k ) = cam1( j, i ) * cam2( k, 3 ) - cam1( j, 3 ) * cam2( k, i );
						}
					}
				}

				//tensor.normalizeExponents();//OPT
			}

//			p.377 HZ
			AVector TTensorCalculator::calculateTrifocalTensorBase2CamsV( const AMatrix &cam1, const AMatrix &cam2 )
			{
				AVector lineTensor( 27 );

				int idx = 0;
				for ( int i = 0; i < 3; ++i )
				{
					for ( int j = 0; j < 3; ++j )
					{
						for ( int k = 0; k < 3; ++k )
						{
							lineTensor( idx++ ) = cam1( j, i ) * cam2( k, 3 ) - cam1( j, 3 ) * cam2( k, i );
						}
					}
				}

				//normalizeExponents(lineTensor);//OPT
				return lineTensor;
			}

//			p.375
			AMatrix TTensorCalculator::calculateCameraMatrix( const AVector &colVector, const AVector2D &epi1, const AVector2D &epi2, int which )
			{
				T[0]( 0, 0 ) = colVector(  0 );
				T[0]( 0, 1 ) = colVector(  1 );
				T[0]( 0, 2 ) = colVector(  2 );
				T[0]( 1, 0 ) = colVector(  3 );
				T[0]( 1, 1 ) = colVector(  4 );
				T[0]( 1, 2 ) = colVector(  5 );
				T[0]( 2, 0 ) = colVector(  6 );
				T[0]( 2, 1 ) = colVector(  7 );
				T[0]( 2, 2 ) = colVector(  8 );

				T[1]( 0, 0 ) = colVector(  9 );
				T[1]( 0, 1 ) = colVector(  10 );
				T[1]( 0, 2 ) = colVector(  11 );
				T[1]( 1, 0 ) = colVector(  12 );
				T[1]( 1, 1 ) = colVector(  13 );
				T[1]( 1, 2 ) = colVector(  14 );
				T[1]( 2, 0 ) = colVector(  15 );
				T[1]( 2, 1 ) = colVector(  16 );
				T[1]( 2, 2 ) = colVector(  17 );

				T[2]( 0, 0 ) = colVector(  18 );
				T[2]( 0, 1 ) = colVector(  19 );
				T[2]( 0, 2 ) = colVector(  20 );
				T[2]( 1, 0 ) = colVector(  21 );
				T[2]( 1, 1 ) = colVector(  22 );
				T[2]( 1, 2 ) = colVector(  23 );
				T[2]( 2, 0 ) = colVector(  24 );
				T[2]( 2, 1 ) = colVector(  25 );
				T[2]( 2, 2 ) = colVector(  26 );
				return calculateCameraMatrix( T, epi1, epi2, which );
			}

			AMatrix TTensorCalculator::calculateCameraMatrix( const ATensor3 &tensor, const AVector2D &epi1, const AVector2D &epi2, int which )
			{
				T[0] = tensor( 0, Range::all(), Range::all() );
				T[1] = tensor( 1, Range::all(), Range::all() );
				T[2] = tensor( 2, Range::all(), Range::all() );
				return calculateCameraMatrix( T, epi1, epi2, which );
			}

			void TTensorCalculator::calculateCameraMatrix( const ATensor3 &tensor, const AVector2D &epi1, const AVector2D &epi2, int which, AMatrix &cam )
			{
				T[0] = tensor( 0, Range::all(), Range::all() );
				T[1] = tensor( 1, Range::all(), Range::all() );
				T[2] = tensor( 2, Range::all(), Range::all() );
				calculateCameraMatrix( T, epi1, epi2, which, cam );
			}

			AMatrix TTensorCalculator::calculateCameraMatrix( AMatrix T[3], const AVector2D &epi1, const AVector2D &epi2, int which )
			{
				AMatrix cam( 3, 4 );
				calculateCameraMatrix( T, epi1, epi2, which, cam );
				return cam;
			}

			void TTensorCalculator::calculateCameraMatrix( AMatrix T[3], const AVector2D &epi1, const AVector2D &epi2, int which, AMatrix &cam )
			{
				epipole1 = epi1;
				epipole2 = epi2;

				normalize( epipole1, 1.0 );//normalize to unit length; important!
				normalize( epipole2, 1.0 );

				AVector temp3(3);
				if ( which == CAMERA_P1 )
				{
					for ( int i = 0; i < 3; ++i )
					{
						matMul2( T[i], epipole2, temp3 );
						cam( Range::all(), i ) = temp3;
					}
					cam( Range::all(), 3 ) = epipole1;
				}
				else if ( which == CAMERA_P2 )
				{
					//this is e''e''T of Algo 15.1(iii)
					matMul2_BT( epipole2, epipole2, mat33 );
					//subtract identity
					mat33( 0, 0 ) -= 1.0;
					mat33( 1, 1 ) -= 1.0;
					mat33( 2, 2 ) -= 1.0;

					for ( int i = 0; i < 3; ++i )
					{
						matMul2_AT( T[i], epipole1, vec3 );//singulars3 is used as a temporary matrix
						matMul2( mat33, vec3, temp3 );
						cam( Range::all(), i ) = temp3;
					}
					cam( Range::all(), 3 ) = epipole2;
//					cout << cam << endl;
//					pressKey();
				}
				else
				{
					error( "wrong argument\n" );
				}
//				assert( isRightHand( cam ) );
//				normalizeExponents( cam.begin(), cam.end() );//OPT
			}

			AMatrix TTensorCalculator::calculateFundamentalMatrix( const ATensor3 &tensor, const AVector2D &epi1, const AVector2D &epi2, int which )
			{
				T[0] = tensor( 0, Range::all(), Range::all() );
				T[1] = tensor( 1, Range::all(), Range::all() );
				T[2] = tensor( 2, Range::all(), Range::all() );

				AMatrix temp(3, 3), temp2(3, 3);
				AVector temp3(3);
				if ( which == FMATRIX_F21 )
				{
					for ( int i = 0; i < 3; ++i )
					{
						matMul2( T[i], epi2, temp3 );
						temp( Range::all(), i ) = temp3;
					}
					matMul2( getCrossMat( epi1 ), temp, temp2 );
					return temp2;
				}
				else if ( which == FMATRIX_F31 )
				{
					for ( int i = 0; i < 3; ++i )
					{
						matMul2_AT( T[i], epi1, temp3 );
						temp( Range::all(), i ) = temp3;
					}
					matMul2( getCrossMat( epi2 ), temp, temp2 );
					return temp2;
				}
				else if ( which == FMATRIX_F32 )
				{
					FMatrixCalculator fcalc;
					AMatrix c2 = calculateCameraMatrix( tensor, epi1, epi2, CAMERA_P1 );
					AMatrix c3 = calculateCameraMatrix( tensor, epi1, epi2, CAMERA_P2 );
					return fcalc.calculateFundamentalMatrix2Cams( c2, c3 );
				}
				else
				{
					error( "wrong argument\n" );
				}

				//normalizeExponents(fmat);//OPT
			}

			AVector2D TTensorCalculator::getPointInThirdView( const AVector2D &pt1,
			                const AVector2D &pt2, const AMatrix &fmat21, const ATensor3 &ttensor )
			{
				assert( pt1.rows() == 3 );
				assert( pt2.rows() == 3 );
				assert( fmat21.rows() == 3 );
				assert( fmat21.cols() == 3 );

				AVector2D result( 3 );

				AVector2D perpendicular( 3 );

				AVector2D line2(3);
				matMul2( fmat21, pt1, line2 );
				perpendicular( 0 ) = line2( 1 );
				perpendicular( 1 ) = -line2( 0 );
				double third = -pt2( 0 ) * line2( 1 ) + pt2( 1 ) * line2( 0 );
				perpendicular( 2 ) =  third;
				for ( int k = 0; k < 3; ++k )
				{
					double val = 0.0;
					for ( int j = 0; j < 3; ++j )
					{
						for ( int i = 0; i < 3; ++i )
						{
							val += pt1( i ) * perpendicular( j ) * ttensor( i, j, k );
						}
					}
					result( k ) =  val;
				}
				return result;
			}

//			a debug function
			void TTensorCalculator::tauCheck( ATensor3 &tensor )
			{
				AVector colVector = buildLineTensorFromTensor( tensor );
				tauCheck( colVector );
			}

//			a debug function
			void TTensorCalculator::tauCheck( AVector &lineTensor )
			{
				Camera Kam1;
				Camera Kam2;
				vector<Camera*> cams( 2 );
				cams[0] = &Kam1;
				cams[1] = &Kam2;

				AVector2D epi1 = calculateEpipole( lineTensor, true );
				AVector2D epi2 = calculateEpipole( lineTensor, false );

				printf( "taucheck - epis\n" );
				printMats( epi1, epi2 );

//				printf( "taucheck - linetensor\n" );
//				printMat( lineTensor );

				ATensor3 tt = buildTensorFromLineTensor( lineTensor );

				printf( "taucheck - tensor\n" );
				std::cout << tt;

				Kam1.getProjMat() = calculateCameraMatrix( tt, epi1, epi2, CAMERA_P1 );
				Kam2.getProjMat() = calculateCameraMatrix( tt, epi1, epi2, CAMERA_P2 );

				printf( "Cam1:\n" );
				std::cout << Kam1.getProjMat();

				printf( "Cam2:\n" );
				std::cout << Kam2.getProjMat();

				Kam1.decompose();
				Kam2.decompose();
				AVector2D p = Kam1.getLocation();
				printf( "taucheck - Kam1: (%f, %f, %f)\n", p( 0 ), p( 1 ), p( 2 ) );
				printMats( Kam1.getCalibrationMatrix(), Kam1.getRotationMatrix() );
				p = Kam2.getLocation();
				printf( "taucheck - Kam2: (%f, %f, %f)\n", p( 0 ), p( 1 ), p( 2 ) );
				printMats( Kam2.getCalibrationMatrix(), Kam2.getRotationMatrix() );

				AVector2D lt1 = calculateTrifocalTensorBase2CamsV( Kam1.getProjMat(), Kam2.getProjMat() );

//				AMatrix id( 3, 4 ); id.setIdentity();
//				AVector2D lt2 = calculateTrifocalTensorBase3CamsV( id, Kam1.getProjMat(), Kam2.getProjMat() );

				normalize( lt1, sqrt( getSqrNorm( lineTensor ) ) );
				printMats( lineTensor, lt1 );

//				normalize( lt2, sqrt( getSqrNorm( lineTensor ) ) );
//				printMats( lineTensor, lt2 );
			}

		}
	}
