

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_PARAMETERIZATIONANGLEAXIS_H
#define ADORE_PARAMETERIZATIONANGLEAXIS_H

#include <float.h>
#include <adrcore/types/aarray.h>
#include <adrcore/util/solvesvd.h>

namespace adore
{

	namespace core
	{

		/**
		 
		*/
		class AngleAxis
		{
			public:
				AngleAxis( double midVal = 1.2 * M_PI );

				//overloaded funtions from parameterization

				AMatrix fromParam( const AVector2D &t );
				static void fromParam( const AVector &t, AMatrix &R );

				AVector2D toParam( const AMatrix &R );
				void toParam( const AMatrix &R, AVector2D &param );

				int getSize()
				{
					return 3;
				}

				AVector2D matToAngleAxis( const AMatrix &R )
				{
					return toParam( R );
				}
				void matToAngleAxis( const AMatrix &R, AVector2D &anax )
				{
					toParam( R, anax );
				}

				//p. 584
				AVector2D matToAngleAxis( const AMatrix &R, double &angle );
				void matToAngleAxis( const AMatrix &R, AVector2D &t, double &angle );

				AMatrix angleAxisToMat( const AVector2D &t )
				{
					return fromParam( t );
				}
				void angleAxisToMat( const AVector2D &t, AMatrix &R )
				{
					fromParam( t, R );
				}

				AMatrix angleAxisToMat( const double t1, const double t2, const double t3 );
				void angleAxisToMat( const double t1, const double t2, const double t3, AMatrix &R );

				bool normAngleToInterval( double &angle );

				bool normLengthToInterval( AVector &r );

				const double *getInterval()
				{
					return interval;
				}

				bool isInInterval( double d )
				{
					return ( d >= interval[0] - FLT_EPSILON ) && ( d <= interval[1] + FLT_EPSILON );
				}

				void setIntervalCentre( double centre )
				{
					interval[0] = centre - M_PI;
					interval[1] = centre + M_PI;
				}

				/**
				 * if the angel is M_PI, an arbitrary perpendicular axis is returned
				 */
				AVector vectorsToAngleAxis( AVector &vec1, AVector &vec2, double &angle );
				void vectorsToAngleAxis( AVector &vec1, AVector &vec2, double &angle, AVector &result );

			private:

				double interval[2];
				AMatrix mat;
				SVDStoraged store3;
				AVector n1, n2;//temporaries
		};

	}

}

#endif
