

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "calibrationparammixin.h"

namespace adore
{

	namespace core
	{

		double CalibrationParamMixIn::calculate44 ( const double a, const double b, const double c, const double d, const double e, const double f, const double g, const double h ) const
		{
			double divident = ( a*e - b*b ) *h*h + ( ( 2*b*c - 2*a*f ) *g + 2*b*d*f - 2*c*d*e ) *h + ( a - c*c ) *g*g + ( 2*c*d*f - 2*b*d ) *g - d*d*f*f + d*d*e;
			double divisor = a*f*f - 2*b*c*f + ( c*c - a ) *e + b*b;

//			printf( "divident: %f\n", divident );
//			printf( "divisor: %f\n", divisor );
//
			return -divident / divisor;
		}

		double CalibrationParamMixIn::calculate44 ( const double a, const double b, const double c, const double d, const double e, const double f, const double g, const double h, const double i ) const
		{
			double divident = ( a*g*g - 2*b*d*g + d*d*e ) *i + ( a*e - b*b ) *h*h + ( ( 2*b*c - 2*a*f ) *g + 2*b*d*f - 2*c*d*e ) *h - c*c*g*g + 2*c*d*f*g - d*d*f*f;
			double divisor = ( a*e - b*b ) *i - a*f*f + 2*b*c*f - c*c*e;

			return divident / divisor;
		}


		void CalibrationParamMixIn::calculateQuadric ( const double a, const double b, const double c, const double d, const double e, const double f, const double g, const double h, const double x, AMatrix &quadric ) const
		{
			quadric ( 0, 0 ) =  a;
			quadric ( 1, 1 ) =  e;
			quadric ( 2, 2 ) =  1;
			quadric ( 3, 3 ) =  x;

			quadric ( 0, 1 ) =  b;
			quadric ( 1, 0 ) =  b;

			quadric ( 0, 2 ) =  c;
			quadric ( 2, 0 ) =  c;

			quadric ( 0, 3 ) =  d;
			quadric ( 3, 0 ) =  d;

			quadric ( 1, 2 ) =  f;
			quadric ( 2, 1 ) =  f;

			quadric ( 1, 3 ) =  g;
			quadric ( 3, 1 ) =  g;

			quadric ( 2, 3 ) =  h;
			quadric ( 3, 2 ) =  h;

		}

		double CalibrationParamMixIn::calculateWij ( const int i, const int j, const double a, const double b, const double c, const double d, const double e, const double f, const double g, const double h, const double x, const AMatrix &cam ) const
		{
			double pi1 = cam ( i, 1 );
			double pi2 = cam ( i, 2 );
			double pi3 = cam ( i, 3 );
			double pi4 = cam ( i, 4 );

			double pj1 = cam ( j, 1 );
			double pj2 = cam ( j, 2 );
			double pj3 = cam ( j, 3 );
			double pj4 = cam ( j, 4 );

			double w = a*pi1*pj1 + e*pi2*pj2 + pi3*pj3 + x*pi4*pj4 +
			           b* ( pi1*pj2 + pi2*pj1 ) + c* ( pi1*pj3 + pi3*pj1 ) + d* ( pi1*pj4 + pi4*pj1 ) +
			           f* ( pi2*pj3 + pi3*pj2 ) + g* ( pi2*pj4 + pi4*pj2 ) + h* ( pi3*pj4 + pi4*pj3 );

			return w;
		}

	}
}
