

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PERSISTENCE_BOOST_FEATUREPOINT_H
#define PERSISTENCE_BOOST_FEATUREPOINT_H

#include <boost/serialization/version.hpp>

#include <adrcore/types/featurepoint.h>

#include <adrcore/persistence/persist_boost_common.h>
#include <adrcore/persistence/boost/aarray.h>
#include <adrcore/persistence/boost/featuredescriptor.h>

BOOST_CLASS_VERSION( adore::core::FeaturePoint, 2 )

namespace boost
{
	namespace serialization
	{
		using namespace adore;
		using namespace adore::core;

		template<typename Archive>
		void save( Archive & ar, const FeaturePoint& pt, unsigned int version )
		{
			ar << ( const int& )pt.getId();
			ar << pt( 0 ); ar << pt( 1 ); ar << pt( 2 );
			ar << ( const float& )pt.getStrength();
			ar << ( const float& )pt.getScale();

			bool hasAMat = pt.hasInvAffineMat();
			if ( hasAMat )
			{
				const AArray<float, 2>& invAff = pt.getInvAffineMat();
				ar << invAff;
			}
			else
			{
				AArray<float, 2> invMat( 2, 2 );
				invMat = 0, 0, 0, 0;
				ar << ( const AArray<float, 2>& )invMat;
			}

			/// descriptor(s)
			if ( OutputHelper::getOptions(&ar).storeDescriptors() )
			{
				for ( uint i = 0; i < maxDescriptorTypeNo; ++i )
				{
					const FeatureDescriptor* fd = pt.getDescriptor( i );
					if ( !fd )
					{
						ar << ( const bool& )false; continue;
					}
					else
					{
						ar << ( const bool& )true; ar << *fd;
					}
				}
			}
		}

		template<typename Archive>
		void load( Archive & ar, FeaturePoint& pt, unsigned int version )
		{
			int pid; ar >> pid; pt.setId( pid );
			ar >> pt( 0 ); ar >> pt( 1 ); ar >> pt( 2 );
			float st; ar >> st; pt.setStrength( st );
			float sc; ar >> sc; pt.setScale( sc );

			AArray<float, 2> invMat( 2, 2 );
			ar >> invMat;
			pt.getInvAffineMat() = invMat;

			if ( InputHelper::getOptions(&ar).storeDescriptors() )
			{
				for ( uint i = 0; i < maxDescriptorTypeNo; ++i )
				{
					bool has_descr; ar >> has_descr;
					if ( !has_descr ) continue;
					FeatureDescriptor* fd = new FeatureDescriptor( 0, i );
					ar >> *fd; pt.setDescriptor( fd );
				}
			}
		}

		template<typename Archive>
		inline void serialize( Archive & ar, FeaturePoint& t, const unsigned int file_version )
		{
			split_free( ar, t, file_version );
		}

	}
}

#endif // PERSISTENCE_BOOST_FEATUREPOINT_H
