

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PERSISTENCE_BOOST_IMAGEPLANE_H
#define PERSISTENCE_BOOST_IMAGEPLANE_H

#include <cassert>

#include <adrcore/types/view.h>
#include <adrcore/util/filelocator.h>

#include <adrcore/persistence/persist_boost_common.h>

namespace boost
{
	namespace serialization
	{
		using namespace adore::core;
		using namespace persist;

		template<class Archive>
		void saveImage( Archive & ar, const View<IMG_TYPE>& view )
		{
			string path = view.getImagePath();
			ar << path;
			if ( persist::embed_mark != path.substr( 0, persist::embed_mark_l ) )
				return;

			const ImagePlane<persist::IMG_TYPE>* plane = view.getImage();
			assert( plane );

			vector<unsigned char> buffer;
			bool ok = writePNG(buffer, plane->getImgArray()->imgData, plane->getNumChannels(), plane->getWidth(), plane->getHeight(), plane->getWStep());
			assert(ok);
			size_t len = buffer.size();

			len = len;
			ar << ( const vector<unsigned char>& )buffer;
		}

		template<class Archive>
		ImagePlane<persist::IMG_TYPE>* loadImage( Archive & ar, const string& path )
		{
			ImagePlane<persist::IMG_TYPE>* plane;
			if ( persist::embed_mark == path.substr( 0, persist::embed_mark_l ) )
			{
				vector<unsigned char> buffer;
				ar >> buffer;
				plane = new ImagePlane<persist::IMG_TYPE>( buffer );
			}
			else
			{
				adore::core::FileLocator locator;
				const std::string& absPath = InputHelper::getAbsPath(&ar);
				locator.addPath( absPath.c_str() );
				string p( locator.getLocation( path.c_str() ) );
				plane = new ImagePlane<persist::IMG_TYPE>( p.c_str() );
			}
			if ( !plane->getImgArray() )
			{
				delete plane;
				plane = 0;
			}
			return plane;
		}

	}
}

#endif // PERSISTENCE_BOOST_IMAGEPLANE_H
