

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PERSISTENCE_BOOST_NVIEWMAP_H
#define PERSISTENCE_BOOST_NVIEWMAP_H

#include <map>

#include <boost/serialization/split_free.hpp>

#include <adrcore/types/nviewmap.h>
#include <adrcore/types/nview.h>
#include <adrcore/types/view.h>

#include <adrcore/persistence/persist.h>
#include <adrcore/persistence/persist_boost_common.h>
#include <adrcore/persistence/boost/view.h>
#include <adrcore/persistence/boost/nview.h>

///  use apps/converter for 0 and 1
BOOST_CLASS_VERSION( adore::core::NViewMap, 2 )

namespace boost
{
	namespace serialization
	{
		using namespace adore::core;
		using namespace persist;
		using namespace std;


		template<class Archive>
		void save( Archive & ar, const NViewMap& nvmap, unsigned int version )
		{
			ar << ( const Options& )OutputHelper::getOptions(&ar);
			ar << ( const uint& )nvmap.getNumAllViews();

			NView::ViewIterator ii = nvmap.getAllViewsBegin();
			NView::ViewIterator end = nvmap.getAllViewsEnd();
			for ( ; ii != end; ++ii )
			{
				ViewBase* v = *ii;
				ar << ( const View<IMG_TYPE>& )( *v );
			}

			ar << ( const uint & )nvmap.getNumEntries();
			NViewMap::EntryIterator nvit = nvmap.getEntriesBegin();
			NViewMap::EntryIterator nvend = nvmap.getEntriesEnd();
			for ( ; nvit != nvend; ++nvit )
			{
				NView* nv = *nvit;
				ar << ( const uint & )nv->getNumViews();
				NView::ViewIterator vit = nv->getViewsBegin();
				NView::ViewIterator vend = nv->getViewsEnd();
				for ( ; vit != vend; ++vit )
				{
					ar << ( const int & )( *vit )->getId();
				}
				ar << ( const NView & )(*nv);
			}
		}

		template<class Archive>
		void load( Archive & ar, NViewMap& nvmap, unsigned int version )
		{
			ar >> InputHelper::getOptions(&ar);
			uint allViewsNum; ar >> allViewsNum;

			map<uint, ViewBase*> viewmap;
			for ( uint i = 0; i < allViewsNum; ++i )
			{
				View<IMG_TYPE>* view = new View<IMG_TYPE>;
				ar >> static_cast<View<IMG_TYPE>&>( *view );
				nvmap.addView( view );
				viewmap[view->getId()] = view;
			}

			uint nviewsNum; ar >> nviewsNum;
			for ( uint i = 0; i < nviewsNum; ++i )
			{
				vector<ViewBase*> views;
				uint viewsNum; ar >> viewsNum;
				for ( uint j = 0; j < viewsNum; ++j )
				{
					int viewId; ar >> viewId;
					views.push_back( viewmap[viewId] );
				}

				NView * nview = nvmap.createNView ( views.begin(), views.end() );
				ar >> *nview;
			}
		}

		template<class Archive>
		inline void serialize( Archive & ar, NViewMap& nvmap, const unsigned int file_version )
		{
			split_free( ar, nvmap, file_version );
		}

	}
}

#endif // PERSISTENCE_BOOST_NVIEWMAP_H
