

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef ADORE_IMGSAVER_H
#define ADORE_IMGSAVER_H

#include <string>

#include <adrcore/types/commontypes.h>
#include <adrcore/imgprocessing/imgarray.h>
#include <adrcore/imgprocessing/roi.h>
#include <adrcore/persistence/pngpersistence.h>
#include <adrcore/persistence/jpegpersistence.h>

namespace adore
{
	namespace core
	{

		template <class T>
		class ImgSaver
		{
			public:
				bool saveImgArray ( string filename, ImgArray *buffer, ROI aROI );
				bool saveImgArray ( string filename, ImgArray *buffer );
			private:
				bool extract_suffix ( string filename );
				void calcMinMax ( ImgArray *buffer, ROI aROI );
				string suffix;
				T min,max;
		};

		template <class T> bool ImgSaver<T>::extract_suffix ( string filename )
		{
			int pos;
			pos=filename.rfind( ".",string::npos );
			if ( pos==0 || pos==( int )string::npos )
			{
				cout<<"Filename not valid: "<< filename << endl;
				return false;
			};
			suffix=filename.substr( filename.rfind( ".",string::npos ),string::npos );
			return true;
		}

		template <class T> bool ImgSaver<T>::saveImgArray( string filename,ImgArray *buffer )
		{
			return saveImgArray( filename,buffer,ROI( 0,0,getImgArrayWidth( buffer ),getImgArrayHeight( buffer ) ) );
		}


		template <> inline bool ImgSaver<uint8_t>::saveImgArray( string filename,ImgArray *buffer, ROI aROI )
		{
			if ( !extract_suffix( filename ) )
			{
				return false;
			}

			if ( suffix==".jpg" || suffix==".JPG" )
			{
				return writeJPEG( filename.c_str(), buffer->imgData, buffer->nChans, buffer->width, buffer->height, buffer->widthStep, 95);
			}
			if ( suffix==".png" || suffix==".PNG" )
			{
				return writePNG( filename.c_str(), buffer->imgData, buffer->nChans, buffer->width, buffer->height, buffer->widthStep);
			}
			return false;
		}

		template <> inline bool ImgSaver<float>::saveImgArray( string filename,ImgArray *buffer, ROI aROI )
		{
			if ( !extract_suffix( filename ) )
			{
				return false;
			}

			if ( suffix==".jpg" || suffix==".JPG" )
			{
				return writeJPEG( filename.c_str(), reinterpret_cast<float*>(buffer->imgData), buffer->nChans, buffer->width, buffer->height, buffer->widthStep, 95);
			}
			if ( suffix==".png" || suffix==".PNG" )
			{
				return writePNG( filename.c_str(), reinterpret_cast<float*>(buffer->imgData), buffer->nChans, buffer->width, buffer->height, buffer->widthStep);
			}
			return false;

		}

		template <class T> void ImgSaver<T>::calcMinMax( ImgArray *buffer, ROI aROI )
		{
			min=getImgArrayData<T>( buffer )[0];
			max=getImgArrayData<T>( buffer )[0];

			uint xStart = aROI.getX();
			uint xStop  = xStart + aROI.getWidth();
			uint yStart = aROI.getY();
			uint yStop  = xStart + aROI.getHeight();

			for ( uint x=xStart;x<xStop;x++ )
				for ( uint y=yStart;y<yStop;y++ )
					for ( uint c=0;c<buffer->nChans;c++ )
					{
						T value;
						uint8_t* pointer=getImgArrayData<uint8_t>( buffer );
						pointer=pointer+y*buffer->widthStep+x*sizeof( T )*buffer->nChans+c*sizeof( T );
						memcpy( &value,pointer,sizeof( T ) );
						if ( min>value )
							min=value;
						if ( max<value )
							max=value;
					}
		}

		template <class T> bool ImgSaver<T>::saveImgArray( string filename,ImgArray *buffer, ROI aROI )
		{
			bool status = false;
			calcMinMax( buffer, aROI );
			float delta = float( max ) - float( min );
			if ( delta < 0.001 ) delta = 1.0F;

			//Do some TypeConvert....
			cout<<"Convert ImgArray to uint8_t"<<endl;
			ImgArray *img = createImgArray<uint8_t>( buffer->width, buffer->height , buffer->nChans );
			for ( uint x=0;x<buffer->width;x++ )
				for ( uint y=0;y<buffer->height;y++ )
					for ( uint c=0;c<buffer->nChans;c++ )
					{
						T value;
						uint8_t* pointer=getImgArrayData<uint8_t>( buffer );
						pointer=pointer+y*buffer->widthStep+x*sizeof( T )*buffer->nChans+c*sizeof( T );
						memcpy( &value,pointer,sizeof( T ) );

						getImgArrayData<uint8_t>( img )[y*img->widthStep+x*img->nChans+c]=
						        uint8_t( ( value - min ) * 255.0F / delta );
					}
			//Save the uint8_t ImgArray
			ImgSaver<uint8_t> mysaver;
			status=mysaver.saveImgArray( filename,img,aROI );
			releaseImgArray( img );
			return status;
		}

	}
} //ns

#endif
