

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef JPEGPERSISTENCE_H
#define JPEGPERSISTENCE_H

#include <string>
#include <vector>

#include <fstream>
#include <iostream>
#include <cstdlib>
#include <csetjmp>
#include <stdint.h>

#include <jpeglib.h>

#ifndef XMD_H
#define XMD_H
#endif

using namespace std;

class JPEGPersistence
{
public:
	typedef uint8_t jpgT;

private:
	struct error_mgr : public jpeg_error_mgr
	{
		jmp_buf setjmp_buffer;
	};

public:
	bool getInfo( const char *filename, int& width, int& height, int& channels );
	bool getInfo( const char *data, size_t size, int& width, int& height, int& channels );
	bool read( const char *filename, jpgT* buffer, int wstep );
	bool read( const char *data, size_t size, jpgT* buffer, int wstep );
	bool write( const char *filename, jpgT* buffer, int channels, int width, int height, int wstep, int quality );

private:
	FILE* prepareDecompress( const char *filename, jpeg_decompress_struct& cinfo, error_mgr& jerr );

	static void error_exit( j_common_ptr cinfo )
	{
		error_mgr* myerr = ( error_mgr* )cinfo->err;
		( *cinfo->err->output_message ) ( cinfo );
		longjmp( myerr->setjmp_buffer, 1 );
	}
};

template<typename T> inline bool readJPEG( const char *, T*, int ) { return false; }

template<> inline bool readJPEG( const char *filename, char* buffer, int wstep )
{
	JPEGPersistence reader;
	return reader.read(filename, reinterpret_cast<uint8_t*>(buffer), wstep);
}

template<typename T> inline bool readJPEG( const char *, size_t, T*, int ) { return false; }

template<> inline bool readJPEG( const char *data, size_t size, char* buffer, int wstep )
{
	JPEGPersistence reader;
	return reader.read(data, size, reinterpret_cast<uint8_t*>(buffer), wstep);
}

template<typename T> inline bool writeJPEG( const char *, T *, int, int, int, int, int ) { return false; }

template<> inline bool writeJPEG( const char * filename, char * buffer, int channels, int width, int height, int wstep, int quality )
{
	JPEGPersistence writer;
	return writer.write(filename, reinterpret_cast<uint8_t*>(buffer), channels, width, height, wstep, quality);
}

#endif

