

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "pngpersistence.h"

bool PNGPersistence::checkfile ( FILE *fp )
{
    png_byte header[8];
    int numRead = fread ( header, 1, 8, fp );
    if ( numRead != 8 )
    {
        fprintf( stderr, "PNGPersistence failed to read Header\n" );
    }

    bool is_png = !png_sig_cmp ( header, 0, 8 );
    if ( !is_png )
    {
        //                 fprintf( stderr, "PNGPersistence: Error! Not a PNG File!\n" );
        return false;
    }
    return is_png;
}

bool PNGPersistence::initInfoStructs()
{
    png_ptr = png_create_read_struct ( PNG_LIBPNG_VER_STRING, ( png_voidp ) NULL, NULL, NULL );
    if ( !png_ptr )
    {
        fprintf( stderr, "PNGPersistence: Alloc of png read struct failed!\n" );
        return false;
    }
    info_ptr = png_create_info_struct ( png_ptr );
    if ( !info_ptr )
    {
        fprintf( stderr, "PNGPersistence: Alloc of png info struct failed!\n" );
        png_destroy_read_struct ( &png_ptr, ( png_infopp ) NULL, ( png_infopp ) NULL );
        return false;
    }

    end_info = png_create_info_struct ( png_ptr );
    if ( !end_info )
    {
        fprintf( stderr, "PNGPersistence: Alloc of png end info failed!\n" );
        png_destroy_read_struct ( &png_ptr, &info_ptr, ( png_infopp ) NULL );
        return false;
    }
    return true;
}
//strips alpha
bool PNGPersistence::read ( const char *filename, pngT *buffer, int wstep )
{
    FILE *fp = fopen ( filename, "rb" );
    if ( !fp )
    {
        fprintf ( stderr, "PNGPersistence: can't open %s\n", filename );
        return false;
    }

    if ( !checkfile ( fp ) )
    {
        fclose ( fp );
        return false;
    }

    fprintf( stderr, "PNGPersistence: PNG-File valid! Getting info ...\n" );
    if ( !initInfoStructs() )
    {
        fclose ( fp );
        return false;
    }

    png_init_io ( png_ptr, fp );
    png_set_sig_bytes ( png_ptr, 8 );

    if ( setjmp ( png_jmpbuf ( png_ptr ) ) )
    {
        png_destroy_read_struct ( &png_ptr, &info_ptr, ( png_infopp ) NULL );
        fprintf( stderr, "PNGPersistence: PNG:something failed ...\n" );
        fclose ( fp );
        return false;
    }

    png_read_png ( png_ptr, info_ptr, PNG_TRANSFORM_PACKING | PNG_TRANSFORM_STRIP_16 | PNG_TRANSFORM_STRIP_ALPHA, NULL );

    png_bytep* row_pointers = png_get_rows ( png_ptr, info_ptr );

    png_uint_32 width, height;
    int bit_depth, color_type, interlace_type, compression_type, filter_method;
    png_get_IHDR ( png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, &compression_type, &filter_method );

    int channels = png_get_channels ( png_ptr, info_ptr );

    for ( int i=0; i < ( int ) height; i++, buffer+=wstep )
    {
        memcpy ( buffer, row_pointers[i], width*channels );
    }

    //png_read_end(png_ptr, end_info);
    png_destroy_read_struct ( &png_ptr, &info_ptr, &end_info );
    fclose ( fp );

    return true;
}

bool PNGPersistence::getInfo ( const char* filename, int &width, int &height, int &channels )
{
    FILE *fp = fopen ( filename, "rb" );
    if ( !fp )
    {
        fprintf ( stderr, "PNGPersistence: can't open %s\n", filename );
        return false;
    }

    if ( !checkfile ( fp ) )
    {
        fclose ( fp );
        return false;
    }

    //         fprintf( stderr, "PNGPersistence: PNG-File valid! Getting info ...\n" );
    if ( !initInfoStructs() )
    {
        fclose ( fp );
        return false;
    }

    png_init_io ( png_ptr, fp );
    png_set_sig_bytes ( png_ptr, 8 );

    if ( setjmp ( png_jmpbuf ( png_ptr ) ) )
    {
        png_destroy_read_struct ( &png_ptr, &info_ptr, ( png_infopp ) NULL );
        fprintf( stderr, "PNGPersistence: PNG:something failed....\n" );
        fclose ( fp );
        return false;
    }
    png_read_info ( png_ptr, info_ptr );
    channels=png_get_channels ( png_ptr, info_ptr );

    //wegen PNG_TRANSFORM_STRIP_ALPHA in PNGPersistence::read
    if(channels == 2) channels = 1;
    if(channels == 4) channels = 3;

    width  = png_get_image_width ( png_ptr, info_ptr );
    height = png_get_image_height ( png_ptr, info_ptr );
    png_destroy_read_struct ( &png_ptr, &info_ptr, &end_info );

    fclose ( fp );
    return true;
}

struct png_mem_in
{
	png_mem_in ( const uint8_t* s ) : src ( s ), pos ( 0 ) {}
	const uint8_t* src;
	unsigned int pos;
};

static void _png_read_data ( png_structp png_ptr, png_bytep data, png_size_t length )
{
	png_mem_in* src = static_cast<png_mem_in*> ( png_get_io_ptr ( png_ptr ) );
	for ( png_size_t i = 0; i < length; ++i )
		data[i] = src->src[src->pos + i];
	src->pos += length;
}

bool PNGPersistence::read ( const uint8_t * data, size_t size, pngT *buffer, int wstep )
{
	if ( !initInfoStructs() || size < 10 )
		return false;

	png_mem_in in ( data );
	png_set_read_fn ( png_ptr, ( png_voidp ) &in, _png_read_data );

	if ( setjmp ( png_jmpbuf ( png_ptr ) ) )
	{
		png_destroy_read_struct ( &png_ptr, &info_ptr, ( png_infopp ) NULL );
		fprintf( stderr, "PNGPersistence: PNG:something failed ...\n" );
		return false;
	}

	png_read_png ( png_ptr, info_ptr,
				   PNG_TRANSFORM_PACKING | PNG_TRANSFORM_STRIP_16 | PNG_TRANSFORM_STRIP_ALPHA
               #ifdef USE_OPENCV
				   | PNG_TRANSFORM_BGR
               #endif
				   , NULL );

	//do some type specific conversions
	png_uint_32 width, height;
	int bit_depth, color_type, interlace_type, compression_type, filter_method;
	png_bytep* row_pointers = png_get_rows ( png_ptr, info_ptr );
	png_get_IHDR ( png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, &compression_type, &filter_method );
	int channels = png_get_channels ( png_ptr, info_ptr );

	for ( int i = 0; i < ( int ) height;i++ )
	{
		memcpy ( buffer +  i * wstep, row_pointers[i], width * channels );
	}

	//png_read_end(png_ptr, end_info);
	png_destroy_read_struct ( &png_ptr, &info_ptr, &end_info );

	return true;
}

bool PNGPersistence::getInfo( const uint8_t *data, size_t size, int& width, int& height, int& channels )
{
	if ( !initInfoStructs() || size < 10 )
		return false;

	png_mem_in in ( data );
	png_set_read_fn ( png_ptr, ( png_voidp ) &in, _png_read_data );

	if ( setjmp ( png_jmpbuf ( png_ptr ) ) )
	{
		fprintf( stderr, "PNGPersistence: PNG:something failed....\n" );
		png_destroy_read_struct ( &png_ptr, &info_ptr, ( png_infopp ) NULL );
		return false;
	}

	png_read_info ( png_ptr, info_ptr );
	channels=png_get_channels ( png_ptr, info_ptr );
	width=png_get_image_width ( png_ptr, info_ptr );
	height=png_get_image_height ( png_ptr, info_ptr );

	png_destroy_read_struct ( &png_ptr, &info_ptr, &end_info );

	return true;
}

//save
bool PNGPersistence::write( const char* filename, pngT * buffer, int nChannels, int width, int height, int wstep)
{
    int bit_depth, color_type, interlace_type, compression_type, filter_method;

    switch(nChannels)
    {
    case 1: color_type=PNG_COLOR_TYPE_GRAY;
        break;
    case 2: color_type=PNG_COLOR_TYPE_GRAY_ALPHA;
        fprintf(stderr, "PNGPersistence: color type PNG_COLOR_TYPE_GRAY_ALPHA\n");
        break;
    case 3: color_type=PNG_COLOR_TYPE_RGB;
        break;
    case 4: color_type=PNG_COLOR_TYPE_RGB_ALPHA;
        fprintf(stderr, "PNGPersistence: color type PNG_COLOR_TYPE_RGB_ALPHA\n");
        break;
    default: fprintf(stderr, "PNGPersistence: Unsupported number of channels (%d)Can't write png image!\n", nChannels);
        return false;
    }

    FILE *fp = fopen( filename, "wb" );
    if ( !fp )
    {
        fprintf(stderr, "PNGPersistence: error opening file for writing");
        return false;
    };

    png_ptr = png_create_write_struct ( PNG_LIBPNG_VER_STRING, ( png_voidp )NULL,NULL,NULL );
    if ( !png_ptr ) return false;

    info_ptr = png_create_info_struct( png_ptr );
    if ( !info_ptr )
    {
        png_destroy_write_struct( &png_ptr,( png_infopp )NULL );
        return false;
    }

    if ( setjmp( png_jmpbuf( png_ptr ) ) )
    {
        png_destroy_write_struct( &png_ptr, &info_ptr );
        fclose( fp );
        return false;
    }

    bit_depth=8;

    png_bytep row_pointers[height];

    interlace_type=PNG_INTERLACE_NONE;
    compression_type=PNG_COMPRESSION_TYPE_DEFAULT;
    filter_method=PNG_FILTER_TYPE_DEFAULT;

    png_set_IHDR( png_ptr, info_ptr, width, height, bit_depth, color_type, interlace_type, compression_type, filter_method );
    png_init_io( png_ptr, fp );

    for ( int y=0; y < height; y++, buffer+=wstep )
    {
        row_pointers[y] = buffer;
    }

    png_set_rows( png_ptr, info_ptr, row_pointers );
    png_write_png( png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL );
    png_destroy_write_struct( &png_ptr, &info_ptr );
    fclose( fp );
    return true;
}

static void _png_write_data( png_structp png_ptr, png_bytep data, png_size_t length )
{
	vector<unsigned char>* dest = static_cast<vector<unsigned char>*>( png_get_io_ptr( png_ptr ) );
	for ( png_size_t i = 0; i < length; ++i )
		dest->push_back( *data++ );
}

static void _png_flush_data( png_structp )
{
}

bool PNGPersistence::write( vector<uint8_t>& output, pngT * buffer, int nChannels, int width, int height, int wstep)
{
	int color_type;
	if ( nChannels == 1 ) color_type = PNG_COLOR_TYPE_GRAY;
	else if ( nChannels == 3 ) color_type = PNG_COLOR_TYPE_RGB;
	else
	{
		fprintf(stderr, "PNGPersistence: Unsupported number of channels (%d). Can't write png image!", nChannels);
		return false;
	}

	png_ptr = png_create_write_struct ( PNG_LIBPNG_VER_STRING, ( png_voidp )NULL, NULL, NULL );
	if ( !png_ptr ) return false;

	info_ptr = png_create_info_struct( png_ptr );
	if ( !info_ptr )
	{
		png_destroy_write_struct( &png_ptr,( png_infopp )NULL );
		return false;
	}

	if ( setjmp( png_jmpbuf( png_ptr ) ) )
	{
		png_destroy_write_struct( &png_ptr, &info_ptr );
		return 0;
	}

	int bit_depth = 8;

	png_bytep * row_pointers = new png_bytep[height];

	int interlace_type = PNG_INTERLACE_NONE;
	int compression_type = PNG_COMPRESSION_TYPE_DEFAULT;
	int filter_method = PNG_FILTER_TYPE_DEFAULT;

    png_set_IHDR( png_ptr, info_ptr, width, height, bit_depth, color_type, interlace_type, compression_type, filter_method );

	output.clear();
	png_set_write_fn( png_ptr, ( png_voidp ) &output, _png_write_data, _png_flush_data );

    for ( uint y = 0; y < height; y++ )
        row_pointers[y] = buffer + y * wstep;

    png_set_rows( png_ptr, info_ptr, row_pointers );
	png_write_png( png_ptr, info_ptr,
#ifdef USE_OPENCV
                   PNG_TRANSFORM_BGR
#else
				   PNG_TRANSFORM_IDENTITY
#endif
				   , NULL );
	png_destroy_write_struct( &png_ptr, &info_ptr );
	delete [] row_pointers;

	return true;
}


