

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_RANDOMRANDOM_H
#define ADORE_RANDOMRANDOM_H
#include <adrcore/util/thread.h>
#include <boost/random.hpp>
namespace adore
{
	namespace core
	{
		using namespace std;
		const int ADR_RAND_MAX = RAND_MAX;
		class RandomProvider
		{
			public:
				typedef boost::mt19937 RandomGeneratorT;
				RandomGeneratorT generator;

				boost::uniform_real<> uni_real;
				boost::variate_generator<RandomGeneratorT&, boost::uniform_real<> > genUniReal;

				boost::uniform_int<> uni_int;
				boost::variate_generator<RandomGeneratorT&, boost::uniform_int<>  > genUniInt;

				RandomProvider():generator( 42u ), uni_real( 0,1 ), genUniReal( generator, uni_real ), uni_int( 0, ADR_RAND_MAX ), genUniInt( generator, uni_int ) {}
		};

		extern RandomProvider dRP;

		class RandomSafe
		{
			private:
				struct elem
				{
					pthread_t id;
					RandomProvider *rP;
				};
				unsigned int seed;
				vector<elem *> threads;
				Mutex l;
			public:
				RandomSafe()
				{
					dRP.generator.seed( static_cast<unsigned int>( std::time( 0 ) ) );
				};
				~RandomSafe();
				RandomProvider* provider();
				void setSeed( unsigned int s )
				{
					provider()->generator.seed( s );
				}
				void setGlobalSeed( unsigned int s )
				{
					dRP.generator.seed( s );
					seed=s;
				}
				void delThread();
		};

		extern RandomSafe sR;
		/**
		 *	a random number between 0 and 1 (inclusive)
		 */
		double dblRand( RandomProvider &rp = dRP );

		/**
		 *	a random number between min and max
		 */
		double dblRand( double min, double max, RandomProvider &rp = dRP );

		/**
		 * a random number between 0 and ADR_RANDOM_MAX
		 */
		int intRand( RandomProvider &rp = dRP );

		/**
		 *	a random number between min and max (inclusive)
		 */
		int intRand( int min, int max, RandomProvider &rp = dRP );

		void setSeed( RandomProvider &rp = dRP );

		void setSeed( unsigned int s, RandomProvider &rp = dRP );
		void setGlobalSeed( unsigned int s );
		void cleaner( void* c );

		void setRandomSeedCallCount(const int c);
		
		unsigned int getTimeSeed();
	}
}
#endif
