

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef INTEGRALIMAGE_H
#define INTEGRALIMAGE_H

#include <adrcore/imgprocessing/imageplane.h>
#include <adrcore/util/calctype.h>

#include <algorithm>
#include <limits>

#include <boost/scoped_array.hpp>

namespace adore
{
	namespace core
	{

		template <typename dstT>
		class IntegralImage
		{
			public:
				typedef uint8_t imgT;
				typedef ImagePlane<imgT> ImageT;

			private:
				typedef typename signedT<typename calcT<dstT>::type>::type dst;
				boost::scoped_array<dst> data;
				uint height, width;
				static const imgT imgTmax;

			public:

				IntegralImage( ImageT* image = NULL );
				~IntegralImage() { }
				void reset() { data.reset(); }

				void init(ImageT* image);
				
				int getWidth() const { return width; }
				int getHeight() const { return height; }

				dst getSum( uint x, uint y, uint w, uint h ) const
				{
					assert_msg(x + w <= width, "IntegralImage: x:%d, w:%d, width:%d", x, w, width);
					assert_msg(y + h <= height, "IntegralImage: y:%d, h:%d, height:%d", y, h, height);

					if(x > 0 && y > 0)//common case first
					{
						const uint x1 = x - 1, y1w = (y - 1) * width;
						const uint x2 = x1 + w, y2w = (y + h - 1) * width;
						const dst A = data[y1w + x1];
						const dst B = data[y1w + x2];
						const dst C = data[y2w + x1];
						const dst D = data[y2w + x2];
						// A---B
						// |   |
						// C---D
						return A + D - C - B;
					}
					else if(x > 0)
					{
						const uint x1 = x - 1, y2w = (h - 1) * width;
						return data[y2w + x1 + w] - data[y2w + x1];
					}
					else if(y > 0)
					{
						const uint y1 = y - 1, w1 = w - 1;
						return data[(y1 + h) * width + w1] - data[y1 * width + w1];
					}
					else
					{
						return data[ (h - 1) * width + (w - 1) ];
					}
					
					assert(false);
					return dst(0);
				}
			private:
				void compute( ImageT* image );
		};

	} // core
} // adore

#endif // INTEGRALIMAGE_H
