

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef SURFCOMMON_H
#define SURFCOMMON_H

#include "integralimage.h"
#include <adrcore/types/view.h>
#include <adrcore/imgprocessing/rgbtogray.h>
#include <adrcore/imgprocessing/upsampler.h>

namespace adore
{
	namespace core
	{

		template<typename T> class IntegralImage;

		typedef int SurfIntegralStoreT;
		typedef IntegralImage<SurfIntegralStoreT> SurfIntegralImageT;

		class SurfImageData
		{
			private:
				typedef uint8_t imgT;
				typedef ImagePlane<imgT> ImageT;
				typedef View<imgT> ViewT;
				
				SurfIntegralImageT iimage;
				SurfIntegralImageT diimage;//doubled
				
				ImageT *img;
				ImageT gray;
				ImageT doubled;
				
				UpSamplerSurf<imgT> upsampler;
				RGBtoGray<imgT> convgray;
			public:
				SurfImageData(ViewT *view, bool doDoubled)
				{
					img = view->getImage();
					int width = img->getWidth();
					int height = img->getHeight();
					if ( img->getNumChannels() == 3 )
					{
						gray.init( width, height, 1 );
						convgray.process( img, &gray );
						img = &gray;
					}
					iimage.init(img);
					
					if ( doDoubled )
					{
						doubled.init( width * 2, height * 2, 1 );
						upsampler.process(img, &doubled);
						img = &doubled;
					}
					diimage.init(img);
				}
		};

	} // core
} // adore

#endif // SURFCOMMON_H
