

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef SURFFINDER_H
#define SURFFINDER_H

#include "surfcommon.h"

#include <adrcore/viewmatch/featurefinder.h>
#include <adrcore/types/view.h>
#include <adrcore/features/featurefilter.h>
#include <adrcore/imgprocessing/rgbtogray.h>
#include <adrcore/imgprocessing/upsampler.h>
#include <boost/array.hpp>

namespace adore
{
	namespace core
	{

		class SurfDescriptor;

		class SurfFinder : public FeatureFinder
		{
			private:
				typedef ImagePlane<imgT> ImageT;
				typedef float DetStoreT;
				typedef float workT;
				typedef ImagePlane<DetStoreT> DetImg;
				typedef boost::array<DetImg*, 3> DetImages;

				enum 
				{ 
					MaxOctaves = 4, 
					MaxIntervals = 4 
				};
				
				int filter_sizes[MaxOctaves][MaxIntervals];
				
				uint id;
				uint m_octaves, m_intervals;
				double m_threshold;
				bool m_doubleSize;
				int maxPts;
				float minDist;
				float minFraction;

				SurfDescriptor* descriptorCreator;
				DescriptorCreator* otherDecrCreator;
				bool ownDescrCreator;
				FeatureGrid candidates;

				mutable AArray<DetStoreT, 3> store;

				ViewT* view;
				int height, width;
				int offX, offY;
				uint num_feats;
				
				// image processing variables
				RGBtoGray<imgT> convgray;
				ImageT doubled;//image of double size as input image
				ImageT gray;//buffer for grayscale version if necessary
				SurfIntegralImageT integralImage;
				UpSampler<imgT> upsampler;
			public:
				/**
				 * @param intervals images per octave
				 */
				SurfFinder( uint id = 0, uint octaves = 1, uint intervals = 3, double threshold = 0.001,
				            bool doubleSize = false, int maxP = 1000, float minD = 2.0f, float minF = 0.000001f );

				SurfFinder( const SurfFinder& rhs );
				FeatureFinder* clone();
				~SurfFinder();

				virtual void setDescriptorCreator ( DescriptorCreator* theValue, bool own = false );

			private:
				void initFilterSizes();
				
				uint findFeaturesDerived ( ViewT * v );

				void fastHessian( const SurfIntegralImageT& image, uint filter_sz, DetImg& det ) const;
				template<typename Arg2NotExtremum> void findExtrema( double thresh, DetImages& dets,
				                int cur_oct, int cur_int, const SurfIntegralImageT& image );

				bool subCoorQuadratic( workT & dx, workT & dy ) const;
				bool goodContrast() const;
				bool notAtEdge( int cur_oct, int cur_int, int x, int y, const SurfIntegralImageT& image ) const;
				void deriv3D ( AVector& b ) const;
		};

	} // core
} // adore

#endif // SURFFINDER_H
