

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORRESPONDENCE_H
#define ADORE_CORRESPONDENCE_H

#include <adrcore/types/feature.h>
#include <adrcore/types/feature.h>
#include <adrcore/types/featureline.h>
#include <adrcore/util/debug.h>

#include <set>

namespace adore
{

	namespace core
	{

		using namespace std;

		template <typename T> class Correspondence
		{
				friend class NView;

				T dummy; //for searching purposes

				set<T *, FeatureViewAmbiComparator<> > corr;

				float badness;

			public:

				typedef typename set<T *, FeatureViewAmbiComparator<> >::iterator Iterator;
				typedef Iterator iterator;

				Iterator getBegin()
				{
					return corr.begin();
				}

				Iterator begin()
				{
					return corr.begin();
				}

				Iterator getEnd()
				{
					return corr.end();
				}

				Iterator end()
				{
					return corr.end();
				}

				T * getFeature( ViewBase *v )
				{
					if ( !v ) return NULL;
					dummy.setView( v );
					Iterator it = corr.find( &dummy );
					if ( it == corr.end() ) return NULL;
					return *it;
				}

				uint getSize()
				{
					return corr.size();
				}

				uint size()
				{
					return corr.size();
				}

				void setBadness ( float theValue )
				{
					badness = theValue;
				}

				float getBadness() const
				{
					return badness;
				}

				std::string toDbgString()
				{
					std::ostringstream oss;
					oss << "Corr-badness: " << setw( 2 ) << badness << ", points: ";
					for ( Iterator it = begin(); it != end(); ++it )
					{
						oss << adore::core::toDbgString( *it ) << " ";
					}
					return oss.str();
				}

			private:

				Correspondence() : dummy(), badness( 0 ) {}
				~Correspondence() {}

				void addFeature( T * f )
				{
					corr.insert( f );
				}

				void removeFeature( T * f )
				{
					Iterator it = corr.find( f );
					if ( it != corr.end() ) corr.erase( it );
				}

		};

		typedef Correspondence<FeatureLine> LineCorr;
	}

}

#endif // ADORE_CORRESPONDENCE_H
