

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_FEATURESFEATUREDESCRIPTOR_H
#define ADORE_FEATURESFEATUREDESCRIPTOR_H

#include <adrcore/types/aarray.h>
#include <adrcore/types/featuretypes.h>
#include <adrcore/util/matmul2.h>
#include <vector>
#include <float.h>

namespace adore
{

	namespace core
	{

		class Feature;

		/**
		     
		 * A Featuredescriptor consists of several Vectors.
		 * Additionally, it has an id that may identify it.
		*/
		class FeatureDescriptor
		{
			public:

				typedef float T_numtype;
				typedef AArray<T_numtype, 1> DescVecT;

			private:

				typedef std::vector< DescVecT > CollectionT;
				typedef CollectionT::iterator iterator;

			public:

				/**
				 * Creates a descriptor. if size > 0, a Vector of size size is allocated.
				 * The id can be used to mark the type of the descriptor
				 */
				FeatureDescriptor ( int size = 0, uint _id = 0 ) :id ( _id )
				{if ( size > 0 ) addVector ( size );}

				virtual ~FeatureDescriptor() {;}

				T_numtype differenceTo ( FeatureDescriptor &other )
				{
					T_numtype minVal = FLT_MAX;
					for ( uint o = 0; o < other.size(); ++o )
					{
						const DescVecT& oVec = other ( o );
						for ( uint m = 0; m < size(); ++m )
						{
							DescVecT diff = oVec - descs[m];//OK6
							T_numtype val = dot2 ( diff, diff );
							if ( val < minVal )
							{
								minVal = val;
							}
						}
					}
					return minVal;
				}

				DescVecT &addVector ( int size )
				{
					descs.push_back ( DescVecT ( size ) );
					return descs[descs.size() - 1];
				}

				void addVector ( const DescVecT & vec )
				{
					descs.push_back ( vec );
				}

				T_numtype &operator() ( int idx1, int idx2 )
				{
					return descs[idx1] ( idx2 );
				}

				DescVecT &operator() ( int idx )
				{
					return descs[idx];
				}

				DescVecT &operator[] ( int idx )
				{
					return descs[idx];
				}

				const DescVecT& operator[] ( int idx ) const
				{
					return descs[idx];
				}

				uint size() const
				{
					return descs.size();
				}

				iterator begin()
				{
					return descs.begin();
				}

				iterator end()
				{
					return descs.end();
				}

				void setId ( const int _id )
				{
					id = _id;
				}

				uint getId() const
				{
					return id;
				}

			protected:

				CollectionT descs;
				uint id;
		};

	}

}

#endif
