

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREAHEAPARRAY_H
#define ADORE_COREAHEAPARRAY_H

namespace adore
{

	namespace core
	{

		/**
		 * a simple array allocated on the head.
			 
		*/
		template<class T>
		class HeapArray
		{
			public:
				typedef T* iterator;
				typedef const T* const_iterator;
			public:
				HeapArray():
						_data( NULL ), _begin( NULL ), _end( NULL )
				{;}

				HeapArray( uint size ):
						_data( new T[size] ), _begin( &_data[0] ), _end( _begin + size )
				{;}

				~HeapArray() {delete [] _data;}

				const HeapArray& operator=( const HeapArray& ha )
				{
					if ( size() != ha.size() )
					{
						delete [] _data;
						_data = new T[ha.size()];
						_begin = &_data[0];
						_end = _begin + ha.size();
					}
					iterator dest = begin();
					for ( const_iterator it = ha.begin(); it != ha.end(); ++it, ++dest )
					{
						*dest = *it;
					}
					return *this;
				}

				bool operator<( const HeapArray &ha ) const
				{
					const_iterator hait = ha.begin();
					for ( const_iterator it = begin(); it != end(); ++it, ++hait )
					{
						if ( *it == *hait ) continue;
						else if ( *it < *hait ) return true;
						else return false;
					}
					return false;
				}

				T &operator[]( int idx )
				{
					return *( _begin + idx );
				}

				T *begin()
				{
					return _begin;
				}

				T *end()
				{
					return _end;
				}

				const T *begin() const
				{
					return _begin;
				}

				const T *end() const
				{
					return _end;
				}


				uint size() const
				{
					return _end - _begin;
				}

				void init( uint size )
				{
					delete [] _data;
					_data  = new T[size];
					_begin = &_data[0];
					_end   = _begin + size;
				}

			private:
				T* _data;
				T* _begin;
				T* _end;
		};

	}

}

#endif
