

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_TYPESRECT_H
#define ADORE_TYPESRECT_H

#include <adrcore/util/mathutil.h>

namespace adore
{

	namespace core
	{

		/**
			 
		*/
		template <class T>
		class Rect
		{
			public:
				Rect( T _x = 0, T _y = 0, T _width = 0, T _height = 0 )
				{
					x = _x;
					y = _y;
					width = _width;
					height = _height;
				}

				Rect( const Rect &r )
						:x( r.getX() ),
						y( r.getY() ),
						width( r.getWidth() ),
						height( r.getHeight() )
				{
				}

				~Rect()
				{
				}

				T getX() const
				{
					return x;
				}
				T getY() const
				{
					return y;
				}
				T getWidth() const
				{
					return width;
				}
				T getHeight() const
				{
					return height;
				}
				
				void scale(float xScale, float yScale)
				{
					x *= xScale;
					width *= xScale;
					y *= yScale;
					height *= yScale;
				}
				
				T getArea() const
				{
					return getWidth() * getHeight();
				}

				void setX( const T x_ )
				{
					x = x_;
				}
				void setY( const T y_ )
				{
					y = y_;
				}
				void setWidth( const T width_ )
				{
					width = width_;
				}
				void setHeight( const T height_ )
				{
					height = height_;
				}

				void set( const T _x, const T _y, const T _w, const T _h )
				{
					x = _x;
					y = _y;
					width = _w;
					height = _h;
				}

				/**
					expand rect to include px, py
				*/
				void include( const T px, const T py )
				{
					if ( px < x ) {width += x - px; x = px;}
					if ( py < y ) {height += y - py; y = py;}
					if ( px > x + width - 1 ) {width = px - x + 1;}
					if ( py > y + height - 1 ) {height = py - y + 1;}
				}
				
				bool contains( const T px, const T py, const T pz) const
				{
					if(tabs<T>(pz) < T(FLT_EPSILON)) return false;
					return contains(px / pz, py / pz);
				}
				
				bool contains( const T px, const T py ) const
				{
					return ( px >= x ) && ( py >= y ) && ( px <= x + width - 1 ) && ( py <= y + height - 1 );
				}
				
				void writeAt( std::ostream& stream ) const
                                {
                                        stream << "(" << x << ", " << y << ")[" << width << ", " << height << "]";
                                }
                                
                                void getCenter(AVector &center)
                                {
					center(0) = double(x) + 0.5 * width;
					center(1) = double(y) + 0.5 * height;
				}
				
				double getCenterX()
                                {
					return double(x) + 0.5 * width;
				}
				
				double getCenterY()
                                {
					return double(y) + 0.5 * height;
				}


			private:

				T x;
				T y;
				T width;
				T height;
		};
		
		template <class T>
		inline std::ostream& operator<<( std::ostream& stream, Rect<T> const& r )
                {
                        r.writeAt( stream );
                        return stream;
                }
                
                template <class IteratorT >
		Rect<double> getBoundingBox( const IteratorT &begin, const int numPoints, bool homogeneousInput = true )
		{
			if ( numPoints <= 0 ) return Rect<double>();

                        typedef typename Ptr<typename std::iterator_traits<IteratorT>::value_type>::type valBaseT;

			double minX=DBL_MAX;
			double maxX=-DBL_MAX;
			double minY=DBL_MAX;
			double maxY=-DBL_MAX;

			IteratorT ii = begin;
			ToPointer<valBaseT> toPointer;
			if ( homogeneousInput )
			{
				for ( int i = 0; i < numPoints; ++i, ++ii )
				{
					valBaseT pt = toPointer( *ii );
					if ( !pt ) continue;

					double w = ( *pt )( 2 );
					double x = ( *pt )( 0 ) / w;
					double y = ( *pt )( 1 ) / w;
					minX = MIN( minX, x );
					maxX = MAX( maxX, x );
					minY = MIN( minY, y );
					maxY = MAX( maxY, y );
				}
			}
			else
			{
				for ( int i = 0; i < numPoints; ++i, ++ii )
				{
					valBaseT pt = toPointer( *ii );
					if ( !pt ) continue;

					double x = ( *pt )( 0 );
					double y = ( *pt )( 1 );
					minX = MIN( minX, x );
					maxX = MAX( maxX, x );
					minY = MIN( minY, y );
					maxY = MAX( maxY, y );
				}
			}

//			printf( "Rect: %f, %f, %f, %f\n", minX, minY, maxX, maxY );

			return Rect<double>( minX, minY, maxX - minX, maxY - minY );
		}

		
	}

}

#endif
