

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "viewrelations.h"

namespace adore
{

	namespace core
	{

		ViewRelations::ViewRelations() :distances ( 20, 20 )
		{
			distances = -1;
		}

		uint ViewRelations::addView( ViewBase *v )
		{
			IndicesIterT iter = indices.find( v );
			if ( iter != indices.end() ) return ( *iter ).second;//already known
			indices[v] = views.size();
			views.push_back( v );
			return views.size() - 1;
		}

		ViewBase *ViewRelations::getView( uint idx )
		{
			if ( idx >= views.size() ) return NULL;
			return views[idx];
		}

		void ViewRelations::replaceView( uint idx, ViewBase *vNew )
		{
			if ( idx >= views.size() ) return;
			ViewBase *vOld = views[idx];
			views[idx] = vNew;
			IndicesIterT iter = indices.find( vOld );
			if ( iter != indices.end() )
			{
				indices.erase( iter );
			}
			indices[vNew] = idx;
		}

		float ViewRelations::getDistance ( uint vidx1, uint vidx2 )
		{
			if ( vidx1 < distances.rows() && vidx1 < distances.rows() )
			{
				return distances ( vidx1, vidx2 );
			}
			return -1.0f;
		}

		void ViewRelations::setDistance ( uint vidx1, uint vidx2, float d )
		{
			uint oldsize = distances.rows();
			if ( vidx1 > oldsize || vidx1 > oldsize )
			{
				//recreate
				int nusize = MAX ( vidx1, vidx2 ) << 1;
				AMatrixf mat ( nusize, nusize );
				mat = -1;
				mat ( Range ( 0, oldsize-1 ), Range ( 0, oldsize-1 ) ) = distances;
                distances.resize( nusize, nusize );
                distances = mat;
			}
			distances ( ( int )vidx1, ( int )vidx2 ) = distances ( ( int )vidx2, ( int )vidx1 ) = d;
		}

		void ViewRelations::adoptDistances ( AMatrixf &distMat )
		{
			assert ( distMat.rows() == distMat.cols() );
                        assert ( distMat.rows() == views.size() );

			distances = distMat;
		}

		bool ViewRelations::areNeighbors ( uint vidx1, uint vidx2 )
		{
			if ( neighbors.size() >= vidx1 ) return false;
			for ( uint i = 0; i < neighbors[vidx1].size(); ++i ) if ( neighbors[vidx1][i] == vidx2 ) return true;
			return false;
		}

		void ViewRelations::setNeighbors ( uint vidx1, uint vidx2, bool n )
		{
			if ( n == areNeighbors ( vidx1, vidx2 ) ) return;
			else
			{
				if ( n )
				{
					//set
					setNeighbors ( vidx1, vidx2 );
					setNeighbors ( vidx2, vidx1 );
				}
				else
				{
					//remove
					unsetNeighbors ( vidx1, vidx2 );
					unsetNeighbors ( vidx2, vidx1 );
				}
			}
		}

		void ViewRelations::adoptNeighbors ( AMatrixf &neighborMat )
		{
            if ( neighbors.size() < neighborMat.rows() )
			{
				neighbors.resize ( neighborMat.rows() );
			}

            for ( unsigned row = 0; row < neighborMat.rows() - 1; ++row )
                for ( unsigned col = row + 1; col < neighborMat.cols(); ++col )
                {
                    setNeighbors ( row, col, neighborMat ( row, col ) > 0 );
                }
		}

		void ViewRelations::setNeighbors( uint vidx1, uint vidx2 )
		{
			if ( neighbors.size() <= vidx1 ) neighbors.resize( 1 + ( vidx1 << 1 ) );
			//printf("vidx1: %d, size: %d\n", vidx1, neighbors.size());
			neighbors[vidx1].push_back( vidx2 );
		}

		void ViewRelations::unsetNeighbors( uint vidx1, uint vidx2 )
		{
			if ( neighbors.size() <= vidx1 ) return;
			for ( uint i = 0; i < neighbors[vidx1].size(); ++i ) if ( neighbors[vidx1][i] == vidx2 )
				{
					for ( uint j = i; j < neighbors[vidx1].size() - 1; ++j )
					{
						neighbors[vidx1][j] = neighbors[vidx1][j + 1];
					}
					neighbors.pop_back();
					break;
				}
		}


	}

}
