

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREVIEWRELATIONS_H
#define ADORE_COREVIEWRELATIONS_H

#include <vector>
#include <map>
#include <adrcore/types/aarray.h>
#include <adrcore/types/view.h>

using std::vector;
using std::map;

namespace adore
{

	namespace core
	{

		/**
		 * contains proximity infos beween views
			 
		*/
		class ViewRelations
		{
			public:
				typedef vector<ViewBase *> ViewsT;
				typedef ViewsT::iterator ViewIterator;
				typedef AMatrixf DisT;//distances to other views
				typedef vector<uint> PalsT;
				typedef vector<PalsT> NeighborsT;
				typedef PalsT::const_iterator IdIterator;
				typedef map<ViewBase*, uint> IndicesT;
				typedef IndicesT::iterator IndicesIterT;

			private:
				DisT distances;
				NeighborsT neighbors;
				IndicesT indices;
				ViewsT views;

			private:
				class DegreeComparator
				{
					public:
						DegreeComparator( const NeighborsT &n ):nb( n ) {}

						bool operator()( uint a, uint b )
						{
							return nb[a].size() > nb[b].size();
						}
					private:
						const NeighborsT &nb;
				};

			public:
				ViewRelations();

				//view maintainance

				/**
				 * adds a view and returns its index
				 */
				uint addView( ViewBase *v );

				uint getNumViews()const { return views.size(); }

				ViewBase *getView( uint idx );

				/**
				 * this may be used to replace thumbnails with their original
				 */
				void replaceView( uint idx, ViewBase *vNew );

				ViewIterator begin() {return views.begin();}

				ViewIterator end() {return views.end();}

				//distance functions, -1 is the distance of an invalid pair

				float getDistance ( uint vidx1, uint vidx2 );

				void setDistance( uint vidx1, uint vidx2, float d );

				void adoptDistances( AMatrixf &distMat );

				//neighbor functions

				bool areNeighbors( uint vidx1, uint vidx2 );

				void setNeighbors( uint vidx1, uint vidx2, bool n );

				void adoptNeighbors( AMatrixf &neighborMat );

				/**
				 * this iterator iterates over all neighbors of the view with the given id
				 */
				IdIterator begin( uint vidx ) const
				{
					assert( vidx < neighbors.size() );
					return neighbors[vidx].begin();
				}

				IdIterator end( uint vidx ) const
				{
					assert( vidx < neighbors.size() );
					return neighbors[vidx].end();
				}

				uint getNumNeighbors( uint vidx ) const
				{
					assert( vidx < neighbors.size() );
					return neighbors[vidx].size();
				}

				//sorts a sequnce of integers according to the number of neighbors their corresponding view has
				template<class Iterator>
				void sort( Iterator begin, Iterator end )
				{
					DegreeComparator comp( neighbors );
					std::sort( begin, end, comp );
				}

			private:

				//set the neighbor status without checks of preexistence
				void setNeighbors( uint vidx1, uint vidx2 );

				void unsetNeighbors( uint vidx1, uint vidx2 );

		};

	}

}

#endif
