

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREFILELOCATOR_H
#define ADORE_COREFILELOCATOR_H

#include <cstdio>
#include <vector>
#include <string>
#include <cstring>
#include <sys/stat.h>
#include <dirent.h>

namespace adore
{

	namespace core
	{

		using std::string;

		/**
		 * tests whether a file exists
		 */
		inline bool exists( const char *path )
		{
			struct stat s;
			int ret = stat( path, &s );
			return ( ret == 0 );
		}
		
		inline bool isDirectory( const char *path )
		{
			struct stat s;
			int ret = stat( path, &s );
			return ( ret == 0 && S_ISDIR(s.st_mode) );
		}

		/**
		 * starts with a slash?
		 */
		inline bool pathIsRelative( const char *path )
		{
			if ( ( strlen( path ) > 0 ) && ( path[0] == '/' ) ) return false;
			return true;
		}

		/**
		 * takes the old complete file name and creates a new one using newPath
		 * newPath may be realtive or absolute
		 * */
		string moveName( string oldPath, string newPath );

		/**
		 * replaces the path component of oldPath with newPath
		 */
		string replacePath( string oldPath, string newPath );

		/**
		 * moves a file. newFile may be constructed with moveName
		 * */
		bool moveFile( string oldFile, string newFile );

		bool createDir( string dir );

		string extractPath( string file );

		string extractName( string file );
		
		string removeExt( string file );

		/**
		 * returns lowercased file extension
		 */
		string fileExt( string file );


		/**
		 * clear an input stream. common usage: fdrain(stdin);
		 */
		inline void fdrain( FILE *fp )
		{
			int ch = 0;
			while ( ( ch = getc( fp ) ) != EOF && ch != '\n' )	{	continue;	}
		}

		template<class Collection>
		int getDirEntries( const char* path, Collection &coll )
		{
			DIR *dp;
			struct dirent *ep;
			int numRead = 0;

			dp = opendir ( path );
			if ( dp != NULL )
			{
				while ( ( ep = readdir ( dp ) ) )
				{
					coll.push_back( ep->d_name );
					++numRead;
				}
				( void ) closedir ( dp );
			}
			else
				numRead = -1;

			return numRead;
		}

		/**
			 
		*/
		class FileLocator
		{
			private:
#define FL_BUF__LEN 512
				static const int bufLen;
				typedef std::vector<std::string> PathsT;
			public:
				FileLocator();

				~FileLocator();

				/**
				* add path to the list of search paths
				* if path is not a directory, dirname(path) ist added
				*/
				void addPath( const char *path );

				std::string getLocation( const char *path ) const;

			public:

				bool exists( const char *path ) const
				{
					int ret = stat( path, &s );
					return ( ret == 0 );
				}

			private:
				PathsT paths;
				mutable struct stat s;
				char buffer[FL_BUF__LEN];
		};

	}

}

#endif
