

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "floatshift.h"
#include <cmath>
#include <adrcore/util/debug.h>

namespace adore
{

	namespace core
	{


		double shift( double d, int digits )
		{
			assert( sizeof( double ) == sizeof( long long ) );

			const int shifter = DBL_MANT_DIG - 1;
			const int masker = ( DBL_MAX_EXP << 1 ) - 1;
			const long long mask = LONG_LONG_MAX & ( ( long long )( masker ) << shifter );
			const long long umask = ! mask;

			long long dg = ( long long )( digits ) << shifter;

			//     long long val = *(long long *) &d; //old version
			long long val = 0;
			memcpy( &val, &d, sizeof( double ) );

			long long uval = val & umask;

			val &= mask;
			val += dg;
			val |= uval;

			//     return *(double *) &val;//old version
			memcpy( &d, &val, sizeof( double ) );
			return d;
		}

		float shift( float d, int digits )
		{
			assert( sizeof( float ) == sizeof( int ) );

			const int shifter = FLT_MANT_DIG - 1;
			const int masker = ( FLT_MAX_EXP << 1 ) - 1;
			const int mask = INT_MAX & ( masker << shifter );
			const int umask = ! mask;

			digits <<= shifter;

//			int val = *( int * ) &d;
			int val = 0;
			memcpy( &val, &d, sizeof( float ) );

			int uval = val & umask;
			val &= mask;
			val += digits;
			val |= uval;

//			return *( float * ) &val;
			memcpy( &d, &val, sizeof( float ) );
			return d;
		}

		int getBinaryExponent( double d )
		{
			assert( sizeof( double ) == sizeof( long long ) );

			if ( fabs( d ) <= DBL_EPSILON )
			{
//				if ( d != 0 )
//				{
//					DBGOUT( DBGNORMAL, "getBinaryExponent: %f is assumed to be 0\n", d );
//				}
				return 0;
			}

			const int shifter = DBL_MANT_DIG - 1;
			const int masker = ( DBL_MAX_EXP << 1 ) - 1;
			const int bias = DBL_MAX_EXP - 1;
			const long long mask = LONG_LONG_MAX & ( ( long long )( masker ) << shifter );

//			long long val = *( long long * ) &d;
			long long val = 0;
			memcpy( &val, &d, sizeof( double ) );

			val &= mask;
			val >>= shifter;
			val -= bias;

			return ( int ) val;
		}

		int getBinaryExponent( float a )
		{
			assert( sizeof( float ) == sizeof( int ) );

			if ( fabsf( a ) <= FLT_EPSILON )
			{
//				if ( a != 0 )
//				{
//					DBGOUT( DBGNORMAL, "getBinaryExponent: %f is assumed to be 0\n", a );
//				}
				return 0;
			}

			const int shifter = FLT_MANT_DIG - 1;
			const int masker = ( FLT_MAX_EXP << 1 ) - 1;
			const int bias = FLT_MAX_EXP - 1;
			const int mask = INT_MAX & ( ( masker ) << shifter );

//			int val = *( int * ) &a;
			int val = 0;
			memcpy( &val, &a, sizeof( float ) );

			val &= mask;
			val >>= shifter;
			val -= bias;

			return val;
		}


	}

}
