

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "gaussians.h"

namespace adore
{

	namespace core
	{

		void getGaussianWeights( float *w, const int hood )
		{
			int hWidth = hood + hood + 1;
			double sigma = getSigmaForKernelOfSize( hWidth );
			sigma *= sigma;
			for ( int y = 0; y <= hood; ++y )
			{
				for ( int x = 0; x <= hood; ++x )
				{
//					printf( "(%d, %d)\n", x, y );
					float val = expf( -0.5*( x*x+y*y )/sigma );
					w[hood + x + hWidth * ( hood + y )] = val;
					w[hood - x + hWidth * ( hood - y )] = val;
					w[hood + x + hWidth * ( hood - y )] = val;
					w[hood - x + hWidth * ( hood + y )] = val;
				}
			}
		}

		void getGaussianWeights2( float *w, const int hood )
		{
			int hWidth = hood + hood;
			double sigma = getSigmaForKernelOfSize( hWidth + 1 );
			sigma *= sigma;
			for ( int y = 0; y < hood; ++y )
			{
				for ( int x = 0; x < hood; ++x )
				{
					float dx = float( x ) + 0.5F;
					float dy = float( y ) + 0.5F;

					float val = expf( -0.5*( dx*dx+dy*dy )/sigma );
//					printf( "(%d, %d) -> %f  %d  %d\n", x, y, val, hood, hWidth );
					w[hood + x + hWidth * ( hood + y )] = val;
					w[hood + x + hWidth * ( hood - y - 1 )] = val;
					w[hood - x - 1 + hWidth * ( hood + y )] = val;
					w[hood - x - 1 + hWidth * ( hood - y - 1 )] = val;
				}
			}
//			pressKey();
		}

		AMatrixf Gaussian[] =
		{
			AMatrixf( 0,0 ),
			AMatrixf( 1,1 ),
			AMatrixf( 2,2 ),
			AMatrixf( 3,3 ),
			AMatrixf( 4,4 ),
			AMatrixf( 5,5 ),
			AMatrixf( 6,6 ),
			AMatrixf( 7,7 ),
			AMatrixf( 8,8 ),
			AMatrixf( 9,9 ),
			AMatrixf( 10,10 ),
			AMatrixf( 11,11 ),
			AMatrixf( 12,12 ),
			AMatrixf( 13,13 ),
			AMatrixf( 14,14 ),
			AMatrixf( 15,15 ),
			AMatrixf( 16,16 ),
			AMatrixf( 17,17 ),
			AMatrixf( 18,18 ),
			AMatrixf( 19,19 ),
			AMatrixf( 20,20 )
		};

		int initGaussians()
		{
// 			printf("INIT GAUSSIANS\n");
			const int num = 21;
			Gaussian[1]( 0, 0 ) = 1;
			for ( int i = 2; i < num; i += 2 )
			{
				getGaussianWeights2( Gaussian[i], i >> 1 );
			}
			for ( int i = 3; i < num; i += 2 )
			{
				getGaussianWeights( Gaussian[i], i >> 1 );
			}
			return num;
		}

		const int numGaussians = initGaussians();


	}

}
