

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREGAUSSIANS_H
#define ADORE_COREGAUSSIANS_H

#include <adrcore/types/aarray.h>
//#include <adrcore/util/mathutil.h>

#include <math.h>

namespace adore
{

	namespace core
	{

		/**
			 
		*/
		inline float getSigmaForKernelOfSize( int size )
		{
			return 0.3F * float( size ); //approximately cuts of at 3*sigma
			//return 0.15F * float(size); //approximately cuts of at 3*sigma
		}

		inline int getKernelSizeForSigma( float sigma )
		{
			return int( 6.67F * sigma + 0.5F ); //approximately cuts of at 3*sigma
		}

		/**
		 * size of w has to be (hood + hood + 1)²
		 */
		template<class ArrayT>
		void getGaussianWeights( ArrayT &w, const int hood )
		{
                        typedef typename ArrayT::scalar_type valueT;
			int hWidth = hood + hood + 1;
//			w.resize( hWidth, hWidth );
			double sigma = getSigmaForKernelOfSize( hWidth );
			sigma *= sigma;
			for ( int y = 0; y <= hood; ++y )
			{
				for ( int x = 0; x <= hood; ++x )
				{
//					printf( "(%d, %d)\n", x, y );
					valueT val = valueT( exp( -0.5*( x*x+y*y )/sigma ) );
					w( hood + x, hood + y ) = val;
					w( hood - x, hood - y ) = val;
					w( hood + x, hood - y ) = val;
					w( hood - x, hood + y ) = val;
				}
			}
		}

		/**
		* size of w has to be (hood + hood)²
		*/
		template<class ArrayT>
		void getGaussianWeights2( ArrayT &w, const int hood )
		{
                        typedef typename ArrayT::scalar_type valueT;
			int hWidth = hood + hood;
			double sigma = getSigmaForKernelOfSize( hWidth + 1 );
			sigma *= sigma;
			for ( int y = 0; y < hood; ++y )
			{
				for ( int x = 0; x < hood; ++x )
				{
					float dx = float( x ) + 0.5F;
					float dy = float( y ) + 0.5F;

					valueT val = expf( -0.5*( dx*dx+dy*dy )/sigma );
//					printf( "(%d, %d) -> %f  %d  %d\n", x, y, val, hood, hWidth );
					w( hood + x, hood + y ) = val;
					w( hood + x, hood - y - 1 ) = val;
					w( hood - x - 1, hood + y ) = val;
					w( hood - x - 1, hood - y - 1 ) = val;
				}
			}
//			pressKey();
		}

		//odd width (2 * hood + 1)
		void getGaussianWeights( float *w, const int hood );

		//even width (2 * hood)
		void getGaussianWeights2( float *w, const int hood );

		extern AMatrixf Gaussian[];
		extern const int numGaussians;

	}

}

#endif
