

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_PRINTIMAGE
#define ADORE_PRINTIMAGE

#include <adrcore/imgprocessing/channel.h>

namespace adore
{
	namespace core
	{

		template <class T>
		inline void printChannelRegion ( const Channel<T> &chan, int _x, int _y, int _w, int _h )
		{
			for ( int y = _y; y < _y + _h; ++y )
			{
				for ( int x = _x; x < _x + _w; ++x )
				{
					T *val = chan.getPointer ( x, y );
//					std::cout << "*" << val << " ";
//					printf( "%p: (x, y) - %u ", *val );
//					printf( " %u ", *val );
					std::cout << " " << *val << " ";
				}
				printf ( "\n" );
			}
		}

		template <>
		inline void printChannelRegion<uint8_t> ( const Channel<uint8_t> &chan, int _x, int _y, int _w, int _h )
		{
			for ( int y = _y; y < _y + _h; ++y )
			{
				for ( int x = _x; x < _x + _w; ++x )
				{
					uint8_t *val = chan.getPointer ( x, y );
//					std::cout << "*" << val << " ";
//					printf( "%p: (x, y) - %u ", *val );
//					printf( " %u ", *val );
					printf ( "%3d ", ( int ) *val );
				}
				printf ( "\n" );
			}
		}

		template <>
		inline void printChannelRegion<int8_t> ( const Channel<int8_t> &chan, int _x, int _y, int _w, int _h )
		{
			for ( int y = _y; y < _y + _h; ++y )
			{
				for ( int x = _x; x < _x + _w; ++x )
				{
					int8_t *val = chan.getPointer ( x, y );
//					std::cout << "*" << val << " ";
//					printf( "%p: (x, y) - %u ", *val );
//					printf( " %u ", *val );
					printf ( "%3d ", ( int ) *val );
				}
				printf ( "\n" );
			}
		}


		template <class T>
		inline void printChannelRegion ( const ChannelSet<T> &chan, int _x, int _y, int _w, int _h )
		{
			for ( int y = _y; y < _y + _h; ++y )
			{
				for ( int x = _x; x < _x + _w; ++x )
				{
					std::cout << " [";
					for ( int c = 0; c < chan.getNumChans() - 1; ++c )
					{
						T *val = chan[c]->getPointer ( x, y );
						std::cout << *val << ",";
					}
					T *val = chan[chan.getNumChans() - 1]->getPointer ( x, y );
					std::cout << *val << "]";
				}
				printf ( "\n" );
			}
		}

		template <>
		inline void printChannelRegion<uint8_t> ( const ChannelSet<uint8_t> &chan, int _x, int _y, int _w, int _h )
		{
			for ( int y = _y; y < _y + _h; ++y )
			{
				for ( int x = _x; x < _x + _w; ++x )
				{
					printf( " [" );
					for ( int c = 0; c < chan.getNumChans() - 1; ++c )
					{
						uint8_t *val = chan[c]->getPointer ( x, y );
						printf ( "%3d,", ( int ) *val );
					}
					uint8_t *val = chan[chan.getNumChans() - 1]->getPointer ( x, y );
					printf ( "%3d]", ( int ) *val );
				}
				printf ( "\n" );
			}
		}

		template <>
		inline void printChannelRegion<int8_t> ( const ChannelSet<int8_t> &chan, int _x, int _y, int _w, int _h )
		{
			for ( int y = _y; y < _y + _h; ++y )
			{
				for ( int x = _x; x < _x + _w; ++x )
				{
					printf( " [" );
					for ( int c = 0; c < chan.getNumChans() - 1; ++c )
					{
						int8_t *val = chan[c]->getPointer ( x, y );
						printf ( "%3d,", ( int ) *val );
					}
					int8_t *val = chan[chan.getNumChans() - 1]->getPointer ( x, y );
					printf ( "%3d]", ( int ) *val );
				}
				printf ( "\n" );
			}
		}

	}
}

#endif
