

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_PRINTMAT
#define ADORE_PRINTMAT

#include <cfloat>
#include <adrcore/util/debug.h>
#include <adrcore/util/mathutil.h>
#include <adrcore/util/avgvar.h>
#include <adrcore/types/aarray.h>

#ifndef DBGLEVEL
#define DBGLEVEL 0
#endif

namespace adore
{
	namespace core
	{

		/**
		 * prints a matrix
		 * @param mat the matrix
		 * @param digits the number of digits to be printed.
		 */
		template<class T>
		inline void printMat( const AArray<T, 2> &mat, int digits = 8,
		                      FILE *f = stdout, int drow = -1,
		                      int dcol = -1 )
		{
			assert( digits >= 0 && digits < 100 );

			double fac = 1.0;
                        if ( drow >= 0 && drow < (int)mat.rows() && dcol >= 0 && dcol < (int)mat.cols() )
			{
				double d = mat( drow, dcol );
				if ( fabs( d ) > FLT_EPSILON ) fac /= d;
			}

			int maxDigits = 0;
			for ( int row = 0; row < (int)mat.rows(); ++row )
			{
				for ( int col = 0; col < (int)mat.cols(); ++col )
				{
					double val = mat( row, col ) * fac;
					maxDigits = std::max( maxDigits, getNumDigits( val ) );
				}
			}

			char format[10];
			sprintf( format, "%% %d.%df  ", maxDigits + digits + 2, digits );

                        for ( unsigned row = 0; row < mat.rows(); ++row )
			{
                                for ( unsigned col = 0; col < mat.cols(); ++col )
				{
					double val = mat( row, col ) * fac;
					fprintf( f, format , val );
				}
				fprintf( f, "\n" );
			}
		}

		/**
		 * prints a matrix
		 * @param mat the matrix
		 * @param digits the number of digits to be printed.
		 */
		template<class T>
		inline void printMat( int debugLevel, const AArray<T, 2> &mat,
		                      int digits = 8, FILE *f = stdout,
		                      int drow = -1, int dcol = -1 )
		{
			if ( debugLevel > DBGLEVEL ) return;
			printMat( mat, digits, f, drow, dcol );
		}


		/**
		 * prints two matrices for comparison side by side
		 * @param mat1 the one matrix
		 * @param mat2 and the others matrix
		 * @param digits the number of digits to be printed.
		 *
		 */
		template<class T>
		inline void printMats( const AArray<T, 2> &mat1, const AArray<T, 2> &mat2,
		                       int digits = 8, FILE *f = stdout,
		                       int drow = -1, int dcol = -1 )
		{
			int minRows = MIN(mat1.rows(), mat2.rows());
			assert( digits >= 0 && digits < 100 );

			double fac1 = 1.0;
			double fac2 = 1.0;
                        if ( drow >= 0 && drow < (int)mat1.rows() && drow < (int)mat2.rows()
                                        && dcol >= 0 && dcol < (int)mat1.cols() && dcol < (int)mat2.cols() )
			{
				double d = mat1( drow, dcol );
				if ( fabs( d ) > FLT_EPSILON ) fac1 /= d;
				else printf( "norming division omitted (d = %f)\n", d );
				d = mat2( drow, dcol );
				if ( fabs( d ) > FLT_EPSILON ) fac2 /= d;
				else printf( "norming division omitted (d = %f)\n", d );
			}

			char *format = new char[10];
			sprintf( format, "%%.%df  ", digits );
			for ( int row = 0; row < minRows; ++row )
			{
                                for ( unsigned col = 0; col < mat1.cols(); ++col )
				{
					double val = mat1( row, col ) * fac1;
					if ( val >= 0 ) printf( " " );
					fprintf( f, format , val );
				}

				fprintf( f, "  |  " );

                                for ( unsigned col = 0; col < mat2.cols(); ++col )
				{
					double val = mat2( row, col ) * fac2;
					if ( val >= 0 ) fprintf( f, " " );
					fprintf( f, format , val );
				}
				fprintf( f, "\n" );
			}
			delete [] format;
		}

		/**
		 * prints two matrices for comparison side by side
		 * @param mat1 the one matrix
		 * @param mat2 and the others matrix
		 * @param digits the number of digits to be printed.
		 *
		 */
		template<class T>
		inline void printMats( int debugLevel, const AArray<T, 2> &mat1, const AArray<T, 2> &mat2,
		                       int digits = 8, FILE *f = stdout,
		                       int drow = -1, int dcol = -1 )
		{
			if ( debugLevel > DBGLEVEL ) return;
			printMats( mat1, mat2, digits, f, drow, dcol );
		}

//











//

		/**
		 * prints a matrix
		 * @param mat the matrix
		 * @param digits the number of digits to be printed.
		 */
		template<class T>
		inline void printMat( const AArray<T, 1> &mat, int digits = 8,
		                      FILE *f = stdout, int drow = -1 )
		{
			assert( digits >= 0 && digits < 100 );
			char *format = new char[10];
			sprintf( format, "%%.%df  ", digits );

			double fac = 1.0;
                        if ( drow >= 0 && drow < (int) mat.rows() )
			{
				double d = mat( drow );
				if ( fabs( d ) > FLT_EPSILON ) fac /= d;
			}

                        for ( unsigned row = 0; row < mat.rows(); ++row )
			{
				double val = mat( row ) * fac;
				if ( val >= 0 ) fprintf( f, " " );
				fprintf( f, format , val );
				fprintf( f, "\n" );
			}
			delete [] format;
		}

		/**
		 * prints a matrix
		 * @param mat the matrix
		 * @param digits the number of digits to be printed.
		 */
		template<class T>
		inline void printMat( int debugLevel, const AArray<T, 1> &mat,
		                      int digits = 8, FILE *f = stdout,
		                      int drow = -1 )
		{
			if ( debugLevel > DBGLEVEL ) return;
			printMat( mat, digits, f, drow );
		}


		/**
		 * prints two matrices for comparison side by side
		 * @param mat1 the one matrix
		 * @param mat2 and the others matrix
		 * @param digits the number of digits to be printed.
		 *
		 */
		template<class T>
		inline void printMats( const AArray<T, 1> &mat1, const AArray<T, 1> &mat2,
		                       int digits = 8, FILE *f = stdout,
		                       int drow = -1 )
		{
			int minRows = MIN(mat1.rows(), mat2.rows());
			assert( digits >= 0 && digits < 100 );

			double fac1 = 1.0;
			double fac2 = 1.0;
                        if ( drow >= 0 && drow < (int)mat1.rows() && drow < (int)mat2.rows() )
			{
				double d = mat1( drow );
				if ( fabs( d ) > FLT_EPSILON ) fac1 /= d;
				else printf( "norming division omitted (d = %f)\n", d );
				d = mat2( drow );
				if ( fabs( d ) > FLT_EPSILON ) fac2 /= d;
				else printf( "norming division omitted (d = %f)\n", d );
			}

			char *format = new char[10];
			sprintf( format, "%%.%df  ", digits );
			for ( int row = 0; row < minRows; ++row )
			{
				double val = mat1( row ) * fac1;
				if ( val >= 0 ) printf( " " );
				fprintf( f, format , val );

				fprintf( f, "  |  " );

				val = mat2( row ) * fac2;
				if ( val >= 0 ) fprintf( f, " " );
				fprintf( f, format , val );

				fprintf( f, "\n" );
			}
			delete [] format;
		}

		/**
		 * prints two matrices for comparison side by side
		 * @param mat1 the one matrix
		 * @param mat2 and the others matrix
		 * @param digits the number of digits to be printed.
		 *
		 */
		template<class T>
		inline void printMats( int debugLevel, const AArray<T, 1> &mat1, const AArray<T, 1> &mat2,
		                       int digits = 8, FILE *f = stdout,
		                       int drow = -1 )
		{
			if ( debugLevel > DBGLEVEL ) return;
			printMats( mat1, mat2, digits, f, drow );
		}


		inline void printLine( int debugLevel, AVector2D line, int w, int h )
		{
			if ( debugLevel > DBGLEVEL ) return;
			double x1, x2, y1, y2;
			pointAsLine( line( 0 ), line( 1 ), line( 2 ), x1, y1, x2, y2, w, h );
			printf( "line is (%f, %f)-(%f, %f)\n", x1, y1, x2, y2 );
		}


		template<class IteratorT>
		void scaleEqualityTest( IteratorT m1Begin, IteratorT m2Begin, int size,
		                        double &scale, double &variance, double limit = 8 * FLT_EPSILON )
		{
			scale = variance = 0.0;
			double vals[size];
			int idx = 0;
			int i = 0;

			for ( ; i < size; ++m1Begin, ++m2Begin, ++i )
			{
				double a = *m1Begin;
				double b = *m2Begin;
				if ( fabs( a ) >= limit && fabs( b ) >= limit )
				{
					vals[idx] = a / b;
					++idx;
				}
				else if ( ( fabs( a ) <= limit && fabs( b ) >= limit ) || ( fabs( a ) >= limit && fabs( b ) <= limit ) )
				{
					printf( "in equalityTest:\na: %.20f\nb: %.20f\n", a, b );
					scale = FLT_MAX;
					variance = FLT_MAX;
				}
			}
			if ( scale != FLT_MAX ) getAvgVar( vals, idx, scale, variance,0 );
		}

		template<class T>
		void scaleEqualityTest( const AArray<T, 3> &mat1, const AArray<T, 3> &mat2,
		                        double &scale, double &variance, double limit = 8 * FLT_EPSILON )
		{
			scaleEqualityTest( mat1.begin(), mat2.begin(), mat1.size(), scale, variance, limit );
		}

		template<class T>
		void scaleEqualityTest( const AArray<T, 2> &mat1, const AArray<T, 2> &mat2,
		                        double &scale, double &variance, double limit = 8 * FLT_EPSILON )
		{
			scaleEqualityTest( mat1.begin(), mat2.begin(), mat1.size(), scale, variance, limit );
		}

		template<class T>
		void scaleEqualityTest( const AArray<T, 1> &mat1, const AArray<T, 1> &mat2,
		                        double &scale, double &variance, double limit = 8 * FLT_EPSILON )
		{
			scaleEqualityTest( mat1.begin(), mat2.begin(), mat1.size(), scale, variance, limit );
		}

		inline void whereUnInited( const AMatrix &A )
		{
			//test for uninit cond
			int counter = 0;
			for ( int row = 0; row < (int)A.rows(); ++row )
			{
				for ( int col = 0; col < (int)A.cols(); ++col )
				{
					if ( A( row, col ) == 0 ) printf( " (%d, %d) ", row, col );
				}
			}
			printf( "\n%d uninited\n", counter );
		}

		inline void whereUnInited( const AVector &A )
		{
			//test for uninit cond
			int counter = 0;
			for ( int row = 0; row < (int)A.rows(); ++row )
			{
				if ( A( row ) == 0 ) printf( " (%d) ", row );
			}
			printf( "\n%d uninited\n", counter );
		}


		inline void whereUnInited( const std::vector<AVector *> &vec )
		{
			for ( uint i = 0; i < vec.size(); ++i )
			{
				printf( "%dth entity:\n", i );
				whereUnInited( *vec[i] );
			}
		}

	}
}//namespace


#endif
