

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include <stdint.h>
#include <adrcore/types/aarray.h>
#include <adrcore/fortran/fortran_switch.h>
#include <adrcore/util/debug.h>
#include <adrcore/util/lapackutil.h>
#include <adrcore/util/storageorder.h>

namespace adore
{
	namespace core
	{
		double qr22angle( const double a1, const double a3 )
		{
			double radiant = a1 * a1 + a3 * a3;
			assert(fabs(radiant) > FLT_EPSILON);
			if(a1 > a3)
			{
				return acos(a1/sqrt(radiant));
			}
			else
			{
				return asin(a3/sqrt(radiant));
			}
		}
		
		double qr22angle( const AArray<double, 2> & A )
		{
			return qr22angle(A(0, 0), A(1, 0));
		}
		
		float qr22angle( const AArray<float, 2> & A )
		{
			return qr22angle(A(0, 0), A(1, 0));
		}

		void rq( const AArray<double, 2> &_A, AArray<double, 2> &R, AArray<double, 2> &Q )
		{

			integer rowsA = _A.rows();
			integer colsA = _A.cols();

			double _Q[rowsA * colsA];
			checkStorageOrder( _A );
			rowToColumnMajor( _A.begin(), _Q, rowsA, colsA );

			assert_msg( rowsA == colsA, "currently, only square matrices are supported\n" );

			integer LDA = rowsA;
			integer LWORK = rowsA;
			integer TAU = min( rowsA, colsA );
			integer returnVal = 0;
			//these "new" are necessary to prevent hitting the stack limit
			AArray<double, 1> work( LWORK );
			AArray<double, 1> tau( TAU );

			dgerq2_( &rowsA, &colsA,
			         _Q, &LDA,
			         tau.data(), work.data(),
			         &returnVal );

			assert( returnVal == 0 );

			//extract R
			R = 0;
			for ( int col = 0; col < colsA; ++col )
			{
				int i = col * LDA;
				for ( int row = 0; row <= col; ++row )
				{
					R( row, col ) = _Q[i++];
				}
			}

			//compute Q
			LWORK = -1;//workspace query
			dorgrq_( &rowsA, &colsA,
			         &TAU,
			         _Q, &LDA,
			         tau.data(),
			         work.data(), &LWORK,
			         &returnVal );

			assert( returnVal == 0 );

			LWORK = integer( work( 0 ) );

			AArray<double, 1> work2( LWORK );
			dorgrq_( &rowsA, &colsA,
			         &TAU,
			         _Q, &LDA,
			         tau.data(),
			         work2.data(), &LWORK,
			         &returnVal );

			assert( returnVal == 0 );

			columnToRowMajor( _Q, Q.begin(), rowsA, colsA );
		}

	}
}

