

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORESTLUTILS_H
#define ADORESTLUTILS_H

#include <cstdlib>
#include <vector>
#include <list>
#include <queue>
#include <stack>
#include <set>
#include <functional>
#include <string>

namespace adore
{
	namespace core
	{

		using namespace std;

//		now the deleteElements functions are generated for some collections
#define ADORE_GENERATE_DESTROYER(COLLECTION_TYPE) \
template <class T> \
void deleteElements(COLLECTION_TYPE<T> *v)\
{\
    typename COLLECTION_TYPE<T>::iterator i = v->begin();\
    while(i != v->end()) {delete *i; ++i;}\
    v->clear();\
}

		/**
		    Deletes the Elements of a vector, so don't use it an vectors containing pointers to arrays.
		*/
		ADORE_GENERATE_DESTROYER( vector );
		/**
		    Deletes the Elements of a list, so don't use it an lists containing pointers to arrays.
		*/
		ADORE_GENERATE_DESTROYER( list );
		/**
		    Deletes the Elements of a queue, so don't use it an queues containing pointers to arrays.
		*/
		ADORE_GENERATE_DESTROYER( queue );
		/**
		    Deletes the Elements of a stack, so don't use it an stacks containing pointers to arrays.
		*/
		ADORE_GENERATE_DESTROYER( stack );
		/**
		    Deletes the Elements of a set, so don't use it an stacks containing pointers to arrays.
		*/
		ADORE_GENERATE_DESTROYER( set );

//#undef ADORE_GENERATE_DESTROYER


		/**
		    Sets the Elements of a vector&lt;T&gt; to T(0)
		 * @param v the vector.
		*/
		template <class T>
		void zeroVector( vector<T> &v )
		{
			uint i;
			for ( i = 0; i < v.size(); ++i ) { v[i] = T( 0 ); }
		}

		/**
		    appends one vector to another
		 * @param v the appendor
		 * @param appendix the appendee
		*/
		template <class T>
		inline void append( vector<T> &v, vector<T> &appendix )
		{
			uint i;
			for ( i = 0; i < appendix.size(); i++ ) v.push_back( appendix[i] );
		}

		inline string stringToUpper( string strToConvert )
		{
			for ( unsigned int i = 0; i < strToConvert.length(); i++ )
			{
				strToConvert[i] = toupper( strToConvert[i] );
			}
			return strToConvert;//return the converted string
		}

		inline string stringToLower( string strToConvert )
		{
			for ( unsigned int i = 0; i < strToConvert.length(); i++ )
			{
				strToConvert[i] = tolower( strToConvert[i] );
			}
			return strToConvert;//return the converted string
		}

	}
}
#endif
