

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef ADORE_CORE_EPILINESEGMENTMATCHER_H
#define ADORE_CORE_EPILINESEGMENTMATCHER_H

#include <adrcore/types/view.h>
#include <adrcore/types/nview.h>
#include <adrcore/types/featuredescriptor.h>
#include <adrcore/nview/fmatrixcalculator.h>
#include <adrcore/util/matmul2.h>

#include <vector>
#include <map>
#include <utility>

#include "matcher.h"

namespace adore
{
	namespace core
	{
		using namespace std;
		class EpiLineSegmentMatcher : public Matcher
		{
			typedef pair<AVector, AVector> segT;
			typedef pair<segT, segT> candT;
			typedef pair<ViewBase::segmentT, ViewBase::segmentT> candOrigT;
			typedef vector<candT> candListT;
			typedef vector<candOrigT> candOrigListT;
			typedef map<uint, candListT> candMapT;
			typedef map<uint, candOrigListT> candOrigMapT;

		public:
			enum Mode { SingleBest, BetterAs };

		public:
			EpiLineSegmentMatcher(double minLength = 15.0, double maxRatio = 1.5, Mode m = BetterAs) :
					minLength2(minLength * minLength),
					maxRatio2(maxRatio * maxRatio),
					mode(m)
			{}
			virtual NViewOperator* clone() { return new EpiLineSegmentMatcher(*this); }
			void setMode(Mode m) { mode = m; }

		protected:
			void matchDerived ( ViewT * v1, ViewT * v2, NView * set );

		private:
			double minLength2, maxRatio2;
			Mode mode;
			void findCandidates(ViewBase* v1, ViewBase* v2, const AMatrix& fmat, candMapT& candidates, candOrigMapT& candOrig); //TODO

			enum Relation { Collinear, Negative, Positive };
			typedef std::pair<Relation, Relation> PointsInfo;
			inline Relation testLinePoint(const AVector& line, const AVector& pt);
			inline PointsInfo testLine2Points(const AVector& line, const AVector& pt1, const AVector& pt2);
			bool evaluateIntersection(const AVector& line, const AVector& pt1, const AVector& pt2, AVector& pt, PointsInfo& x);
			double dist2(const AVector& pt1, const AVector& pt2) {
				AVector temp(2);
				temp = pt2(0) - pt1(0), pt2(1) - pt1(1);
				return temp(0) * temp(0) + temp(1) * temp(1);
			}

		};

	}

}

#endif
