

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_FEATUREFINDER_H
#define ADORE_FEATUREFINDER_H


#include <adrcore/types/featurepoint.h>
#include <adrcore/types/nviewmap.h>
#include <adrcore/viewmatch/descriptorcreator.h>
namespace adore
{
	namespace core
	{

		class FeatureFinder
		{
			protected:
				typedef uint8_t imgT;
				typedef View<imgT> ViewT;

			public:
				virtual ~FeatureFinder() {;}

				unsigned int findFeatures ( ViewT *v )
				{
					return findFeaturesDerived ( v );
				}

				virtual void setDescriptorCreator ( DescriptorCreator* theValue, bool own = false ) {}

				template<class ViewIterator>
				void findFeatures ( ViewIterator vit, int numViews )
				{
					for ( int i = 0; i < numViews; ++i, ++vit )
					{
						ViewT *v = dynamic_cast<ViewT *> ( *vit );
						DBGOUT( DBGNORMAL, "FeatureFinder: finding features in View %p (%s)\n", v, v->getImagePath().c_str() );
						findFeaturesDerived ( v );
					}
				}

				virtual void findFeatures ( NViewMap &map )
				{
					findFeatures ( map.getAllViewsBegin(), map.getNumAllViews() );
				}

				virtual FeatureFinder* clone() {error( "missing implementation\n" );}

			protected:
				virtual unsigned int findFeaturesDerived ( ViewT *v ) = 0;

		};

	}
}//ns

#endif
