

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef FLANNMATCHER_H
#define FLANNMATCHER_H

#include <adrcore/viewmatch/matcher.h>

#include <vector>
#include <boost/shared_array.hpp>

namespace adore
{
	namespace core
	{

		/**
		 *	 
		*/
		class FlannMatcher : public Matcher
		{
			float m_threshold; // theshold on matching badness
			uint m_count_threshold; // theshold on number of corrs

		public:
			/**
			  * @param threshold minimum acceptable distance
			  * @param count_threshold maximum number of correspondences
			  */
			FlannMatcher(float threshold = 0.1, uint count_threshold = INT_MAX);
			FlannMatcher(const FlannMatcher& rhs);

			virtual NViewOperator* clone() { return new FlannMatcher( *this ); }

			void setDistThresh ( const float threshold ) { m_threshold = threshold; }
			float getDistThresh() const { return m_threshold; }
			void setCountThresh ( const float count_threshold ) { m_count_threshold = count_threshold; }
			float getCountThresh() const { return m_count_threshold; }

			void match ( ViewBase * v1, ViewBase * v2, NView * set );

		protected:
			void matchDerived ( ViewT * v1, ViewT * v2, NView * set )
			{ match ( static_cast<ViewBase *> ( v1 ), static_cast<ViewBase *> ( v2 ), set ); }

		private:
			boost::shared_array<float> copy_data(ViewBase * view, std::vector<FeaturePoint*>& fps, uint& columns);

		public:
			// 'good' value for threshold
			static float getGoodDistanceThreshold() { return 0.1f; }

			// 'good' value for count_threshold
			static uint getGoodCountThreshold() { return INT_MAX; }
		};

	} // core
} // adore

#endif // FLANNMATCHER_H
