

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "harrisfinder.h"
#include <adrcore/imgprocessing/dizenzogradients.h>
#include <adrcore/imgprocessing/harris.h>
#include <adrcore/imgprocessing/harrisfast.h>
#include <adrcore/imgprocessing/harrisfasthood.h>
#include <adrcore/imgprocessing/rgbtogray.h>
#include <adrcore/features/extremafeatures.h>

namespace adore
{

	namespace core
	{

		HarrisFinder::HarrisFinder ( uint ID, uint hHood, float mind, ouT minst, float minfr, int maxpts ) :
				fid ( ID ), minStrength ( minst ), minDist ( mind ), minFraction ( minfr ), harrisHood ( hHood ),
                maxPoints ( maxpts ) {;}

		unsigned int HarrisFinder::findFeaturesDerived ( ViewT *v )
		{
			this->v = v;

			vector<FeaturePoint *> pts;
			ImagePlane<imgT> *img = v->getImage();
			assert ( img );

			//allocate harris image
			ImagePlane<ouT> h ( img->getWidth(), img->getHeight(), 1 );

			if ( img->getNumChannels() == 1 )
			{
				if ( harrisHood == 1 )
				{
					DBGOUT( DBGNORMAL, "HarrisFinder uses one Channel/3x3 -> HarrisFast\n" );
					HarrisFast<imgT> harris;
					harris.process ( img, &h );
				}
				else
				{
					HarrisFastHood<imgT> harris ( harrisHood );
					harris.process ( img, &h );
				}
			}
			else
			{
				if ( harrisHood == 1 )
				{
					DBGOUT( DBGNORMAL, "HarrisFinder uses three Channels/3x3 -> HarrisFast\n" );
					ChannelSet<imgT> in( *img, true );
					HarrisFast<imgT> harris;
					harris.process ( in, &h );
				}
				else
				{
					calcDizenzo ( *img );
					ChannelSet<ouT> out;
					out.add ( &gxx );
					out.add ( &gyy );
					out.add ( &gxy );

					Harris<ouT> harris;
					harris.setHood ( harrisHood );
					harris.process ( out, &h );
				}
			}

			ExtremaFeatures<ouT, HarrisFinder> extrema ( fid, minDist, minStrength, minFraction, maxPoints );
			extrema.getFeatures ( h, *this );
			return v->getNumFeaturePoints();
		}

		void HarrisFinder::calcDizenzo ( ImagePlane<imgT> &img )
		{
			//re-initialize g??
			gxx.init ( img.getWidth(), img.getHeight(), 1 );
			gyy.init ( img.getWidth(), img.getHeight(), 1 );
			gxy.init ( img.getWidth(), img.getHeight(), 1 );

			UsingChannelMode cm ( &img );//don't forget to restore the original value

			DiZenzoGradients<imgT, ouT> grad;

			ChannelSet<imgT> gradInput ( img, true );

			ChannelSet<ouT> out;
			out.add ( &gxx );
			out.add ( &gyy );
			out.add ( &gxy );

			grad.process ( gradInput, out );
		}

	}
}
