

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORENVIEWDEPENDENCYGRAPH_H
#define ADORE_CORENVIEWDEPENDENCYGRAPH_H

#include <adrcore/types/nviewmap.h>
#include <map>
#include <adrcore/util/c++0x.h>
#include "viewknitter.h"

#include <boost/config.hpp>
#include <boost/graph/adjacency_list.hpp>

namespace adore
{

	namespace core
	{

		class NViewDependencyGraph
		{
			public:
				typedef adjacency_list<vecS, vecS, directedS, NView *> Graph;
				typedef graph_traits<Graph>::vertex_descriptor Vertex;
				typedef graph_traits<Graph>::edge_descriptor Edge;
				typedef graph_traits<Graph>::vertex_iterator VertexIter;
				typedef vector<Vertex> rootColl;
				typedef graph_traits<Graph>::adjacency_iterator AdjacencyIter;
				typedef rootColl::iterator RootIter;
				typedef rootColl::const_iterator ConstRootIter;

			private:
				Graph graph;
				vector<Vertex> roots;
				std::map<NView *, Vertex> nvToVertex;

			public:
				NViewDependencyGraph() {}

				NViewDependencyGraph( NViewMap &map ) {create( map );}

				NViewDependencyGraph( NViewMap &map, ViewKnitter &knit ) {create( map, knit );}

				void create( NViewMap &map, ViewKnitter &knit );

				void create( NViewMap &map );

				uint numRoots() const {return roots.size();}

				uint numVertices() const {return num_vertices( graph );}

				NView *getNView( Vertex vertex ) const {return graph[vertex];}

				RootIter getRootsBegin() {return roots.begin();}
				ConstRootIter getRootsBegin() const {return roots.begin();}

				RootIter getRootsEnd() {return roots.end();}
				ConstRootIter getRootsEnd() const {return roots.end();}

				AdjacencyIter getAdjacentBegin( Vertex vertex ) const {return adjacent_vertices( vertex, graph ).first;}

				AdjacencyIter getAdjacentEnd( Vertex vertex ) const {return adjacent_vertices( vertex, graph ).second;}

				int numChildren( Vertex vertex ) const
				{
					return out_degree( vertex, graph );
				}

				string toString();

			private:

				void takeRoots();
		};

	}

}

#endif
