

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORESPLITTER_H
#define ADORE_CORESPLITTER_H

#include <adrcore/viewmatch/viewknitter.h>

namespace adore
{

	namespace core
	{

		/**
		 * uses a knitter and splits a large nview into many small ones
		 * each new NView references only those corrs that belong to at least two of their views
			 
		*/
		class Splitter
		{
			public:

				template <class ViewIterator>
				void fill( const NView &sourceNV, ViewIterator vit, int numViews, NView *nv )
				{
//					ViewIterator vit1 = vit;
					for ( int i = 0; i < numViews; ++i, ++vit )
					{
						ViewBase *v = *vit;
						if(v)
						{
							nv->addView( v );
							
							Camera *cam1 = sourceNV.getCamera(v);
							if(cam1)
							{
								DBGOUT(DBGALL, "Splitter: adding camera\n");
								Camera *cam2 = nv->getCamera(v, true/*create*/);
								*cam2 = *cam1;
							}
							else
							{
								DBGOUT(DBGALL, "Splitter: no camera for view %p\n", v);
							}
						}
					}

//					for ( NView::CorrIterator cit = sourceNV.getPointCorrsBegin(); cit != sourceNV.getPointCorrsEnd(); ++cit )
//					{
//						PointCorr *corr = *cit;
//						int score = getViewScore( vit1, numViews, corr );
//						if ( score >= 2 ) nv->addPointCorr( corr );
//					}
					vector<FeaturePoint *>pts( nv->getNumViews() );
					for ( NView::CorrIterator cit = sourceNV.getPointCorrsBegin(); cit != sourceNV.getPointCorrsEnd(); ++cit )
					{
						PointCorr *corr = *cit;
						int idx = 0;
						for ( NView::ViewIterator nvit = nv->getViewsBegin(); nvit != nv->getViewsEnd(); ++nvit, ++idx )
						{
							FeaturePoint *fp = corr->getFeature( *nvit );
							if ( fp ) pts.push_back( fp );
						}
						if ( pts.size() > 1 ) nv->addPointCorr( pts.begin(), pts.end(), corr->getBadness() );
						pts.clear();
					}
				}

				/**
				 * fills NViews with Correspondences and Views
				 */
				template<class NViewIterator>
				void split( const NView &sourceNV, const ViewKnitter &knit, NViewIterator nit, uint knitPos = 0 )
				{
					for ( ViewKnitter::iterator tit = knit.begin( knitPos ); tit != knit.end( knitPos ); ++tit, ++nit )
					{
						NView *nv = *nit;
						assert( nv );
						const ViewKnitter::VTuple *tup = ( *tit ).get<2>();
						fill( sourceNV, tup->begin(), tup->size(), nv );
					}
				}

				void split( NViewMap &map, const ViewKnitter &knit, uint knitPos = 0 )
				{
					for ( ViewKnitter::const_iterator tit = knit.begin( knitPos ); tit != knit.end( knitPos ); ++tit )
					{
						const ViewKnitter::VTuple *tup = ( *tit ).get<2>();
						NView *nv = map.createNView( tup->begin(), tup->end() );
						assert( nv );
						fill( map.getMasterNView(), tup->begin(), tup->size(), nv );
					}
				}

			private:
				template<class ViewIterator>
				int getViewScore( ViewIterator vit, int numViews, PointCorr *corr )
				{
					int score = 0;
					for ( int i = 0; i < numViews; ++i, ++vit )
					{
						if ( corr->getFeature( *vit ) ) ++score;
					}
					return score;
				}
		};

	}

}

#endif
