

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREVIEWKNITTER_H
#define ADORE_COREVIEWKNITTER_H

#include <adrcore/types/nviewmap.h>
#include <deque>
#include <vector>
#include <adrcore/util/c++0x.h>

namespace adore
{

	namespace core
	{

		/**
		 * creates NViews that cover subsets of all views
			 
		*/
		class ViewKnitter
		{
			public:
				typedef vector<ViewBase *> VTuple;
				typedef deque<VTuple> VTupleCollT;//holds all vtuples

				typedef tuple<VTuple*, VTuple*, VTuple*, VTuple*> Tuple;//(1st, 2nd)->3rd(union),intersection  references to vtuples
				typedef list<Tuple> InnerTupleCollT;
				typedef deque<InnerTupleCollT> OuterTupleCollT;

				typedef InnerTupleCollT::iterator iterator;
				typedef InnerTupleCollT::const_iterator const_iterator;

			private:
				VTupleCollT vtuples;
				OuterTupleCollT tuples;
				int curPos;//diffent meaning on build-up and traversal
			public:

				ViewKnitter():tuples(), curPos( 0 ) {vtuples.push_back( VTuple ( 0 ) ); /*add empty tuple, used when adding first tuples*/}

				/**
				 * uses the pairs to create an incremental merging plan
					rewind, if you like to use the plan after its creation.
					if you just want to reconstruct the state used for a loaded nview, don't rewind.
				 */
				void createPlan( int overlap, bool rewind = true, int maxSteps = INT_MAX, bool filterFromTop = true );

				iterator begin() {return begin( curPos );}
				const_iterator begin() const {return begin( curPos );}
				iterator end() {return end( curPos );}
				const_iterator end() const {return end( curPos );}
				int size() const {return size( curPos );}
				int getNumViewsOfUnion() {return getNumViewsOfUnion( curPos );}

				void next()
				{
					++curPos;
				}

				void prev()
				{
					--curPos;
				}

				void rewind()
				{
					curPos = 1;
				}

				bool hasNext() const
				{
					return ( curPos < ( int )tuples.size() - 1 );
				}

				bool hasPrev() const
				{
					return curPos > 1;
				}

				bool done() const
				{
					return ( curPos >= ( int )tuples.size() );
				}

				int numSteps() const
				{
					return tuples.size();
				}

				int getPos() const
				{
					return curPos;
				}

				iterator begin( const uint pos )
				{
					assert( pos < tuples.size() );
					return tuples[pos].begin();
				}

				iterator end( const uint pos )
				{
					assert( pos < tuples.size() );
					return tuples[pos].end();
				}

				const_iterator begin( const uint pos ) const
				{
					assert( pos < tuples.size() );
					return tuples[pos].begin();
				}

				const_iterator end( const uint pos ) const
				{
					assert( pos < tuples.size() );
					return tuples[pos].end();
				}

				/**
				 * returns the number od tuples of size size
				 */
				int size( const uint pos ) const
				{
					assert( pos < tuples.size() );
					return tuples[pos].size();
				}

				int getNumViewsOfUnion( const uint pos )
				{
					assert( pos < tuples.size() );
					assert( tuples[pos].size() > 0 );
					return tuples[pos].front().get<2>()->size();
				}

				/**
					interface that creates the starting pairs
				*/
				void addTuple ( ViewBase *v1, ViewBase *v2 );

				/**
				 * creates the pairs from the viewrelations stored inside nviewmap
				 */
				void createPairs( NViewMap &map );

			protected:
				/**
					* create tuples of the next larger size
					* if 2-tuples are present, 3 tuples will be generated
					* @return the number of generated tuples
				 */
				int combine( int overlap )
				{
					assert( curPos > 0 );
					//adjust the overlap to a sane range
					overlap = MIN( getNumViewsOfUnion( curPos - 1 ) - 1, overlap );

					return combineBasic( overlap );
				}

				void confirmPlanBasic();

				void uniSection( VTuple *v1, VTuple *v2, VTuple &uni, VTuple &sec );

				//another dumb implementation
				//for those subclasses that dont come up with a more clever one
				int combineBasic( int overlap );

				void addTuple ( VTuple *old1, VTuple *old2, VTuple &t, VTuple &sec )
				{
					vtuples.push_back( t );
					VTuple *tPtr   = &vtuples.back();

					vtuples.push_back( sec );
					VTuple *secPtr = &vtuples.back();

					if ( ( int )tuples.size() < curPos + 1 ) tuples.push_back( InnerTupleCollT() );
					tuples[curPos].push_back( make_tuple( old1, old2, tPtr, secPtr ) );
				}

				/**
				removes superfluous merge steps
				don't call it manually unless you know exactly why
				 */
				void confirmPlan()
				{
					confirmPlanBasic();
				}

				int countViews();
		};

	}

}

#endif
