

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_OPENGLDISPLAYCLIENT_H
#define ADORE_OPENGLDISPLAYCLIENT_H

#include <GL/glut.h>
#include <GL/gl.h>  
#include <GL/glu.h> 
#include <adrcore/util/debug.h>
#include <adrcore/types/commontypes.h>

namespace adore {
namespace visual {

/**
	 
*/
    class DisplayClient
    {
        public:
        
            DisplayClient()
            {
            }
        
            virtual ~DisplayClient()
            {
            }
        
            void drawScene(){drawSceneImpl(); glClear(GL_DEPTH_BUFFER_BIT);}
        
            void idleFunc(){idleFuncImpl();}
        
            void keyPressed(uint8_t key, int x, int y){keyPressedImpl(key, x, y);}
        
            void enable2D()
            {
                int vPort[4];
            
                glGetIntegerv(GL_VIEWPORT, vPort);
            
                glMatrixMode(GL_PROJECTION);
                glPushMatrix();
                glLoadIdentity();
            
                glOrtho(0, vPort[2], 0, vPort[3], -1, 1);
                glMatrixMode(GL_MODELVIEW);
                glPushMatrix();
                glLoadIdentity();
            }

            void disable2D()
            {
                glMatrixMode(GL_PROJECTION);
                glPopMatrix();
                glMatrixMode(GL_MODELVIEW);
                glPopMatrix();
            }
            
            /**
             * width/height of the channel
             */
            void prepareRemap(int chanWidth, int chanHeight)
            {
                width = chanWidth;
                height = chanHeight;
                int winWidth, winHeight;
                int vPort[4];
                glGetIntegerv(GL_VIEWPORT, vPort);
                winWidth = vPort[2];
                winHeight = vPort[3];
            
                double imgAspect;
                imgAspect = (double) width / (double) height;
                resWidth = (int) ((double) winHeight * imgAspect);
                resHeight = winHeight;
                if(resWidth > winWidth)
                {
                    resWidth = winWidth;
                    resHeight = (int) (double(winWidth) / imgAspect);
                }
                hBorder = (winWidth - resWidth) >> 1;
                vBorder = (winHeight - resHeight) >> 1;
            }
        
            void imageToWindow(float &x, float &y)
            {
                x+=0.5;//move center inside pixel
                y+=0.5;
                x = x * resWidth  / width + hBorder;
                y = resHeight - y * resHeight /height + vBorder;
            }
            
            void windowToImage(float &x, float &y)
            {
                x = (x-hBorder) * width  / resWidth + hBorder;
                y = height - (y-vBorder) * height / resHeight;
            }
        
            float getScale()
            {
                return float(resWidth)  / float(width);
            }
            
            int getResHeight(){return resHeight;}
            int getResWidth(){return resWidth;}
            
        private:
        
            virtual void drawSceneImpl() = 0;
        
            virtual void idleFuncImpl(){/*drawScene();*/}
        
            virtual void keyPressedImpl(uint8_t key, int x, int y)
            {
                ADORE_UNUSED(key);
                ADORE_UNUSED(x);
                ADORE_UNUSED(y);
            }

            //remapping of coordinates
            int height, width; //of channel
            int resHeight, resWidth; //of scaled region centered inside gl window
            int vBorder, hBorder;//offset to window border
    };
    
}}

#endif
