

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Andrei Tchouev


#ifndef ADORE_GLDELAUNAYRENDERER_H
#define ADORE_GLDELAUNAYRENDERER_H

#include <vector>
#include <map>

#include "glrenderer.h"
#include <adrcore/surface/geometry3.h>
#include <adrcore/types/aarray.h>

namespace adore
{
	namespace visual
	{
		/**	Performs Delaunay triangulation of the NView points.
		 *
		 *	 
		 */
		class GLDelaunayRenderer : public GLRenderer
		{
			struct Entry
			{
				core::ViewBase* view;					///< Relevant View
				unsigned width;							///< Original image width
				unsigned height;						///< Original image height
				RGBA* pixels;							///< Image data
				GLEntity* entity;						///< Triangulat
				Vector3D eye;							///< Camera location
				Quaternion rot;							///< Camera orientation
			};

		protected:
			core::NView* nv_ref;						///< reference to the NView
			std::vector<Entry> data;					///< reference to Views
			std::vector<core::AVector3D*> nv_points;	///< model points
			std::map<core::AVector3D*,int> vertex_map;	///< points to their indices in nv_points mapping
			core::Point3D centroid;						///< center of mass of all 3d points
			GLEntity* camavatar;						///< camera avatar
			int highlighted;							///< highlighted camera
			float maxdistance;							///< reference scale for clipping planes
			float minobjectsize;						///< refernece sclae for camera avatar size

			std::map<core::NView*, std::vector<Entry> > nview2mesh;

			void clear();
			void triangulate(unsigned index, std::vector<core::Point3D>& vertices);
			void loadCamera(unsigned index);
			bool loadCameraAvatar(unsigned index);
			void drawCamera(unsigned index);

			virtual void select(Vector3D& org, Vector3D& dir);

		public:
			/**	Constructor
			 */
			GLDelaunayRenderer();

			/**	Destructor
			 */
			virtual ~GLDelaunayRenderer();

			virtual void attach(core::NView* nv);
			virtual core::NView* nview()const { return nv_ref; }
			virtual void init();
			virtual void select(const core::ViewBase* const v, const bool reset);
			virtual void paint();
			virtual void pick(int x, int y);
			virtual void perspective(float fov);
			virtual void translate(const float delta[3], bool absolute);
			virtual void rotate(const float axis[3], float angle, bool absolute);
			virtual void scale(const float factor[3], bool absolute);
			virtual void identity();

			virtual void getTranslation(float (& v3)[3], uint idx);

			void selectView(unsigned index, const bool reset);
			void safeView();
			unsigned getNumTriangulations();
			void setActiveTriangulation(unsigned index);
			void setActiveCamera(unsigned index);
			void save(const char* filename);
		};
	}
}

#endif	// ADORE_GLDELAUNAYRENDERER_H
