

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Andrei Tchouev


#include "glentity.h"

#include <GL/gl.h>

#include "glavatar.h"

namespace adore
{
	namespace visual
	{
		GLMesh GLEntity::bbrd;
		bool GLEntity::statready = false;

		GLEntity::GLEntity()
			:pointcolor(0xff,0x00,0x00),linecolor(0xff,0xff,0xff), objUp(0, 1, 0)
		{
			material = 0;
			pointsize = 1.0f;
			thickness = 1.0f;
			highlighted = -1;
			drawmesh = true;
			drawvertices = false;
			drawnormals = false;
			drawframe = false;

			if(!statready)
			{
				GLAvatar::polygon(bbrd,6);
				statready = true;
			}
		}

		int GLEntity::getMaterial()
		{
			assert(material >= 0);
			return material;
		}

		void GLEntity::render()
		{
			world.setOpengGLMatrix();
			texture.setOpenGLTexture();

			if(drawmesh)
				drawMesh();

			if(drawvertices || drawnormals || drawframe)
			{
				GLboolean texturing;
				GLboolean lighting;
				glGetBooleanv(GL_LIGHTING,&texturing);
				glGetBooleanv(GL_LIGHTING,&lighting);
				glDisable(GL_TEXTURE_2D);
				glDisable(GL_LIGHTING);

				if(drawvertices)
				{
					assert(statready);

					glColor4ub(pointcolor.red,pointcolor.green,pointcolor.blue,pointcolor.alpha);
					glPointSize(pointsize);
					glDisable(GL_DEPTH_TEST);

					drawVertices();
					if(highlighted >= 0 && highlighted < mesh.getNumVertices())
					{
						glColor4ub(pointcolor.red,~pointcolor.green,~pointcolor.blue,~pointcolor.alpha);
						mesh.drawBillboardRange(highlighted,1,bbrd,pointsize);
					}

					glEnable(GL_DEPTH_TEST);
				}
				if(drawnormals)
				{
					glLineWidth(thickness);
					glColor4ub(linecolor.red,linecolor.green,linecolor.blue,linecolor.alpha);
					drawNormals();
				}
				if(drawframe)
				{
					glLineWidth(thickness);
					glDisable(GL_DEPTH_TEST);
					drawFrame();
					glEnable(GL_DEPTH_TEST);
				}

				if(texturing==GL_TRUE)
					glEnable(GL_TEXTURE_2D);
				if(lighting==GL_TRUE)
					glEnable(GL_LIGHTING);
			}
		}

		void GLEntity::drawMesh()
		{
			mesh.draw();
		}

		void GLEntity::drawVertices()
		{
			assert(statready);
			mesh.drawBillboardRange(0,mesh.getNumVertices(),bbrd,pointsize);
		}

		void GLEntity::drawNormals()
		{
			mesh.drawNormals();
		}

		void GLEntity::drawFrame()
		{
			// Draw coordinate frame
			glBegin(GL_LINES);
			glColor3f(1.0f,0.0f,0.0f);
			glVertex3f(0.0f,0.0f,0.0f);
			glVertex3f(1.0f,0.0f,0.0f);
			glColor3f(0.0f,1.0f,0.0f);
			glVertex3f(0.0f,0.0f,0.0f);
			glVertex3f(0.0f,1.0f,0.0f);
			glColor3f(0.0f,0.0f,1.0f);
			glVertex3f(0.0f,0.0f,0.0f);
			glVertex3f(0.0f,0.0f,1.0f);
			glEnd();
		}

		int GLEntity::select(Vector3D org, Vector3D dir)
		{
			Vector4D src,dst;
			
// 			printf("GLEntity::select org(%f, %f, %f), dir(%f, %f, %f)\n", org.x, org.y, org.z, dir.x, dir.y, dir.z);

			src.set(dir.x,dir.y,dir.z,0.0f);
			world.unprojectVector(src,dst);
			dir.set(dst.x,dst.y,dst.z);
			dir.normalize();

			src.set(org.x,org.y,org.z,1.0f);
			world.unprojectVector(src,dst);
			float inv_w = 1.0f / src.w;
			org.set(dst.x * inv_w,dst.y * inv_w,dst.z * inv_w);

			return mesh.select(org,dir,pointsize);
		}

		float GLEntity::hit(Vector3D org, Vector3D dir)
		{
			Vector4D src,dst;
			float t = std::numeric_limits<float>::infinity();

			src.set(dir.x,dir.y,dir.z,0.0f);
			world.unprojectVector(src,dst);
			dir.set(dst.x,dst.y,dst.z);
			dir.normalize();

			src.set(org.x,org.y,org.z,1.0f);
			world.unprojectVector(src,dst);
			float inv_w = 1.0f / src.w;
			org.set(dst.x * inv_w,dst.y * inv_w,dst.z * inv_w);
			mesh.select(org,dir,&t);
			return t;
		}
	}
}
