

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Andrei Tchouev


#include "glmath.h"

namespace adore {
namespace visual
{
	const Matrix4x4 Matrix4x4::mat_zero(	Vector4D(0.0f,0.0f,0.0f,0.0f),
						Vector4D(0.0f,0.0f,0.0f,0.0f),
						Vector4D(0.0f,0.0f,0.0f,0.0f),
						Vector4D(0.0f,0.0f,0.0f,0.0f) );

	const Matrix4x4 Matrix4x4::mat_identity(	Vector4D(1.0f,0.0f,0.0f,0.0f),
							Vector4D(0.0f,1.0f,0.0f,0.0f),
							Vector4D(0.0f,0.0f,1.0f,0.0f),
							Vector4D(0.0f,0.0f,0.0f,1.0f) );

	float Matrix4x4::determinant() const
	{
		// 2x2 sub-determinants
		float det2_01_01 = mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0];
		float det2_01_02 = mat[0][0] * mat[1][2] - mat[0][2] * mat[1][0];
		float det2_01_03 = mat[0][0] * mat[1][3] - mat[0][3] * mat[1][0];
		float det2_01_12 = mat[0][1] * mat[1][2] - mat[0][2] * mat[1][1];
		float det2_01_13 = mat[0][1] * mat[1][3] - mat[0][3] * mat[1][1];
		float det2_01_23 = mat[0][2] * mat[1][3] - mat[0][3] * mat[1][2];

		// 3x3 sub-determinants
		float det3_201_012 = mat[2][0] * det2_01_12 - mat[2][1] * det2_01_02 + mat[2][2] * det2_01_01;
		float det3_201_013 = mat[2][0] * det2_01_13 - mat[2][1] * det2_01_03 + mat[2][3] * det2_01_01;
		float det3_201_023 = mat[2][0] * det2_01_23 - mat[2][2] * det2_01_03 + mat[2][3] * det2_01_02;
		float det3_201_123 = mat[2][1] * det2_01_23 - mat[2][2] * det2_01_13 + mat[2][3] * det2_01_12;

		return (
			- det3_201_123 * mat[3][0] + 
			det3_201_023 * mat[3][1] - 
			det3_201_013 * mat[3][2] + 
			det3_201_012 * mat[3][3] );
	}

	Matrix4x4 Matrix4x4::transpose() const
	{
		Matrix4x4 transpose;
		for(int i = 0; i < 4; ++i)
			for(int j = 0; j < 4; ++j)
				transpose[i][j] = mat[j][i];
		return transpose;
	}

	Matrix4x4& Matrix4x4::transposeSelf()
	{
		float temp;
		for(int i = 0; i < 4; ++i)
			for(int j = i + 1; j < 4; ++j)
			{
				temp = mat[i][j];
				mat[i][j] = mat[j][i];
				mat[j][i] = temp;
			}
		return *this;
	}

	bool Matrix4x4::inverseSelf()
	{
		double det, inv_det;

		// 2x2 sub-determinants required to calculate 4x4 determinant
		float det2_01_01 = mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0];
		float det2_01_02 = mat[0][0] * mat[1][2] - mat[0][2] * mat[1][0];
		float det2_01_03 = mat[0][0] * mat[1][3] - mat[0][3] * mat[1][0];
		float det2_01_12 = mat[0][1] * mat[1][2] - mat[0][2] * mat[1][1];
		float det2_01_13 = mat[0][1] * mat[1][3] - mat[0][3] * mat[1][1];
		float det2_01_23 = mat[0][2] * mat[1][3] - mat[0][3] * mat[1][2];

		// 3x3 sub-determinants required to calculate 4x4 determinant
		float det3_201_012 = mat[2][0] * det2_01_12 - mat[2][1] * det2_01_02 + mat[2][2] * det2_01_01;
		float det3_201_013 = mat[2][0] * det2_01_13 - mat[2][1] * det2_01_03 + mat[2][3] * det2_01_01;
		float det3_201_023 = mat[2][0] * det2_01_23 - mat[2][2] * det2_01_03 + mat[2][3] * det2_01_02;
		float det3_201_123 = mat[2][1] * det2_01_23 - mat[2][2] * det2_01_13 + mat[2][3] * det2_01_12;

		det = (
			- det3_201_123 * mat[3][0] + 
			det3_201_023 * mat[3][1] - 
			det3_201_013 * mat[3][2] + 
			det3_201_012 * mat[3][3] );

		if (std::fabs( det ) < MATRIX_INVERSE_EPSILON)
			return false;

		inv_det = 1.0f / det;

		// remaining 2x2 sub-determinants
		float det2_03_01 = mat[0][0] * mat[3][1] - mat[0][1] * mat[3][0];
		float det2_03_02 = mat[0][0] * mat[3][2] - mat[0][2] * mat[3][0];
		float det2_03_03 = mat[0][0] * mat[3][3] - mat[0][3] * mat[3][0];
		float det2_03_12 = mat[0][1] * mat[3][2] - mat[0][2] * mat[3][1];
		float det2_03_13 = mat[0][1] * mat[3][3] - mat[0][3] * mat[3][1];
		float det2_03_23 = mat[0][2] * mat[3][3] - mat[0][3] * mat[3][2];

		float det2_13_01 = mat[1][0] * mat[3][1] - mat[1][1] * mat[3][0];
		float det2_13_02 = mat[1][0] * mat[3][2] - mat[1][2] * mat[3][0];
		float det2_13_03 = mat[1][0] * mat[3][3] - mat[1][3] * mat[3][0];
		float det2_13_12 = mat[1][1] * mat[3][2] - mat[1][2] * mat[3][1];
		float det2_13_13 = mat[1][1] * mat[3][3] - mat[1][3] * mat[3][1];
		float det2_13_23 = mat[1][2] * mat[3][3] - mat[1][3] * mat[3][2];

		// remaining 3x3 sub-determinants
		float det3_203_012 = mat[2][0] * det2_03_12 - mat[2][1] * det2_03_02 + mat[2][2] * det2_03_01;
		float det3_203_013 = mat[2][0] * det2_03_13 - mat[2][1] * det2_03_03 + mat[2][3] * det2_03_01;
		float det3_203_023 = mat[2][0] * det2_03_23 - mat[2][2] * det2_03_03 + mat[2][3] * det2_03_02;
		float det3_203_123 = mat[2][1] * det2_03_23 - mat[2][2] * det2_03_13 + mat[2][3] * det2_03_12;

		float det3_213_012 = mat[2][0] * det2_13_12 - mat[2][1] * det2_13_02 + mat[2][2] * det2_13_01;
		float det3_213_013 = mat[2][0] * det2_13_13 - mat[2][1] * det2_13_03 + mat[2][3] * det2_13_01;
		float det3_213_023 = mat[2][0] * det2_13_23 - mat[2][2] * det2_13_03 + mat[2][3] * det2_13_02;
		float det3_213_123 = mat[2][1] * det2_13_23 - mat[2][2] * det2_13_13 + mat[2][3] * det2_13_12;

		float det3_301_012 = mat[3][0] * det2_01_12 - mat[3][1] * det2_01_02 + mat[3][2] * det2_01_01;
		float det3_301_013 = mat[3][0] * det2_01_13 - mat[3][1] * det2_01_03 + mat[3][3] * det2_01_01;
		float det3_301_023 = mat[3][0] * det2_01_23 - mat[3][2] * det2_01_03 + mat[3][3] * det2_01_02;
		float det3_301_123 = mat[3][1] * det2_01_23 - mat[3][2] * det2_01_13 + mat[3][3] * det2_01_12;

		mat[0][0] =	- det3_213_123 * inv_det;
		mat[1][0] = + det3_213_023 * inv_det;
		mat[2][0] = - det3_213_013 * inv_det;
		mat[3][0] = + det3_213_012 * inv_det;

		mat[0][1] = + det3_203_123 * inv_det;
		mat[1][1] = - det3_203_023 * inv_det;
		mat[2][1] = + det3_203_013 * inv_det;
		mat[3][1] = - det3_203_012 * inv_det;

		mat[0][2] = + det3_301_123 * inv_det;
		mat[1][2] = - det3_301_023 * inv_det;
		mat[2][2] = + det3_301_013 * inv_det;
		mat[3][2] = - det3_301_012 * inv_det;

		mat[0][3] = - det3_201_123 * inv_det;
		mat[1][3] = + det3_201_023 * inv_det;
		mat[2][3] = - det3_201_013 * inv_det;
		mat[3][3] = + det3_201_012 * inv_det;

		return true;
	}

	void Quaternion::matrix(const Matrix4x4& rotmat)
	{
		float trace = rotmat[0][0] + rotmat[1][1] + rotmat[2][2];
		float mem_inv_x4;
		if(trace>0)
		{
			w = std::sqrt(trace + 1.0f) * 0.5f;

			mem_inv_x4 = 0.25f / w;
			x = ( rotmat[2][1] - rotmat[1][2] ) * mem_inv_x4;
			y = ( rotmat[0][2] - rotmat[2][0] ) * mem_inv_x4;
			z = ( rotmat[1][0] - rotmat[0][1] ) * mem_inv_x4;
		}
		else
		{
			if( rotmat[0][0] > rotmat[1][1] && rotmat[0][0] > rotmat[2][2])
			{
				// rotmat[0][0] is the maximum diagonal term
				x = std::sqrt(rotmat[0][0] - rotmat[1][1] - rotmat[2][2] + 1.0f) * 0.5f;
				mem_inv_x4 = 0.25f / x;
				w = ( rotmat[2][1] - rotmat[1][2] ) * mem_inv_x4;
				y = ( rotmat[0][1] + rotmat[1][0] ) * mem_inv_x4;
				z = ( rotmat[0][2] + rotmat[2][0] ) * mem_inv_x4;
			}
			else if( rotmat[1][1] > rotmat[2][2] )
			{
				// rotmat[1][1] is the maximum diagonal term
				y = std::sqrt(rotmat[1][1] - rotmat[0][0] - rotmat[2][2] + 1.0f) * 0.5f;
				mem_inv_x4 = 0.25f / y;
				w = ( rotmat[0][2] - rotmat[2][0] ) * mem_inv_x4;
				x = ( rotmat[0][1] + rotmat[1][0] ) * mem_inv_x4;
				z = ( rotmat[1][2] + rotmat[2][1] ) * mem_inv_x4;
			}
			else
			{
				// rotmat[2][2] is the maximum diagonal term
				z = std::sqrt(rotmat[2][2] - rotmat[1][1] - rotmat[0][0] + 1.0f) * 0.5f;
				mem_inv_x4 = 0.25f / z;
				w = ( rotmat[1][0] - rotmat[0][1] ) * mem_inv_x4;
				x = ( rotmat[0][2] + rotmat[2][0] ) * mem_inv_x4;
				y = ( rotmat[1][2] + rotmat[2][1] ) * mem_inv_x4;
			}
		}
	}

	Matrix4x4 Quaternion::toMatrix() const
	{
		Matrix4x4	mat;

		float	wx, wy, wz;
		float	xx, yy, yz;
		float	xy, xz, zz;
		float	x2, y2, z2;

		x2 = x + x;	y2 = y + y;	z2 = z + z;

		xx = x * x2;	xy = x * y2;	xz = x * z2;
				yy = y * y2;	yz = y * z2;
						zz = z * z2;
		wx = w * x2;	wy = w * y2;	wz = w * z2;

		mat[0][0] = 1.0f - ( yy + zz );
		mat[0][1] = xy - wz;
		mat[0][2] = xz + wy;
		mat[0][3] = 0;

		mat[1][0] = xy + wz;
		mat[1][1] = 1.0f - ( xx + zz );
		mat[1][2] = yz - wx;
		mat[1][3] = 0;

		mat[2][0] = xz - wy;
		mat[2][1] = yz + wx;
		mat[2][2] = 1.0f - ( xx + yy );
		mat[2][3] = 0;

		mat[3][0] = 0;
		mat[3][1] = 0;
		mat[3][2] = 0;
		mat[3][3] = 1;

		return mat;
	}
}}
