

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Andrei Tchouev


#include "glplaneloader.h"

namespace adore
{
	namespace visual
	{
		GLFeatureLoader::GLFeatureLoader(core::ViewBase* vb)
			:view(vb)
		{
			assert(view);
		}

		void GLFeatureLoader::load(double* points)
		{
			double temp;
			unsigned index = 0;
			core::ViewBase::PointsIterator b = view->getPointsBegin();
			core::ViewBase::PointsIterator e = view->getPointsEnd();
			if(view->isNormalized())
			{
				core::AMatrix trafo = view->getInvTrafo();
				double scale = trafo(0,0);
				double xOff  = trafo(0,2);
				double yOff  = trafo(1,2);
				for(core::ViewBase::PointsIterator iter = b; iter != e; ++iter)
				{
					temp = 1.0 / (**iter)(2);
					points[index*2] = scale * (**iter)(0) * temp + xOff;
					points[index*2+1] = scale * (**iter)(1) * temp + yOff;
					++index;
				}
			}
			else
			{
				for(core::ViewBase::PointsIterator iter = b; iter != e; ++iter)
				{
					temp = 1.0 / (**iter)(2);
					points[index*2] = (**iter)(0) * temp ;
					points[index*2+1] = (**iter)(1) * temp;
					++index;
				}
			}
		}

		unsigned GLFeatureLoader::count()
		{
			return view->getNumFeaturePoints();
		}

		GLCameraLoader::GLCameraLoader(core::Camera* cam)
			:camera(cam)
		{
			assert(camera);
			camera->decompose();
		}

		Vector3D GLCameraLoader::location()
		{
			core::AVector3D eye = camera->getLocation();
			double inv_w = 1.0 / eye(3);
			return Vector3D(eye(0)*inv_w,eye(1)*inv_w,eye(2)*inv_w);
		}

		Quaternion GLCameraLoader::orientation()
		{
			Quaternion rot;
			core::AMatrix rotmat = camera->getRotationMatrix();

			Matrix4x4 temp(	rotmat(0,0),	rotmat(0,1),	rotmat(0,2),	0.0f,
							-rotmat(1,0),	-rotmat(1,1),	-rotmat(1,2),	0.0f,
							-rotmat(2,0),	-rotmat(2,1),	-rotmat(2,2),	0.0f,
							0.0f,			0.0f,			0.0f,			1.0f );

			temp.transposeSelf();
			rot.matrix(temp);
			return rot;
		}

		void GLCameraLoader::load(GLCamera& glcamera)
		{
			Vector3D eye, aim, up;
			load(eye,aim,up);
			glcamera.set(eye,aim,up);
		}

		void GLCameraLoader::load(Vector3D& eye, Vector3D& aim, Vector3D& up)
		{
			core::AMatrix rotmat = camera->getRotationMatrix();
			eye = location();
			aim.set(-rotmat(2,0),-rotmat(2,1),-rotmat(2,2));
			up.set(-rotmat(1,0),-rotmat(1,1),-rotmat(1,2));
			aim += eye;
		}
	}
}
