

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Andrei Tchouev


#ifndef ADORE_GLVISUALIZER_H
#define ADORE_GLVISUALIZER_H

#include "selectnotifier.h"
#include "glnavigator.h"

namespace adore
{
	namespace core
	{
		class NView;
		class ViewBase;
	}

	namespace visual
	{
		/**	Defines visible object types and rendering options
		 */
		enum DISPLAY_FLAGS
		{
			DISPLAY_MESH = 1,				///< Show solid mesh
			DISPLAY_VERTICES = 1<<1,		///< Show model vertices
			DISPLAY_NORMALS = 1<<2,			///< Show face normals
			DISPLAY_LOCAL_FRAME = 1<<3,		///< Show mesh local frame
			DISPLAY_CAMERAS = 1<<4,			///< Show cameras
			DISPLAY_TEXTURE = 1<<5,			///< Enable texturing
			DISPLAY_LIGHTING = 1<<6,		///< Enable lighting
			DISPLAY_PICKING = 1<<7			///< enable picking
		};

		/**	Interface for scene visualization.
		 *	Supports camera and scene transformation.
		 *
		 *	 
		 */
		class GLVisualizer : public SelectNotifier, public GLNavigator
		{
		public:
			/**	Virtual destructor
			 */
			virtual ~GLVisualizer() {};

			/**	Get presentation parametres
			 *
			 *	@return A union of flags from the DISPLAY_FLAGS enumeration
			 */
			virtual int settings() = 0;

			/**	Set presentation parametres
			 *
			 *	Default mode is DISPLAY_MESH | DISPLAY_TEXTURE
			 *
			 *	@param flags One or more flags from the DISPLAY_FLAGS enumeration
			 */
			virtual void settings(int flags) = 0;

			/**	Attach NView
			 *	@param nv NView
			 */
			virtual void attach(core::NView* nv) = 0;

			virtual core::NView* nview()const = 0;

			/**	Select one of several possible views of the NView for displaying
			 *
			 *	All scene transformations and color keying apply to the currently
			 *	selected View only.
			 *
			 *	@param v The View to select
			 *	@param reset if true, the Visualazier shall reset the model transformations for this view
			 */
			virtual void select(const core::ViewBase* const v, const bool reset) = 0;

			/**	Set maximum edge deviation
			 *
			 *	Faces with edges that deviate from the mean edge length by more
			 *	than the given value are not rendered.
			 *
			 *	@param value Deviation value
			 */
			virtual void deviation(float value) = 0;

			/**	Set pen size and color
			 *
			 *	@param color Color of points and lines in RGBA form
			 *	@param size Point size and line thickness value
			 *
			 */
			virtual void pen(const unsigned char color[4], float size) = 0;

			/**	Set background color
			 *
			 *	@param color Background color in RGBA form
			 */
			virtual void bgcolor(const unsigned char color[4]) = 0;
		};
	}
}

#endif	// ADORE_GLVISUALIZER_H
