

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "glvisualizermouse.h"

namespace adore {
namespace visual {

GLVisualizerMouse::GLVisualizerMouse()
{
    camera[0] = 0.0f; //posX
    camera[1] = 0.0f; //posY
    camera[2] = -200.0f; //posZ
    camera[3] = 0.0f;//lookat
    camera[4] = 0.0f;
    camera[5] = 1.0f;
    camera[6] = 0.0f;//upvector
    camera[7] = 1.0f;
    camera[8] = 0.0f;
    centroid[0] = centroid[1] = centroid[2] = 0.0;
    fov = 90;
    cliprangeFar = 3000;
    cliprangeNear = 1;
    maxDist = 200; //distance of camra from scene. the slider chooses a percentage of this value
    
    xAngle = yAngle = 0;
    
    pickBufferSize = 100;
    pickBuffer = new GLuint[pickBufferSize];

}


GLVisualizerMouse::~GLVisualizerMouse()
{
}

void GLVisualizerMouse::init()
{
}

void GLVisualizerMouse::focus()
{
//    glClearColor( 1.0, 1.0, 1.0, 1.0 );
    glClearColor( 0, 0, 0, 0 );
    glEnable(GL_DEPTH_TEST);
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	glDisable(GL_CULL_FACE);
	glShadeModel(GL_SMOOTH);
}

void GLVisualizerMouse::resize(int w, int h)
{
    aspectratio = 1.0f * float(w) / float(h);
    // Reset the coordinate system before modifying
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glViewport( 0, 0, (GLint)w, (GLint)h );
    setPerspective();    
    glMatrixMode(GL_MODELVIEW);
}


void GLVisualizerMouse::setModelView()
{
    glTranslatef(centroid[0], centroid[1], centroid[2]);
//     glRotatef(xAngle, 1.0, 0, 0);
//     glRotatef(yAngle, 0, 1.0, 0);
    double x1 = camera[4] * camera[8] - camera[5] * camera[7];
    double x2 = camera[5] * camera[6] - camera[3] * camera[8];
    double x3 = camera[6] * camera[7] - camera[4] * camera[6];
    glRotatef(xAngle, x1, x2, x3);
    glRotatef(yAngle, camera[6], camera[7], camera[8]);
    
    glTranslatef(-centroid[0], -centroid[1], -centroid[2]);
}

//reset perspective with current values of class members
void GLVisualizerMouse::setPerspective()
{
    GLint viewport[4];
    glGetIntegerv(GL_VIEWPORT, viewport);
    
    glViewport( viewport[0], viewport[1], viewport[2], viewport[3] );
    gluPerspective(fov,aspectratio,cliprangeNear,cliprangeFar);
}

void GLVisualizerMouse::setDistancePercentage(float percent)
{
    if(percent < 0) percent = 0;
    if(percent > 100) percent  = 100;
    camera[2] = centroid[2] - 0.01 * percent * maxDist;
}

void GLVisualizerMouse::pick( int x, int y )
{
    select(x, y);
    /*    
    int slowAngle = 3;
    int fastAngle = 6;
    
    GLint viewport[4];
    glGetIntegerv(GL_VIEWPORT, viewport);
    if(viewport[2] > 60 && viewport[3] > 60)//scrolling only if viewport big enough
    {
        glMatrixMode(GL_MODELVIEW);
        if(x < 10)
        {
            yAngle += fastAngle;
        }
        else if(x < 30)
        {
            yAngle += slowAngle;
        }
        else if(x > viewport[2] - 10)
        {
            yAngle -= fastAngle;
        }
        else if(x > viewport[2] - 30)
        {
            yAngle -= slowAngle;
        }
        
        if(y < 10)
        {
            xAngle += fastAngle;
        }
        else if(y < 30)
        {
            xAngle += slowAngle;
        }
        else if(y > viewport[3] - 10)
        {
            xAngle -= fastAngle;
        }
        else if(y > viewport[3] - 30)
        {
            xAngle -= slowAngle;
        }

        yAngle = yAngle % 360;
        
//         if(xAngle > 89) xAngle = 89;
//         if(xAngle < -89) xAngle = -89;
        xAngle = xAngle % 360;
    } 
    */   
}

void GLVisualizerMouse::paint()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    glLoadIdentity();
    gluLookAt(  camera[0], camera[1], camera[2], camera[3], camera[4], camera[5], camera[6], camera[7], camera[8]);
    setModelView();

    paintScene();
}

void GLVisualizerMouse::slide(const float delta[3])
{
    camera[0] += delta[0];
    camera[1] += delta[1];
    camera[2] += delta[2];
    camera[3] += delta[0];
    camera[4] += delta[1];
    camera[5] += delta[2];
}

void GLVisualizerMouse::rotate(const float axis[3], float angle, bool absolute)
{
    int a = int(angle * 180.0 / M_PI);
    if(axis[1] == 0 && axis[2] == 0)
    {
        xAngle = (xAngle + a) % 360;
    }
    if(axis[0] == 0 && axis[2] == 0)
    {
        yAngle = (yAngle + a) % 360;
    }
    PARAM_UNUSED(absolute);
}

void GLVisualizerMouse::turnTable(float angle, bool absolute)
{
	const float axis[3] = {0, 1, 0};
	rotate(axis, angle, absolute);
}

}}
