

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "calcthread.h"

#include <string>
#include <adrcore/util/scaleshifttrafo.h>

#include <adrcore/viewmatch/viewrelater.h>
#include <adrcore/viewmatch/sequenceviewrelater.h>
#include <adrcore/viewmatch/arrangingviewrelater.h>
#include <adrcore/viewmatch/viewrelatethumbsproxy.h>
#include <adrcore/viewmatch/splitter.h>

#include <adrcore/viewmatch/featurefinder.h>
#include <adrcore/viewmatch/harrisfinder.h>
#include <adrcore/viewmatch/cannyfinder.h>
#include <adrcore/viewmatch/scaledogfinder.h>
#include <adrcore/viewmatch/scaleharrisfinder.h>
#include <adrcore/surf/surffinder.h>

#include <adrcore/viewmatch/linesegmentfinder.h>

#include <adrcore/viewmatch/parallelfeaturefinder.h>
#include <adrcore/util/recommendations.h>

#include <adrcore/viewmatch/descriptorcreator.h>
#include <adrcore/viewmatch/siftdescriptor.h>
#include <adrcore/surf/surfdescriptor.h>

#include <adrcore/viewmatch/matcher.h>
#include <adrcore/viewmatch/kdtreematcher.h>
#include <adrcore/viewmatch/epilinesegmentmatcher.h>
#include <adrcore/viewmatch/flannmatcher.h>

#include <adrcore/viewmatch/tracker.h>
#include <adrcore/klt/klttracker.h>

#include <adrcore/nview/normalizer.h>
#include <adrcore/nview/merger.h>

#include <adrcore/viewmatch/epilinematcher.h>
#include <adrcore/mestimate/filterransac.h>
#include <adrcore/mestimate/filterutil.h>
#include <adrcore/nview/featurepointfilter2.h>

#include <adrcore/viewmatch/linesfilteringmerger.h>

#include <adrcore/nview/tensorupdater.h>
#include <adrcore/nview/point3dcalculator.h>
#include <adrcore/nview/badpointremover.h>
#include <adrcore/nview/baprojective.h>
#include <adrcore/nview/defaultopcollection.h>

#include <adrcore/parameterization/calibrationparamversatile.h>
#include <adrcore/parameterization/softconstraints.h>
#include <adrcore/nview/autocalibrator2.h>
#include <adrcore/nview/oprcalculator.h>

#include <adrcore/nview/automerger.h>
#include <adrcore/nview/parallelautomerger.h>

#include "configconstants.h"
#include <adrcore/util/clock.h>
#include <adrcore/persistence/persist.h>
#include <adrcore/persistence/persist_boost.h>
#include <adrcore/test/assessor.h>

#include <adrcore/util/mathutil.h>
#include <adrcore/imgprocessing/drawing.h>

#include <adrcore/nview/globalbundleadjuster.h>

using namespace std;
using namespace adore::core;
using namespace batch;

CalcThread::CalcThread ( const NViewMap & nv, const vector<ViewT*>& sel,
                         const Config & cfg, bool multithreaded, QObject * parent )
		: QThread ( parent ), stopFlag ( false ),
		nviewmap_ref ( nv ), selection ( sel ),
		config ( cfg ), try_multithreaded ( multithreaded ),
		knitter ( NULL ),
		modifier( ), pmodifier( NULL, figureNumThreads() )
{
}

static void getWH ( NViewMap::ViewIterator vit, NViewMap::ViewIterator vend,
                    int& W, int& H )
{
	W = INT_MAX >> 1;
	H = INT_MAX >> 1;
	while ( vit != vend )
	{
		ViewT* view = dynamic_cast<ViewT*> ( *vit );
		double width = view->getWidth();
		double height = view->getHeight();
		bool norm = view->getNormalized();
		if ( norm )
		{
			double scale = view->getInvTrafo() ( 0, 0 );
			width = ceil ( scale * width );
			height = ceil ( scale * height );
		}
		if ( ( ( int ) width + ( int ) height ) < ( W + H ) )
		{
			W = ( int ) width;
			H = ( int ) height;
		}
		++vit;
	}
}


void CalcThread::run()
{
	size_t selsz = selection.size();
	size_t mapsz = nviewmap_ref.getNumAllViews();

	if ( 3 > selsz && 3 > mapsz ) return;
	else if ( 3 <= selsz && selsz != mapsz )
	{
		std::vector<ViewT*>::const_iterator ii = selection.begin();
		while ( ii != selection.end() )
		{
			nviewmap.addView ( *ii );
			++ii;
		}
	}
	else if ( 3 > mapsz ) return;
	else
	{
		int cnt = nviewmap_ref.getNumEntries();
		if ( !cnt )
		{
			NViewMap::ViewIterator ii = nviewmap_ref.getAllViewsBegin();
			while ( ii != nviewmap_ref.getAllViewsEnd() )
			{
				nviewmap.addView ( *ii );
				++ii;
			}
		}
		NViewMap::EntryIterator nvit = nviewmap_ref.getEntriesBegin();
		while ( nvit != nviewmap_ref.getEntriesEnd() )
		{
			NView* nv = *nvit;
			nviewmap.setNView ( nv->getViewsBegin(), nv->getViewsEnd(), nv );
			++nvit;
		}
	}

	int W, H; getWH ( nviewmap.getAllViewsBegin(), nviewmap.getAllViewsEnd(), W, H );
	HUBER_VALUE = getNormalizedPixel ( 12, W, H );

	timeval t1, t2;	gettimeofday( &t1, NULL );

	size_t cnt = config.getCmdCount();
	for ( size_t i = 0; i < cnt && !stopped(); ++i )
	{
		string cmd_s ( config.getCmd ( i ) );
		Keyword cmd_enum = batch::strToEnum ( cmd_s );

		switch ( cmd_enum )
		{

			case batch::CreateViewRelations:
				createViewRelations ( config, i );
				break;

			case batch::MakePairs:
				makePairs();
				break;

			case batch::CALCFEATURES:
				calcFeatures ( config, i );
				break;

			case batch::FindLines:
				findLines ( config, i );
				break;

			case batch::CALCDESCR:
				calcDescriptors ( config, i );
				break;

			case batch::CALCCORRS:
				calcCorrs ( config, i );
				break;

			case batch::TRACK:
				track ( config, i );
				break;

			case batch::NORMALIZE:
				normalize ( true );
				break;

			case batch::DENORMALIZE:
				normalize ( false );
				break;

			case batch::FILTER:
				filter ( config, i );
				break;

			case batch::FilterAndMergeLines:
				filterAndMergeLines ( config, i );
				break;

			case batch::CALCTENSOR:
				calcTensor();
				break;

			case batch::PROJRECONST:
				projectReconst();
				break;

			case batch::METRICRECONST:
				metricReconst ( config, i );
				break;

			case batch::DEFAULTRECONST:
				defaultReconst( config, i );
				break;

			case batch::GLOBALADJUSTMENT:
				globalAdjustment( config, i );
				break;

			default: break;
		}
	}

	gettimeofday( &t2, NULL );
	fprintf( stderr, "CalcThread: time elapsed while reconstructing: %ld\n", 1000000 * t2.tv_sec + t2.tv_usec - 1000000 * t1.tv_sec - t1.tv_usec );
}

void CalcThread::createViewRelations ( const Config & cfg, size_t idx )
{
	string alg_str ( cfg.getAlgorithm ( idx ) );
	cout << "(CalcThread::createViewRelations): " << alg_str << endl;
	emit sigOpBegin();

	auto_ptr<ViewRelater> relater;
	if ( "ArrangingRelater" == alg_str )
	{
		relater = auto_ptr<ViewRelater>( new ArrangingViewRelater );
		ViewRelateThumbsProxy knitproxy ( relater.get() );
		knitproxy.ViewRelater::createRelations ( nviewmap );
	}
	else
	{
		relater = auto_ptr<ViewRelater>( new SequenceViewRelater );
		relater->ViewRelater::createRelations ( nviewmap );
	}
	emit sigOpEnd();
}

void CalcThread::makePairs()
{
	cout << "(CalcThread::makePairs)" << endl;
	emit sigOpBegin();
	knitter = auto_ptr<ViewKnitter>( new ViewKnitter() );
	knitter->createPairs ( nviewmap );
	knitter->createPlan ( 2/*overlap*/ );
	//create 2-NViews
	Splitter splitter;
	splitter.split ( nviewmap, *knitter );
	emit sigOpEnd();
}

void CalcThread::calcFeatures ( const Config & cfg, size_t idx )
{
	string alg_str ( cfg.getAlgorithm ( idx ) );
	cout << "(CalcThread::calcFeatures): " << alg_str << endl;
	emit sigOpBegin();
	batch::Keyword algorithm = batch::strToEnum ( cfg.getAlgorithm ( idx ) );

        int feat_id, max_pts, hood, min_str, smooths, images, thresh;
        double min_dist, min_fract;
	string descriptor_name;
	cfg.getOptionInt ( idx, "FeatureID", feat_id );
	cfg.getOptionInt ( idx, "MaxPoints", max_pts );
	cfg.getOptionStr ( idx, "Descriptorcreator", descriptor_name );

	cout << "(CalcThread::calcFeatures): descriptorcreator - "
	     << descriptor_name << endl;
	auto_ptr<DescriptorCreator> descr_creator;
	if ( "None" != descriptor_name )
	{
                if ( "Sift" == descriptor_name )
			descr_creator.reset( new SiftDescriptor ( feat_id, 0 ) );
		else if ( "SurfD" == descriptor_name )
			descr_creator.reset( new SurfDescriptor( feat_id, 0 ) );
	}

	auto_ptr<FeatureFinder> finder;
	switch ( algorithm )
	{
		case batch::HARRIS:
			cfg.getOptionInt ( idx, "Hood", hood );
			cfg.getOptionDouble ( idx, "MinDist", min_dist );
			cfg.getOptionInt ( idx, "MinStrength", min_str );
			cfg.getOptionDouble ( idx, "MinFraction", min_fract );
			finder.reset( new HarrisFinder ( feat_id, hood,
			                                 min_dist, min_str, min_fract, max_pts ) );
			break;
		case batch::CANNY:
			cfg.getOptionInt ( idx, "Hood", hood );
			cfg.getOptionDouble ( idx, "MinDist", min_dist );
			cfg.getOptionInt ( idx, "MinStrength", min_str );
			cfg.getOptionDouble ( idx, "MinFraction", min_fract );
			finder.reset( new CannyFinder ( feat_id, hood,
			                                min_dist, min_str, min_fract, max_pts ) );
			break;
		case batch::DOG:
			cfg.getOptionInt ( idx, "Smooths", smooths );
			cfg.getOptionInt ( idx, "Images", images );
			cfg.getOptionInt ( idx, "HarrisThresh", thresh );
			finder.reset( new ScaleDOGFinder ( feat_id, images, thresh, 2000, 3, 0.000001 ) );
			break;
		case batch::SCALE:
		{
			cfg.getOptionInt ( idx, "Smooths", smooths );
			cfg.getOptionInt ( idx, "Images", images );
			cfg.getOptionInt ( idx, "HarrisThresh", thresh );
			cfg.getOptionDouble ( idx, "MinDist", min_dist );
			cfg.getOptionDouble ( idx, "MinFraction", min_fract );
			finder.reset( new ScaleHarrisFinder ( feat_id, smooths, images,
			                                      thresh, max_pts, min_dist, min_fract ) );
			ScaleHarrisFinder *shfinder = static_cast<ScaleHarrisFinder*>( finder.get() );
			shfinder->setSnsRemove ( false );
		}
		break;
		case batch::SURF:
		{
			double thres; cfg.getOptionDouble ( idx, "thres", thres );
			int octaves; cfg.getOptionInt ( idx, "Octaves", octaves );
			int intervals; cfg.getOptionInt ( idx, "Intervals", intervals );
			bool doubleSize; cfg.getOptionBool( idx, "doubleImageSize", doubleSize );
			cfg.getOptionDouble ( idx, "MinDist", min_dist );
			cfg.getOptionDouble ( idx, "MinFraction", min_fract );
			finder.reset( new SurfFinder ( feat_id, octaves, intervals, thres,
			                               doubleSize, max_pts, min_dist, min_fract ) );
		}
		default: break;
	}

	if ( NULL == finder.get() )
		finder = auto_ptr<FeatureFinder>( new HarrisFinder ( 0, 5, 3, 50, 0.001 ) );
	if ( NULL != descr_creator.get() )
		finder->setDescriptorCreator ( descr_creator.get() );

	if ( try_multithreaded )
	{
		ParallelFeatureFinder pff ( finder.get(), figureNumThreads() );
		pff.findFeatures ( nviewmap );
	}
	else
		finder->findFeatures ( nviewmap );

#ifndef NDEBUG
	for ( NView::ViewIterator vit = nviewmap.getAllViewsBegin(); vit != nviewmap.getAllViewsEnd(); ++vit )
	{
		ViewBase *v = *vit;
		if ( v )
		{
			printf ( "%d features in view %p\n", v->getNumAllFeaturePoints(), v );
		}
	}
#endif

	emit sigOpEnd();
}

void CalcThread::findLines( const batch::Config& cfg, size_t idx )
{
	cout << "(CalcThread::findLines): " << endl;
	emit sigOpBegin();

	double minLength = 0.0, maxDist = 0.0;
	int filter_sz = 0;

	cfg.getOptionDouble ( idx, "MinLength", minLength );
	cfg.getOptionDouble ( idx, "MaxDist", maxDist );
	cfg.getOptionInt ( idx, "FilterSize", filter_sz );

	LineSegmentFinder lFinder( minLength, maxDist, filter_sz );

	if ( try_multithreaded )
	{
		ParallelFeatureFinder pff ( &lFinder, figureNumThreads() );
		pff.findFeatures ( nviewmap );
	}
	else
		lFinder.findFeatures ( nviewmap );

	emit sigOpEnd();
}

void CalcThread::calcDescriptors ( const Config & cfg, size_t idx )
{
	string alg_str ( cfg.getAlgorithm ( idx ) );
	cout << "(CalcThread::calcDescriptors): " << alg_str << endl;
	emit sigOpBegin();
	batch::Keyword algorithm = batch::strToEnum ( cfg.getAlgorithm ( idx ) );

	int feat_id, descr_id;
	cfg.getOptionInt ( idx, "FeatureID", feat_id );
	cfg.getOptionInt ( idx, "DescriptorID", descr_id );

	auto_ptr<DescriptorCreator> descr_creator;
	switch ( algorithm )
	{
		case batch::SIFT:
			descr_creator = auto_ptr<DescriptorCreator>( new SiftDescriptor ( feat_id, descr_id ) );
			break;
		case batch::SURFD:
		{
			bool upright; cfg.getOptionBool( idx, "Upright", upright );
			int subregions; cfg.getOptionInt( idx, "Subregions", subregions );
			bool splitSums; cfg.getOptionBool( idx, "SplitSums", splitSums );
			descr_creator = auto_ptr<DescriptorCreator>
			                ( new SurfDescriptor( feat_id, descr_id, upright, subregions, splitSums ) );
			break;
		}
		default:
			printf ( "CalcThread::calcDescriptors: not implemented !?\n" );
			break;
	}

	if ( NULL == descr_creator.get() )
		descr_creator = auto_ptr<DescriptorCreator>( new SiftDescriptor ( 0, 0 ) );

	descr_creator->performAction ( nviewmap );

	emit sigOpEnd();
}

void CalcThread::calcCorrs ( const Config & cfg, size_t idx )
{
	string alg_str ( cfg.getAlgorithm ( idx ) );
	cout << "(CalcThread::calcCorrs): " << alg_str << endl;
	emit sigOpBegin();
	batch::Keyword algorithm = batch::strToEnum ( cfg.getAlgorithm ( idx ) );

	int feat_id, descr_id;
        double dist_thresh;
	std::string strmode;
	KDTREE_MATCH_MODE mode = KDTREE_SINGLE_BEST;

	cfg.getOptionInt ( idx, "FeatureID", feat_id );
	cfg.getOptionInt ( idx, "DescriptorID", descr_id );

	auto_ptr<Matcher> matcher;
	switch ( algorithm )
	{
		case batch::KDTREE:
		{
			cfg.getOptionStr ( idx, "Mode", strmode );
			cfg.getOptionDouble ( idx, "Distthresh", dist_thresh );
			if ( "RatioBest" == strmode ) mode = KDTREE_RATIO_BEST;
			else if ( "BBFRatioBest" == strmode ) mode = KDTREE_BBF_RATIO_BEST;
			double goodRatio; cfg.getOptionDouble( idx, "GoodRatio", goodRatio );
			matcher = auto_ptr<Matcher>( new KDTreeMatcher ( mode ) );
			KDTreeMatcher* m = static_cast<KDTreeMatcher*>( matcher.get() );
			m->setDistThresh ( dist_thresh );
			m->setGoodRatio( ( float )goodRatio );
		}
		break;
		case batch::MatchLines:
			matcher = auto_ptr<Matcher>( new EpiLineSegmentMatcher ( ) );
			break;
		case batch::FLANN:
		{
			double dist_thresh = FlannMatcher::getGoodDistanceThreshold();
			int count_thresh = FlannMatcher::getGoodCountThreshold();
			cfg.getOptionDouble ( idx, "DistanceThreshold", dist_thresh );
			cfg.getOptionInt ( idx, "CountThreshold", count_thresh );
			matcher = auto_ptr<Matcher>( new FlannMatcher ( dist_thresh, count_thresh ) );
			break;
		}
		default: break;
	}
	if ( NULL == matcher.get() )
                matcher = auto_ptr<Matcher>( new KDTreeMatcher ( KDTREE_RATIO_BEST ) );

	if ( try_multithreaded )
		pmodifier.modifyAll ( nviewmap, *matcher );
	else
		modifier.modifyAll ( nviewmap, *matcher );

	emit sigOpEnd();
}

void CalcThread::track ( const batch::Config & cfg, size_t idx )
{
	string alg_str ( cfg.getAlgorithm ( idx ) );
	cout << "(CalcThread::track): " << alg_str << endl;
	emit sigOpBegin();
	batch::Keyword algorithm = batch::strToEnum ( cfg.getAlgorithm ( idx ) );

	auto_ptr<Tracker> tracker;
	switch ( algorithm )
	{
		case batch::KLT_TRACKER:
		{
			int feat_id; cfg.getOptionInt ( idx, "FeatureID", feat_id );
			int max_pts; cfg.getOptionInt ( idx, "MaxPoints", max_pts );

			int mindist; cfg.getOptionInt ( idx, "MinDist", mindist );
			int w_width; cfg.getOptionInt ( idx, "WindowWidth", w_width );
			int w_height; cfg.getOptionInt ( idx, "WindowHeight", w_height );
			string smooth; cfg.getOptionStr ( idx, "SmoothBeforeSelecting", smooth );
			bool smoothing = "Yes" == smooth ? true : false;
			string light; cfg.getOptionStr ( idx, "LightingInsensitive", light );
			bool lighting = "Yes" == light ? true : false;
			int minval; cfg.getOptionInt ( idx, "MinValue", minval );
			string checking; cfg.getOptionStr ( idx, "ConsistencyCheck", checking );
			int check_mode = -1;
			if ( "Translation" == checking ) check_mode = 0;
			else if ( "Similarity" == checking ) check_mode = 1;
			else if ( "Affine" == checking ) check_mode = 2;

			tracker = auto_ptr<Tracker>( new KltTracker ( mindist, w_width, w_height,
			                             smoothing, lighting, check_mode, minval, max_pts, feat_id ) );
		}
		break;
		default: break;
	}

	if ( NULL != tracker.get() )
		tracker->track ( nviewmap );
	emit sigOpEnd();
}

void CalcThread::normalize ( bool norm_denorm )
{
	cout << "(CalcThread::normalize): "
	     << ( norm_denorm ? "Normalizing" : "Denormalizing" ) << endl;
	NView* nv = *nviewmap.getEntriesBegin();
	if ( !nv )
	{
		emit sigOpBegin();
		NViewMap::ViewIterator vit = nviewmap.getAllViewsBegin();
		while ( vit != nviewmap.getAllViewsEnd() )
		{
			if ( norm_denorm ) ( *vit )->normalize();
			else ( *vit )->denormalize();
			++vit;
		}
		emit sigOpEnd();
		return;
	}

	if ( nv->isNormalized() && norm_denorm ) return;
	else if ( !nv->isNormalized() && !norm_denorm ) return;

	emit sigOpBegin();
	Normalizer normer;
	normer.setNormalize ( norm_denorm );
	modifier.modifyAll ( nviewmap, normer );
	emit sigOpEnd();
}

void CalcThread::filter ( const Config & cfg, size_t idx )
{
	if ( NULL == knitter.get() ) return;

	cout << "(CalcThread::filter): " << endl;
	emit sigOpBegin();

	double error2 = getOptThreshFM(), error3 = getOptThreshTT();
	cfg.getOptionDouble ( idx, "Rancac2PixelError", error2 );
	cfg.getOptionDouble ( idx, "Rancac3PixelError", error3 );

	string m;
	cfg.getOptionStr ( idx, "Ransac2OptMethod", m );
	RANSAC_OPT_METHOD method2 = INNER_RANSAC;
	if ( "NONE" == m ) method2 = NONE;
	else if ( "ITERATIVE" == m ) method2 = ITERATIVE;

	cfg.getOptionStr ( idx, "Ransac3OptMethod", m );
	RANSAC_OPT_METHOD method3 = INNER_RANSAC;
	if ( "NONE" == m ) method3 = NONE;
	else if ( "ITERATIVE" == m ) method3 = ITERATIVE;

	int W, H; getWH ( nviewmap.getAllViewsBegin(), nviewmap.getAllViewsEnd(), W, H );
	float opt = getNormalizedPixel ( error2, W, H );

	auto_ptr<NVIRANSACClient> ranser2;
	string err2;
	cfg.getOptionStr ( idx, "Ransac2Error", err2 );
	if ( "Standard" == err2 )
		ranser2 = auto_ptr<NVIRANSACClient>( new FilterFMat( opt, method2 ) );
	else if ( "Sampson" == err2 )
		ranser2 = auto_ptr<NVIRANSACClient>( new FilterFMatSampson( opt, method2 ) );
	else assert( false );

	opt = getNormalizedPixel ( error3, W, H );
	string err3;
	cfg.getOptionStr ( idx, "Ransac3Error", err3 );
	auto_ptr<NVIRANSACClient> ranser3;
	if ( "Standard" == err3 )
		ranser3 = auto_ptr<NVIRANSACClient>( new FilterTTensor( opt, method3 ) );
	else if ( "Sampson" == err3 )
		ranser3 = auto_ptr<NVIRANSACClient>( new FilterTTensorSampson( opt, method3 ) );
	else assert( false );

	EpiLineMatcher guidedMatcher( getNormalizedPixel( 2.0, W, H ), 0.3f );

	FeaturePointFilter2 fpf ( NULL, NULL );
	//filter 2
	fpf.setRanser2 ( ranser2.get() );
	fpf.setRanser3 ( NULL );
	if ( try_multithreaded )
	{
		pmodifier.modifyAll ( nviewmap, fpf );

		//guided re-matching
//		pmodifier.modifyAll( nviewmap, guidedMatcher );
//		pmodifier.modifyAll ( nviewmap, fpf );
	}
	else
	{
		modifier.modifyAll ( nviewmap, fpf );

		//guided re-matching
//		modifier.modifyAll( nviewmap, guidedMatcher );
//		modifier.modifyAll ( nviewmap, fpf );
	}

	bool deleteOld2; cfg.getOptionBool( idx, "deleteOldNViews", deleteOld2 );
	//create Tripplets
	Merger merger;
	mergeStep ( merger, nviewmap, *knitter, deleteOld2 );

	//filter ttensor
	fpf.setRanser2 ( NULL );
	fpf.setRanser3 ( ranser3.get() );
	if ( try_multithreaded )
	{
		pmodifier.modifyAll ( nviewmap, fpf );
	}
	else
	{
		modifier.modifyAll ( nviewmap, fpf );
	}

	emit sigOpEnd();
}

void CalcThread::filterAndMergeLines( const batch::Config& cfg, size_t idx )
{
	cout << "(CalcThread::filterAndMergeLines): " << endl;
	emit sigOpBegin();

	double threshold = 0.8;
	cfg.getOptionDouble ( idx, "Threshold", threshold );

	LinesFilteringMerger merger( threshold );
	merger.merge( nviewmap );

	emit sigOpEnd();
}

void CalcThread::calcTensor()
{
	cout << "(CalcThread::calcTensor): " << endl;
	emit sigOpBegin();

	int numViews = 3;

	TensorUpdater tup;
	tup.setMinAlgebraic ( true );
	if ( try_multithreaded )
		modifier.modifyNumViews ( nviewmap, numViews, tup );
	else
		modifier.modifyNumViews ( nviewmap, numViews, tup );

	emit sigOpEnd();
}

void CalcThread::projectReconst()
{
	cout << "(CalcThread::projectReconst): " << "calculating cameras" << endl;
	emit sigOpBegin();

	int numViews = 3;
	CameraCalculator calcCam;
	if ( try_multithreaded )
		pmodifier.modifyNumViews ( nviewmap, numViews, calcCam );
	else
		modifier.modifyNumViews ( nviewmap, numViews, calcCam );

	cout << "(CalcThread::projectReconst): " << "calculating 3D" << endl;
	Point3DCalculator calc3D;
	calc3D.setMinimizeError ( true );
	if ( try_multithreaded )
		pmodifier.modifyNumViews ( nviewmap, numViews, calc3D );
	else
		modifier.modifyNumViews ( nviewmap, numViews, calc3D );

	BAProjectiveSimple BA;
	BA.setThresh( 0.1 * FLT_EPSILON );
	if ( try_multithreaded )
		pmodifier.modifyNumViews ( nviewmap, numViews, BA );
	else
		modifier.modifyNumViews ( nviewmap, numViews, BA );

	//remove large repro errs
//	int W, H; getWH ( nviewmap.getAllViewsBegin(), nviewmap.getAllViewsEnd(), W, H );
//	float opt = getNormalizedPixel ( getOptThreshFM(), W, H );
//	BadPointRemover bpr2( opt );
//	if ( try_multithreaded )
//		modifier.modifyNumViews ( nviewmap, numViews, bpr2 );
//	else
//		modifier.modifyNumViews ( nviewmap, numViews, bpr2 );

	emit sigOpEnd();
}

void CalcThread::metricReconst ( const Config & cfg, size_t idx )
{
	if ( NULL == knitter.get() ) return;

	cout << "(CalcThread::metricReconst): " << endl;
	emit sigOpBegin();

	static string opts[] =
	{
		"FocalLegth", "AspectRatio",
		"Skew", "PrincipalPointX",
		"PrincipalPointY"
	};
	static string opts1[] =
	{
		"FL", "AR", "SKEW", "PPX", "PPY"
	};
	static adore::core::CALIBRATION_ENTITY o1_enum[] =
	{
		adore::core::FL, adore::core::AR, adore::core::SKEW,
		adore::core::PPX, adore::core::PPY
	};

	Autocalibrator2<CalibrationParamVersatile> calibrator;
	CalibrationParamVersatile &calibParam = calibrator.getCalibrationParam();
	SoftConstraintSet &constSoftConstraints = calibParam.getCONSTSoftConstraints();
	CamParamChoice &metricChoice = calibParam.getChoice();

	PARAMETER_TYPE params[5];
	for ( int i = 0; i < 5; ++i )
	{
		std::string val;
		cfg.getOptionStr ( idx, opts[i], val );
		if ( "Free" == val ) params[i] = FREE;
		else if ( "FreeConst" == val ) params[i] = FREE_CONST;
		else if ( "Soft" == val ) params[i] = SOFT;
		else if ( "SoftConst" == val ) params[i] = SOFT_CONST;
		else if ( "Hard" == val ) params[i] = HARD;
		else if ( "HardConst" == val ) params[i] = HARD_CONST;
		else params[i] = FREE_CONST;
	}

	metricChoice.setFLyMode ( params[0] );
	metricChoice.setARMode ( params[1] );
	metricChoice.setSkewMode ( params[2] );
	metricChoice.setPPxMode ( params[3] );
	metricChoice.setPPyMode ( params[4] );

	double val;
	cfg.getOptionDouble ( idx, "FL", val );
	metricChoice.setFLy ( val );
	cfg.getOptionDouble ( idx, "AR", val );
	metricChoice.setAR ( val );
	cfg.getOptionDouble ( idx, "SKEW", val );
	metricChoice.setSkew ( val );
	cfg.getOptionDouble ( idx, "PPX", val );
	metricChoice.setPPx ( val );
	cfg.getOptionDouble ( idx, "PPY", val );
	metricChoice.setPPy ( val );

	//calibration stuff

	for ( int i = 0; i < 5; ++i )
	{
		if ( SOFT_CONST == params[i] )
		{
			double v1, v2;
			cfg.getOptionDouble ( idx, opts1[i], v1, 0 );
			cfg.getOptionDouble ( idx, opts1[i], v2, 1 );
			constSoftConstraints.addConstraint ( SoftConstraint ( o1_enum[i], v1, v2 ), 0 );
		}
	}
	//calibration stuff - end

//	if ( try_multithreaded )
//		pmodifier.modifyAll( nviewmap, calibrator );
//	else
//		modifier.modifyAll( nviewmap, calibrator );

	string mergingType;
	cfg.getOptionStr ( idx, "MergingType", mergingType );
	bool deleteMergedNViews;
	cfg.getOptionBool( idx, "deleteMergedNViews", deleteMergedNViews );
	if ( "Iterative" == mergingType )
		mergeIterative();
	else
	{
		//instantiate Automerger
		int w, h; maxWH ( w, h );
		AutoMerger automerger ( getNormalizedPixel ( 4, w, h ) );
		ParallelAutoMerger pautomerger ( getNormalizedPixel ( 2, w, h ), getNumThreads() );

		//automerge
		if ( try_multithreaded )
			pautomerger.merge ( nviewmap, *knitter, deleteMergedNViews );
		else
			automerger.merge ( nviewmap, *knitter, deleteMergedNViews );
	}

	//calibrate
	if ( try_multithreaded )
		pmodifier.modifyAll ( nviewmap, calibrator );
	else
		modifier.modifyAll ( nviewmap, calibrator );

	//remove points nor in front of their camera
//	BadPointRemover3 bpr3;
//	if ( try_multithreaded )
//		pmodifier.modifyAll( nviewmap, bpr3 );
//	else
//		modifier.modifyAll( nviewmap, bpr3 );
	emit sigOpEnd();
}

void CalcThread::defaultReconst( const Config & cfg, size_t idx )
{
	cout << "(CalcThread::defaultReconst): " << endl;
	emit sigOpBegin();

	double thresh = getOptThreshFM();
	bool calibrate = true;
	int numCalibs = 1;
	cfg.getOptionDouble ( idx, "Thresh", thresh );
	cfg.getOptionBool ( idx, "Calibrate", calibrate );
	cfg.getOptionInt ( idx, "CalibrationTries", numCalibs );

	int W, H; getWH ( nviewmap.getAllViewsBegin(), nviewmap.getAllViewsEnd(), W, H );
	thresh = getNormalizedPixel ( thresh, W, H );

	DefaultOpCollection doc( thresh );
	doc.setCalibrate( calibrate );
	doc.setNumCalibrationTries( numCalibs );
	if ( try_multithreaded )
	{
		pmodifier.modifyAll ( nviewmap, doc );
		ParallelAutoMerger pautomerger ( thresh, getNumThreads() );
		pautomerger.merge ( nviewmap, *knitter, false/*deleteMergedNViews*/ );
	}
	else
	{
		modifier.modifyAll ( nviewmap, doc );
		AutoMerger automerger ( thresh );
		automerger.merge ( nviewmap, *knitter, false );
	}
	emit sigOpEnd();

}

void CalcThread::mergeIterative( )
{
	vector<NView*> nviews;
	vector<NView*> toDelete;
	NViewMap::EntryIterator nvit = nviewmap.getEntriesBegin();
	while ( nvit != nviewmap.getEntriesEnd() )
	{
		NView* nview = *nvit;
		if ( 3 == nview->getNumViews() )
			nviews.push_back( nview );
		else
			toDelete.push_back( nview );
		++nvit;
	}

	vector<NView *>::iterator it = toDelete.begin();
	while ( it != toDelete.end() )
	{
		NView * nv = *it;
		assert_msg( nv->getViewResetMode() != DELETE_VIEW, "Dangerous VIEW-RESET-Mode in merger. The merging-result will point to nonexisting views." );
		assert_msg( nv->getCorrResetMode() == DELETE_CORR, "Leaky CORR-RESET-Mode. Old corrs wont get deleted." );
		nviewmap.setNView( nv->getViewsBegin(), nv->getViewsEnd(), NULL );
		delete nv;
		++it;
	}

	vector<NView*>::iterator nit = nviews.begin();
	vector<NView*>::iterator nend = nviews.end();
	if ( nit == nend ) return;
	NView* nv1 = *nit;
	nit++;
	if ( nit == nend ) return;

	Merger merger;
	merger.setMinimizeReproErr( true );
	merger.setUseCameras ( true );
	merger.setUsePoints3D ( false );
	BAProjectiveSimple ba;

	while ( nit != nend )
	{
		NView* nv2 = *nit;
		vector<ViewBase*> inA, inAandB, inAorB;
		NView::ViewIterator vit1 = nv1->getViewsBegin();
		while ( vit1 != nv1->getViewsEnd() )
		{
			inA.push_back( *vit1 );
			cout << "inA: " << *vit1 << endl;
			++vit1;
		}
		inAorB.assign( inA.begin(), inA.end() );
		NView::ViewIterator vit2 = nv2->getViewsBegin();
		while ( vit2 != nv2->getViewsEnd() )
		{
			if ( find( inAorB.begin(), inAorB.end(), *vit2 ) == inAorB.end() )
			{
				inAorB.push_back( *vit2 );
				cout << "inAorB: " << *vit2 << endl;
			}
			else
			{
				inAandB.push_back( *vit2 );
				cout << "inAandB: " << *vit2 << endl;
			}
			++vit2;
		}
		NView * result = nviewmap.createNView( inAorB.begin(), inAorB.end() );
		assert( result );
		bool okay = merger.merge( *nv1, *nv2, *result, inA, inAandB );
		if ( !okay )
		{
			DBGOUT( DBGNORMAL, "merger.merge is not okay\n" );
			return;
		}
		NView::CameraIterator camit = result->getCamerasBegin();
		while ( camit != result->getCamerasEnd() )
		{
			Camera* cam = *camit;
			::normalize( cam->getProjMat(), 1.0 );
			++camit;
		}
		ba.modify( *result );
		DBGOUT( DBGNORMAL, "BA after merging %d views needs %d iterations\nresidual(before - after): %.30f - %.30f\n",
		        result->getNumViews(), ba.getLastIterations(), ba.getInitialError(), ba.getLastError() );

		assert_msg( nv1->getViewResetMode() != DELETE_VIEW, "Dangerous VIEW-RESET-Mode in merger. The merging-result will point to nonexisting views." );
		assert_msg( nv1->getCorrResetMode() == DELETE_CORR, "Leaky CORR-RESET-Mode. Old corrs wont get deleted." );
		nviewmap.setNView( nv1->getViewsBegin(), nv1->getViewsEnd(), NULL );
		delete nv1;
		assert_msg( nv2->getViewResetMode() != DELETE_VIEW, "Dangerous VIEW-RESET-Mode in merger. The merging-result will point to nonexisting views." );
		assert_msg( nv2->getCorrResetMode() == DELETE_CORR, "Leaky CORR-RESET-Mode. Old corrs wont get deleted." );
		nviewmap.setNView( nv2->getViewsBegin(), nv2->getViewsEnd(), NULL );
		delete nv2;

		nv1 = result;
		nit++;
	}
}

void CalcThread::maxWH ( int & w, int & h ) const
{
	w = 0; h = 0;
	int max = INT_MAX >> 1;
	int W = max, H = max;
	NViewMap::ViewIterator ii = nviewmap.getAllViewsBegin();
	NViewMap::ViewIterator end = nviewmap.getAllViewsEnd();
	while ( ii != end )
	{
		ViewBase* view = * ( ii );
		if ( ( ( int ) view->getWidth() + ( int ) view->getHeight() ) < ( W + H ) )
		{
			W = ( int ) view->getWidth();
			H = ( int ) view->getHeight();
		}
		++ii;
	}
	if ( max != w || max != h )
	{
		w = W; h = H;
	}
}

const adore::core::NViewMap & CalcThread::getNViewMap()
{
	QMutexLocker ( &this->mutex );
	nviewmap.setNViewResetMode ( KEEP_NVIEW );
	nviewmap.setViewResetMode ( KEEP_VIEW );
	nviewmap.setCorrResetMode ( KEEP_CORR );
	return nviewmap;
}

void CalcThread::globalAdjustment(const batch::Config &cfg, size_t idx)
{
	cout << "(CalcThread::globalAdjustment): " << endl;
	emit sigOpBegin();

	int initial_it = GlobalBundleAdjuster::getGoodInitialIt();
	double thresh_ratio = GlobalBundleAdjuster::getGoodThreshRatio();
	bool use_2 = true;
	cfg.getOptionInt ( idx, "Number_of_initial_iterations", initial_it );
	cfg.getOptionDouble ( idx, "Threshold_ratio", thresh_ratio );
	cfg.getOptionBool ( idx, "Use_2_Views_correspondences", use_2 );

	GlobalBundleAdjuster adjuster(initial_it, thresh_ratio, use_2);
	adjuster.adjust(nviewmap);

	emit sigOpEnd();
}
