

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <boost/spirit/include/classic_core.hpp>
#include <boost/spirit/include/classic_grammar_def.hpp>

//#define BATCH_VERBOSE

#ifdef BATCH_VERBOSE
#include <iostream>
#endif

#include "configparser.h"
#include "configswitch.h"

namespace batch
{

	using namespace std;
	using namespace boost::spirit::classic;
//	using namespace boost::spirit;

	Parser* Parser::static_instance = 0;

	void Parser::init_instance()
	{
		if ( !static_instance )
		{
			static_instance = new Parser;
			static_instance->parseAll();
		}
	}

	template < typename P >
	struct cmd_helper : public grammar< cmd_helper<P> >
	{

		typedef void ( P::*callback )( const char*, const char*, const char* );

		cmd_helper( P* obj, callback c ) : object( obj ), clbk( c ) {}
		P* object; callback clbk;

		void notify( const char* what, char const* str, char const* end ) const
		{
			( object->*clbk )( what, str, end );
		}

		struct hit
		{
			hit( const cmd_helper& c, const char* w ) : cmd( c ), what( w ) {}
			const cmd_helper& cmd;
			const char* what;
			void operator() ( char const* str, char const* end ) const
			{
				cmd.notify( what, str, end );
			}
			void operator()( const char ) const
			{
				cmd.notify( what, 0, 0 );
			}
		};

		template <typename ScannerT> struct definition
					: public grammar_def<rule<ScannerT>, same, same>
		{
			definition( cmd_helper const& self )
			{

				command = str_p( "<command>" ) [ hit( self, "cmd_begin" ) ] >> name >>
				          *algorithm >> *option >> str_p( "</command>" ) [ hit( self, "cmd_end" ) ];

				name = str_p( "<name>" ) >> lexeme_d[ +alpha_p >> *( alnum_p | ch_p( "_" ) ) ] [ hit( self, "name" ) ] >> str_p( "</name>" );

				algorithm = str_p( "<algorithm>" ) [ hit( self, "alg_begin" ) ] >> name >>
				            *option >> str_p( "</algorithm>" ) [ hit( self, "alg_end" ) ];

				option = str_p( "<option>" ) [ hit( self, "opt_begin" ) ] >> type >> name >>
				         value >> *value >> str_p( "</option>" ) [ hit( self, "opt_end" ) ];

				type = str_p( "<type>" ) >> ( str_p( "int" ) | str_p( "double" ) | str_p( "bool" ) | str_p( "enum" ) ) [ hit( self, "type" ) ] >> str_p( "</type>" );

				value = str_p( "<value>" ) >> ( dig_value | str_value ) [ hit( self, "value" ) ]  >> str_p( "</value>" );

				dig_value = lexeme_d[ longest_d[ int_p | real_p ] ];

				enum_ = str_p( "<enum>" ) [ hit( self, "enum_begin" ) ] >> name >>
				        enum_value >> *enum_value >> str_p( "</enum>" ) [ hit( self, "enum_end" ) ];

				enum_value = str_p( "<enum_value>" ) [ hit( self, "enum_val_begin" ) ] >>
				             str_value [ hit( self, "value" ) ] >> str_p( "</enum_value>" ) [ hit( self, "enum_val_end" ) ];

				str_value = lexeme_d[ +alpha_p >> *( alnum_p | ch_p( "_" ) ) ];

				this->start_parsers( command, option, enum_ );

			}
			rule<ScannerT> command, name, algorithm, option, type, value, dig_value, str_value, enum_, enum_value;
		};

	};

	bool Parser::parseAll()
	{
		char const* str = get_init_string();
		cmd_helper<Parser> c_h( this, &Parser::parseAllHit );
		bool done = false, hit = false;;
		parse_info<> info;
		config.cmds.clear();
		config.types.clear();

		do
		{
			info = boost::spirit::classic::parse( str, c_h.use_parser<0>(), space_p );
//			info = boost::spirit::parse( str, c_h.use_parser<0>(), space_p );

			if ( info.full || info.hit )
			{
#ifdef BATCH_VERBOSE
				cout << "(batch::parseAll): OK\n";
#endif
				if ( !info.full )
				{
					str = info.stop;
					hit = true;
				}
				else done = true;
			}
			else
			{
				if ( hit )
				{
#ifdef BATCH_VERBOSE
					cout << "(batch::parseAll): INTERMEDIATE!\n";
#endif
				}
				else
				{
#ifdef BATCH_VERBOSE
					cout << "(batch::parseAll): FAILED!\n";
					cout << "stopped at: \"" << info.stop << "\"\n";
#endif
				}
				return false;
			}
		}
		while ( !done );

		done = hit = false;
		str = get_enum_string();
		do
		{
			info = boost::spirit::classic::parse( str, c_h.use_parser<2>(), space_p );
//			info = boost::spirit::parse( str, c_h.use_parser<2>(), space_p );

			if ( info.full || info.hit )
			{
#ifdef BATCH_VERBOSE
				cout << "(batch::parseAll): OK\n";
#endif
				if ( !info.full )
				{
					str = info.stop;
					hit = true;
				}
				else done = true;
			}
			else
			{
				if ( hit )
				{
#ifdef BATCH_VERBOSE
					cout << "(batch::parseAll): INTERMEDIATE!\n";
#endif
				}
				else
				{
#ifdef BATCH_VERBOSE
					cout << "(batch::parseAll): FAILED!\n";
					cout << "stopped at: \"" << info.stop << "\"\n";
#endif
				}
				return false;
			}
		}
		while ( !done );
		return true;
	}

	void Parser::parseAllHit( const char * what, char const * str, char const * end )
	{
		switch_helper = string( str, end );
		typedef case_< Parser > _case;
		static switch_< Parser > _switch =
		        switch_< Parser > ()
		        (  _case( "cmd_begin", &Parser::cmd_begin )  )
		        (  _case( "alg_begin", &Parser::alg_begin )  )
		        (  _case( "opt_begin", &Parser::opt_begin )  )
		        (  _case( "opt_end", &Parser::opt_end )  )
		        (  _case( "alg_end", &Parser::alg_end )  )
		        (  _case( "cmd_end", &Parser::cmd_end )  )
		        (  _case( "enum_begin", &Parser::enum_begin )  )
		        (  _case( "enum_val_begin", &Parser::enum_val_begin )  )
		        (  _case( "enum_val_end", &Parser::enum_val_end )  )
		        (  _case( "enum_end", &Parser::enum_end )  )
		        (  _case( "type", &Parser::_type )  )
		        (  _case( "name", &Parser::_name )  )
		        (  _case( "value", &Parser::_value )  );
		_switch( this, what );
	}

	void Parser::cmd_begin()
	{
#ifdef BATCH_VERBOSE
		cout << "cmd_begin\n";
#endif
		next = "cmd_name";
		cur_cmd.algs.clear();
		cur_cmd.opts.clear();
	}

	void Parser::cmd_end()
	{
#ifdef BATCH_VERBOSE
		cout << "cmd_end\n";
#endif
		config.cmds.push_back( cur_cmd );
	}

	void Parser::alg_begin()
	{
#ifdef BATCH_VERBOSE
		cout << "alg_begin\n";
#endif
		next = "alg_name";
		cur_alg.opts.clear();
	}

	void Parser::alg_end()
	{
#ifdef BATCH_VERBOSE
		cout << "alg_end\n";
#endif
		cur_cmd.algs.push_back( cur_alg );
	}

	void Parser::opt_begin()
	{
#ifdef BATCH_VERBOSE
		cout << "opt_begin\n";
#endif
		cur_opt.values.clear();
		if ( "alg_or_opt" == next )
			next = "cmd_opt_type";
		else if ( "alg_opt" == next )
			next = "alg_opt_type";
	}

	void Parser::opt_end()
	{
#ifdef BATCH_VERBOSE
		cout << "opt_end\n";
#endif
		if ( "cmd_opt_end" == next )
		{
			cur_cmd.opts.push_back( cur_opt );
			next = "alg_or_opt";
		}
		else if ( "alg_opt_end" == next )
		{
			cur_alg.opts.push_back( cur_opt );
			next = "alg_opt";
		}
	}

	void Parser::_type()
	{
#ifdef BATCH_VERBOSE
		cout << "type: " << switch_helper << "\n";
#endif
		cur_opt.type = switch_helper;
		if ( "cmd_opt_type" == next )
			next = "cmd_opt_name";
		else if ( "alg_opt_type" == next )
			next = "alg_opt_name";
	}

	void Parser::_name()
	{
#ifdef BATCH_VERBOSE
		cout << "name: " << switch_helper << "\n";
#endif
		if ( "cmd_name" == next )
		{
			cur_cmd.name = switch_helper;
			next = "alg_or_opt";
		}
		else if ( "alg_name" == next )
		{
			cur_alg.name = switch_helper;
			next = "alg_opt";
		}
		else if ( "enum_name" == next )
		{
			cur_enum.name = switch_helper;
			next = "enum_val";
		}
		else
		{
			cur_opt.name = switch_helper;
			if ( "cmd_opt_name" == next )
				next = "cmd_opt_value";
			else if ( "alg_opt_name" == next )
				next = "alg_opt_value";
		}
	}

	void Parser::_value()
	{
#ifdef BATCH_VERBOSE
		cout << "value: " << switch_helper << "\n";
#endif
		cur_opt.values.push_back( switch_helper );
		if ( "cmd_opt_value" == next )
			next = "cmd_opt_end";
		else if ( "alg_opt_value" == next )
			next = "alg_opt_end";
		else if ( "enum_val" == next )
			cur_enum.enums.push_back( switch_helper );
	}

	void Parser::enum_begin()
	{
#ifdef BATCH_VERBOSE
		cout << "enum_begin\n";
#endif
		cur_enum.enums.clear();
		next = "enum_name";
	}

	void Parser::enum_end()
	{
#ifdef BATCH_VERBOSE
		cout << "enum_end\n";
#endif
		config.types.push_back( cur_enum );
	}

	void Parser::enum_val_begin()
	{
//		cout << "enum_val_begin\n";
	}

	void Parser::enum_val_end()
	{
//		cout << "enum_val_end\n";
	}

	vector< string > Parser::getCommands()
	{
		init_instance();
		vector< string > retval;
		int cnt = static_instance->config.cmds.size();
		for ( int i = 0; i < cnt; ++i )
			retval.push_back( static_instance->config.cmds[i].name );
		return retval;
	}

	bool Parser::hasCmd( const string & cmd )
	{
		init_instance();
		int idx = cmdIndex( cmd );
		if ( 0 > idx ) return false;
		return true;
	}

	vector< string > Parser::getAlgorithms( const string & cmd )
	{
		init_instance();
		vector< string > retval;
		int cnt = static_instance->config.cmds.size();
		int cmd_index = -1;
		for ( int i = 0; i < cnt; ++i )
		{
			if ( cmd == static_instance->config.cmds[i].name )
			{
				cmd_index = i; break;
			}
		}
		if ( -1 == cmd_index ) return retval;
		cnt = static_instance->config.cmds[cmd_index].algs.size();
		for ( int i = 0; i < cnt; ++i )
			retval.push_back( static_instance->config.cmds[cmd_index].algs[i].name );
		return retval;
	}

	int Parser::algIndex( const string & cmd, const string & alg )
	{
		init_instance();
		int alg_index = -1;
		int cmd_index = cmdIndex( cmd );
		if ( -1 == cmd_index ) return alg_index;

		int cnt = static_instance->config.cmds[cmd_index].algs.size();
		for ( int i = 0; i < cnt; ++i )
			if ( alg == static_instance->config.cmds[cmd_index].algs[i].name )
				return i;
		return alg_index;
	}

	string Parser::getOptions( const string & cmd )
	{
		init_instance();
		string retval;
		vector< string > commands( getCommands() );
		int cnt = commands.size();
		int cc = -1;
		for ( int i = 0; i < cnt; ++i )
		{
			if ( cmd == commands[i] )
			{
				cc = i; break;
			}
		}
		if ( -1 == cc ) return retval;
		_cmd_ command = static_instance->config.cmds[cc];
		int ocnt = command.opts.size();
		for ( int i = 0; i < ocnt; ++i )
		{
			retval += "<option>";
			retval += "<type>" + command.opts[i].type + "</type>";
			retval += "<name>" + command.opts[i].name + "</name>";
			int vcnt = command.opts[i].values.size();
			for ( int j = 0; j < vcnt; ++j )
				retval += "<value>" + command.opts[i].values[j] + "</value>";
			retval += "</option>\n";
		}
		return retval;
	}

	string Parser::getOptions( const string & cmd, const string & alg )
	{
		init_instance();
		string retval;
		vector< string > commands( getCommands() );
		int cnt = commands.size();
		int cc = -1;
		for ( int i = 0; i < cnt; ++i )
		{
			if ( cmd == commands[i] )
			{
				cc = i; break;
			}
		}
		if ( -1 == cc ) return retval;

		vector< string > algorithms( getAlgorithms( cmd ) );
		int acnt = algorithms.size();
		int aa = -1;
		for ( int i = 0; i < acnt; ++i )
		{
			if ( alg == algorithms[i] )
			{
				aa = i; break;
			}
		}
		if ( -1 == aa ) return retval;

		_alg_ algorithm = static_instance->config.cmds[cc].algs[aa];
		int ocnt = algorithm.opts.size();
		for ( int i = 0; i < ocnt; ++i )
		{
			retval += "<option>";
			retval += "<type>" + algorithm.opts[i].type + "</type>";
			retval += "<name>" + algorithm.opts[i].name + "</name>";
			int vcnt = algorithm.opts[i].values.size();
			for ( int j = 0; j < vcnt; ++j )
				retval += "<value>" + algorithm.opts[i].values[j] + "</value>";
			retval += "</option>\n";
		}
		return retval;
	}

	vector< string > Parser::getEnums( const string & name )
	{
		init_instance();
		vector< string > retval;
		int cnt = static_instance->config.types.size();
		for ( int i = 0; i < cnt; ++i )
		{
			if ( name == static_instance->config.types[i].name )
				return static_instance->config.types[i].enums;
		}
		return retval;
	}

	bool Parser::parseOptions( const string & optstr, vector< _opt_> & options )
	{
		char const* str = optstr.c_str();
		cur_options.clear();
		cur_cmd.opts.clear();
		next = "alg_or_opt";

		cmd_helper<Parser> c_h( this, &Parser::parseOptionsHit );
		bool done = false, has_hit = false;
		parse_info<> info;
		do
		{
			info = boost::spirit::classic::parse( str, c_h.use_parser<1>(), space_p );
//			info = boost::spirit::parse( str, c_h.use_parser<1>(), space_p );

			if ( info.full || info.hit )
			{
#ifdef BATCH_VERBOSE
				cout << "(batch::parseOptions): OK\n";
#endif
				if ( !info.full )
				{
					str = info.stop;
					has_hit = true;
				}
				else done = true;
			}
			else
			{
				if ( has_hit )
				{
#ifdef BATCH_VERBOSE
					cout << "(batch::parseAll): INTERMEDIATE!\n";
#endif
				}
				else
				{
#ifdef BATCH_VERBOSE
					cout << "(batch::parseAll): FAILED!\n";
					cout << "stopped at: \"" << info.stop << "\"\n";
#endif
					return false;
				}
				done = true;
			}
		}
		while ( !done );
		options = cur_cmd.opts;
		return true;
	}

	void Parser::parseOptionsHit( const char * what, char const * str, char const * end )
	{
		switch_helper = string( str, end );
		typedef case_< Parser > _case;
		static switch_< Parser > _switch =
		        switch_< Parser > ()
		        (  _case( "opt_begin", &Parser::opt_begin )  )
		        (  _case( "opt_end", &Parser::opt_end )  )
		        (  _case( "type", &Parser::_type )  )
		        (  _case( "name", &Parser::_name )  )
		        (  _case( "value", &Parser::_value )  );
		_switch( this, what );
	}

	bool Parser::parseConfig( const string & cfgstr, _cfg_ & cfg )
	{
		char const* str = cfgstr.c_str();
		config.cmds.clear();
		cmd_helper<Parser> c_h( this, &Parser::parseAllHit );
		bool done = false, has_hit = false;
		parse_info<> info;
		do
		{
			info = boost::spirit::classic::parse( str, c_h.use_parser<0>(), space_p );
//			info = boost::spirit::parse( str, c_h.use_parser<0>(), space_p );

			if ( info.full || info.hit )
			{
#ifdef BATCH_VERBOSE
				cout << "(batch::parseAll): OK\n";
#endif
				if ( !info.full )
				{
					has_hit = true;
					str = info.stop;
				}
				else done = true;
			}
			else
			{
				if ( has_hit )
				{
#ifdef BATCH_VERBOSE
					cout << "(batch::parseAll): INTERMEDIATE!\n";
#endif
				}
				else
				{
#ifdef BATCH_VERBOSE
					cout << "(batch::parseAll): FAILED!\n";
					cout << "stopped at: \"" << info.stop << "\"\n";
#endif
					return false;
				}
				done = true;
			}
		}
		while ( !done );
		cfg = config;
		return true;
	}

	void Parser::makeConform( const string & cfgstr, _cfg_ & cfg )
	{
		init_instance();
		parseConfig( cfgstr, cfg );
		if ( cfg.cmds.empty() ) return;
		uint cnt = cfg.cmds.size();
		for ( uint i = 0; i < cnt; ++i )
		{
			///**** CHECK COMMAND ****///
			string cmd_name( cfg.cmds[i].name );
			int c_idx = cmdIndex( cmd_name );
			if ( 0 > c_idx )
			{
				cfg.cmds.erase( cfg.cmds.begin() + i );
				cnt--; continue;
			}

			///**** CHECK ALGORITHM ****///
			bool has_alg = false;
			if ( !cfg.cmds[i].algs.empty() )
			{
				if ( 1 < cfg.cmds[i].algs.size() )
					cfg.cmds[i].algs.erase( cfg.cmds[i].algs.begin() + 1,
					                        cfg.cmds[i].algs.end() );
				has_alg = true;
			}
			const _cfg_& def_cfg( static_instance->config );
			if ( def_cfg.cmds[c_idx].algs.empty() )
			{
				if ( has_alg )
				{
					has_alg = false;
					cfg.cmds[i].algs.clear();
				}
			}
			else
			{
				if ( !has_alg )
				{
					cfg.cmds[i].algs.push_back( def_cfg.cmds[c_idx].algs[0] );
					continue;
				}
			}

			///**** CHECK OPTIONS ****///
			if ( has_alg )
			{
				_alg_& cur_alg( cfg.cmds[i].algs[0] );
				string alg_name( cur_alg.name );
				int a_idx = algIndex( cmd_name, alg_name );
				if ( 0 > a_idx )
				{
					cur_alg = def_cfg.cmds[c_idx].algs[0];
					continue;
				}
				else
				{
					vector< _opt_ > opts;
					const _alg_& def_alg = def_cfg.cmds[c_idx].algs[a_idx];
					uint o_cnt = def_alg.opts.size();
					for ( uint j = 0; j < o_cnt; ++j )
					{
						_opt_ opt( def_alg.opts[j] );
						string o_name( opt.name );
						for ( uint k = 0; k < cur_alg.opts.size(); ++k )
						{
							if ( o_name == cur_alg.opts[k].name )
							{
								if ( opt.type == cur_alg.opts[k].type )
								{
									uint max = opt.values.size() < cur_alg.opts[k].values.size()
									           ? opt.values.size() : cur_alg.opts[k].values.size();
									for ( uint l = 0; l < max; ++l )
										opt.values[l] = cur_alg.opts[k].values[l];
								}
								cur_alg.opts.erase( cur_alg.opts.begin() + k );
								break;
							}
						}
						opts.push_back( opt );
					}
					cur_alg.opts = opts;
				}
			}
			else
			{
				_cmd_& cur_cmd( cfg.cmds[i] );
				vector< _opt_ > opts;
				const _cmd_& def_cmd = def_cfg.cmds[c_idx];
				uint o_cnt = def_cmd.opts.size();
				for ( uint j = 0; j < o_cnt; ++j )
				{
					_opt_ opt( def_cmd.opts[j] );
					string o_name( opt.name );
					for ( uint k = 0; k < cur_cmd.opts.size(); ++k )
					{
						if ( o_name == cur_cmd.opts[k].name )
						{
							if ( opt.type == cur_cmd.opts[k].type )
							{
								uint max = opt.values.size() < cur_cmd.opts[k].values.size()
								           ? opt.values.size() : cur_cmd.opts[k].values.size();
								for ( uint l = 0; l < max; ++l )
									opt.values[l] = cur_cmd.opts[k].values[l];
							}
							cur_cmd.opts.erase( cur_cmd.opts.begin() + k );
							break;
						}
					}
					opts.push_back( opt );
				}
				cur_cmd.opts = opts;
			}
		}
	}

	int Parser::cmdIndex( const string & cmd )
	{
		init_instance();
		int cnt = static_instance->config.cmds.size();
		for ( int i = 0; i < cnt; ++i )
			if ( cmd == static_instance->config.cmds[i].name )
				return i;
		return -1;
	}

} //namespace batch
