

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef CONTROLLER_H
#define CONTROLLER_H

#include <QObject>
#include <QStringList>
#include <QDir>
#include <QMutex>

#include <adrcore/types/nviewmap.h>
#include <adrcore/viewmatch/nviewdependencygraph.h>

#include "constants.h"

#include "configconstants.h"
#include "config.h"

#include <adrvisual/selectlistener.h>

using namespace std;
using namespace adore::core;
using namespace adore::visual;

class MainWindow;
class FeaturesView;
class GLWidget;
class NViewList;
class CalcThread;

/**
 
*/

class Controller : public QObject, public SelectListener
{
		Q_OBJECT

	public:
		Controller( MainWindow* f, const string& configFile );
		~Controller() {}

		void addViews( const QStringList& fileList, const QDir& dir );
		void to3D( bool multithreaded = false );

		void save( const QString& s, bool simple = false, bool embed = false, bool descr = true );
		void saveSQL( const QString& s );
		void load( const QString& s );
		void loadSQL( const QString& s );

		void point3DChosen( AVector3D *pt );
		void cameraChosen( Camera *cam );

		void resetAll();
		void resetAllButViews();
		void resetHighlights();

	public slots:
		void nViewSelected( NView* nview );
		void nViewDoubleClicked( NView* nview );
		void toStandardCamera();

		/**
			toggles whether touched 3D points get deleted
		*/
		void deletePoint3D( bool );

	private slots:
		void slotViewSelected( const ViewT * const v, bool selected );
		void slotCameraSelected( const ViewT * const v );

		void slotThreadFinished();

	private:
		NViewMap nviewmap;

		MainWindow* mainwin;
		FeaturesView* fView;
		GLWidget* glwidget;
		NViewList* nViewSetList;

		NView* lastSelected;
		CalcThread* thread;
		QMutex mutex;

		bool delete3DPoints;

		void addViews( NViewMap& nviewmap );
		void fillNViewSetTreeView( NViewMap& nviewmap );
		NView* fillNViewSetTreeView( NViewDependencyGraph& nvdg, NViewDependencyGraph::Vertex nview, NView* parent, NView* below = 0 );

		void resetViews();
		void resetMap();
		void resetGL();

		void setDeviation( int ticks );
		void setPointSize( int ticks );
};

#endif
