

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef DIRMODEL_H
#define DIRMODEL_H

#include <QtGui/QDirModel>
#include <QtGui/QSortFilterProxyModel>

#include "filteringiconprovider.h"

class DirModel : public QSortFilterProxyModel
{
public:

	enum Mode { Files, Dirs };

	DirModel( Mode mode );

	QModelIndex index( const QString& path ) const
	{
		return mapFromSource( model.index( path ) );
	}

	QModelIndex index( int row, int column ) const
	{
		return model.index(row, column);
	}

	bool isDir( const QModelIndex& index ) const
	{
		return model.isDir( mapToSource( index ) );
	}
	void setFileNameFilters( const QStringList& filters )
	{
		_filters = filters;
		if(workMode == Dirs)
			model.setIconProvider(new FilteringIconProvider(filters));
	}
	QString toPath( const QModelIndex& index ) const
	{
		return model.filePath( mapToSource( index ) );
	}
	void refresh( const QModelIndex& parent = QModelIndex() )
	{
		model.refresh( mapToSource( parent ) );
	}

private:
	QDirModel model;
	Mode workMode;

	QStringList _filters;
	virtual bool filterAcceptsRow( int source_row, const QModelIndex& source_parent ) const;

	QVariant data ( const QModelIndex & proxyIndex, int role = Qt::DisplayRole ) const;
};

#endif // DIRMODEL_H
