

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef FEATURESSCENE_H
#define FEATURESSCENE_H

#include <QtCore/QObject>
#include <QtCore/QRectF>
#include <QtCore/QPointF>

#include "constants.h"

#include <map>

#include <boost/shared_ptr.hpp>

class PixmapPlane;
class FeaturesLayer;
class CorrGL2D;
class CorrLayerItem;

namespace adore
{
	namespace core
	{
		class ViewBase;
		class PointCorr;
		class NView;
	}
}

class FeaturesScene
{
	public:
		FeaturesScene( QObject* parent );
		~FeaturesScene();
		void reset();
		void resetPreservingViews();
		void removeEpipolarLines();

		void addView( ViewT* view );
		void getSelectedViews( std::vector<ViewT*>& views ) const;
		ViewT* getFocusedView() const;
		uint numSelectedViews() const { return orderedImages.size(); }
		void highlightView( const adore::core::ViewBase* v );
		void focusView( const adore::core::ViewBase* v );
		void selectView( const adore::core::ViewBase* v, bool on );

		void paint( const QRectF& r );
		const QRectF sceneRect() const { return rect; }

		void mouseClicked( const QPointF&, Qt::MouseButton button );
		void mouseCtrlClicked( const QPointF& pos );

		void featuresChanged();
		void nViewSelected( adore::core::NView* nv );
		void highlightPointCorr( adore::core::PointCorr* c );

		void setShowFeats( bool on );
		void setShowLines( bool on );
		void setShowPointCorrs( bool on );
		void setShowLineCorrs( bool on );

	private:
		enum { imagePadding = 8 };

		QObject* parent;
		QRectF rect;
		bool featsShown, linesShown, pCorrsShown, lCorrsShown;

		typedef boost::shared_ptr<PixmapPlane> ImagePtr;
		typedef std::map<const adore::core::ViewBase*, ImagePtr > View2Image;
		typedef std::vector< ImagePtr > OrderedImagesT;
		typedef View2Image::iterator I_iterator;
		typedef View2Image::const_iterator I_const_iterator;
		View2Image imagePlanes;
		OrderedImagesT orderedImages;
		ImagePtr lastFocusedImage;
		ImagePtr lastHighlightedImage;

		typedef boost::shared_ptr<FeaturesLayer> FeaturesPtr;
		typedef std::map<const adore::core::ViewBase*, FeaturesPtr > View2Features;
		typedef View2Features::iterator FL_iterator;
		typedef View2Features::const_iterator FL_const_iterator;
		View2Features featureLayers;

		typedef boost::shared_ptr<CorrLayerItem> CorrsPtr;
		typedef std::map<const adore::core::NView*, CorrsPtr > NView2Corrs;
		typedef NView2Corrs::iterator CL_iterator;
		typedef NView2Corrs::const_iterator CL_const_iterator;
		NView2Corrs corrLayers;
		adore::core::NView* lastNView;
		ViewT* firstView, * secondView;
		QPointF ptInFirstView, ptInSecondView;

		boost::shared_ptr<CorrGL2D> lastSelectedCorr;

		QPointF nextViewPos( ImagePtr image ) const;
		void calcSceneRect();

		void updateEpipolarLines();
};

#endif // FEATURESSCENE_H
