

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef FEATURESVIEW_H
#define FEATURESVIEW_H

#include <QGLWidget>

#include "constants.h"

#include <vector>

#include <boost/shared_ptr.hpp>

class FeaturesScene;
class SliderGL;
class ImageGL;
class ScrollBarGL;
class ControlGL;

namespace adore
{
	namespace core
	{
		class NView;
		class PointCorr;
		class ViewBase;
	}
}

class FeaturesView : public QGLWidget
{
		Q_OBJECT

	enum { ZoomMin = 0, ZoomMax = 1000, Zoom1 = 600 };
	static double MinZoomFactor, MaxZoomFactor;

	public:
		FeaturesView( QWidget* parent = NULL );
		void reset();
		void resetPreservingViews();

		void addView( ViewT* v );
		void getSelectedViews( std::vector<ViewT*>& views ) const;
		ViewT* getFocusedView() const;
		void highlightView( const adore::core::ViewBase* v );
		void focusView( const adore::core::ViewBase* v );
		void selectView( const adore::core::ViewBase* v, bool on );

		void calculateGoodZoom();

		void featuresChanged();
		void nViewSelected( adore::core::NView* nv );
		void highlightPointCorr( adore::core::PointCorr* c );

	public slots:
		void slotSelected( const ViewT* const view, bool on );
		void slotFocused( const ViewT* const view );
		void removeEpipolarLines();
		void setShowFeats( bool on );
		void setShowLines( bool on );
		void setShowPointCorrs( bool on );
		void setShowLineCorrs( bool on );

	private slots:
		void zoom( int n );
		void scrollHorizontal(double pos);
		void scrollVertical(double pos);

	signals:
		void sigViewSelected( const ViewT* const, bool );
		void sigCameraSelected( const ViewT* const );

	private:
		void initializeGL();
		void resizeGL( int width, int height );
		void paintGL();

		void resizeGui( int width, int height );
		void paintControls();

		FeaturesScene* scene;
		double factor;

		void mousePressEvent ( QMouseEvent * event );
		void mouseMoveEvent ( QMouseEvent * event );
		void mouseReleaseEvent ( QMouseEvent * event );
		void wheelEvent ( QWheelEvent * event );

		QPointF toScene( const QPointF& p ) const;

		boost::shared_ptr<ImageGL>  m_image_zoom;
		boost::shared_ptr<SliderGL>  m_zoom_slider;
		boost::shared_ptr<ScrollBarGL> m_hscroll, m_vscroll;
		ControlGL * m_focused;
};

#endif // FEATURESVIEW_H
