

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <iostream>
using std::cout;
using std::endl;
#include <QtCore/QDir>
#include <QtGui/QPainter>

#include "filteringiconprovider.h"

FilteringIconProvider::FilteringIconProvider( const QStringList & filters ) :
		_filters(filters),
		can_filter(false)
{
	initialize();
}

QIcon FilteringIconProvider::icon ( const QFileInfo & info ) const
{
	if(!info.isDir() || !can_filter || _filters.empty())
		return QFileIconProvider::icon(info);

	QDir dir(info.absoluteFilePath());
	QStringList filtered( dir.entryList( _filters, QDir::Files | QDir::NoSymLinks | QDir::Readable ) );
	if ( filtered.isEmpty() )
		return iconFalse;
	else
		return iconTrue;
}

void FilteringIconProvider::initialize()
{
	QIcon dirIcon(QFileIconProvider::icon(Folder));

	QIcon icon( dirIcon );
	for ( int i = 0; i < 2; ++i ) {
		QPixmap p( icon.pixmap( 16 ) );
		QSizeF sz = p.size();
		QPainter painter( &p );
		painter.setRenderHints( QPainter::Antialiasing );
		double ext = sz.width() / 3;
		QRectF rect( ext, ext, ext, ext );
		if ( i == 0 ) painter.setBrush( Qt::green );
		else painter.setBrush( Qt::red );
		painter.drawEllipse( rect );
		if ( i == 0 ) iconTrue = QIcon( p );
		else iconFalse = QIcon( p );
	}

	can_filter = true;
}
