

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin
/// Author: Klaus Haeming


#ifndef NVIEWGLWIDGET_H
#define NVIEWGLWIDGET_H

#include <QGLWidget>
#include <QTimer>

#include "cameratourist.h"

namespace adore
{
	namespace visual
	{
		class GLVisualizer;
		class SelectListener;
	}
	namespace core
	{
		class NView;
	}
}

using namespace adore::visual;
using namespace adore::core;

#include <boost/shared_ptr.hpp>
#include <adrcore/types/view.h>

class GroupBoxGL;
class ExclusiveOptionsGroup;
class SliderGL;
class ImageGL;
class ControlGL;
class CheckButtonGL;

/**
 
*/
class GLWidget : public QGLWidget
{
		Q_OBJECT
	public:
		enum vis_type { vis_dots, vis_delaunay };

		GLWidget( const QGLFormat & format = QGLFormat(), GLWidget::vis_type vis = vis_delaunay, QWidget* parent = 0 );
		~GLWidget() {}

		void setListener( SelectListener* li );
		void set_GL_Config( const std::string& cfg );
		void nextVisualizer();
		vis_type visualizerType()const { return vis_t; }
		const QString& visualizerName()const;
		GLVisualizer* visualizer() { return _visualizer; }


		QSize minimumSize () const {return minSize;}
		int minimumWidth () const {return minSize.width();}
		int minimumHeight () const {return minSize.height();}

		void updateKeyActions( QKeyEvent *e, bool set );
		void nViewSelected( NView *nv, ViewBase *v = NULL, const bool reset = true );
		
		void setPointSize( int ticks );
		void setDeviation( int ticks );

	public slots:
		void slotToggleShowCams();
		void slotCamsOFF();
		void slotSetStandardView();
		void slotToggleAnimation();
		void dotSliderMoved( int i );
		void depthSliderMoved( int i );

	private slots:
		void moveModel();
		void slotOptionActivated();

	signals:
		void deletePoint3D( bool );
		void standardViewRequested();

	private:
		QSize minSize;
		float mouse_sensitivity;
		QPoint mouse_pos;
		int wheel_pos;

		vis_type vis_t;
		GLVisualizer* _visualizer;
		std::string config_file;
		SelectListener* listener;

		QTimer timer;
		Qt::MouseButton buttons;
		int numKeysPressed;
		bool rotateUp;
		bool rotateDown;
		bool rotateLeft;
		bool rotateRight;
		bool slideForward;
		bool slideLeft;
		bool slideBack;
		bool slideRight;

		bool glInitialized;

		//tourism variables
		CameraTourist tourist;
		bool cameraAction;
		MovementAccu maccu;
		bool reportAccu;

		void initVisualizer();

		// it would be better so...(private)
		void resizeGL( int w, int h );
		void paintGL();
		void initializeGL();

		void resizeControls( int w, int h);
		void paintControls();

		void mouseMoveEvent( QMouseEvent* e );
		void mousePressEvent ( QMouseEvent * e );
		void mouseReleaseEvent ( QMouseEvent * e );
		void wheelEvent ( QWheelEvent * e );
		void keyReleaseEvent( QKeyEvent *e );
		void keyPressEvent( QKeyEvent *e );

		boost::shared_ptr<GroupBoxGL> m_navigate_options_box, m_actions_box;
		boost::shared_ptr<ExclusiveOptionsGroup> m_navigate_options_group;
		boost::shared_ptr<SliderGL>  m_dots_slider, m_depth_slider;
		ControlGL * m_focused;
		boost::shared_ptr<ImageGL>  m_image_dots, m_image_depth;
		CheckButtonGL * m_animate_checkbox;
};

#endif
