

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "groupboxgl.h"

#include <GL/gl.h>

using std::string;

const double margin = 2.0;

GroupBoxGL::GroupBoxGL() : m_focused(NULL)
{
}

GroupBoxGL::~GroupBoxGL()
{
	ContentT::const_iterator it = m_content.begin();
	while(it != m_content.end()) {
		delete *it;
		++it;
	}
}

void GroupBoxGL::initialize(QGLWidget* binder)
{
	ContentT::iterator it = m_content.begin();
	while(it != m_content.end()) {
		(*it)->initialize(binder);
		++it;
	}

	double maxWidth = getMaxWidth();

	double Y = 0.0;
	it = m_content.begin();
	while(it != m_content.end()) {
//		double w = (*it)->width();
		double X = margin/* + (maxWidth - w) * 0.5*/;
		(*it)->setPosition(QPointF(X, margin + Y));
		Y += (*it)->height();
		++it;
	}

	size = QSizeF(maxWidth + margin * 2, Y + margin * 2);
}

void GroupBoxGL::cleanup()
{
	ContentT::const_iterator it = m_content.begin();
	while(it != m_content.end()) {
		(*it)->cleanup();
		++it;
	}
}

void GroupBoxGL::paint() const
{
	glTranslated(position.x(), position.y(), 0.0);
	ContentT::const_iterator it = m_content.begin();
	while(it != m_content.end()) {
		(*it)->paint();
		++it;
	}
	glTranslated(-position.x(), -position.y(), 0.0);
}

void GroupBoxGL::addControl(ControlGL * control)
{
	m_content.push_back(control);
}

double GroupBoxGL::getMaxWidth() const
{
	double maxWidth = 0;
	ContentT::const_iterator it = m_content.begin();
	while(it != m_content.end()) {
		double w = (*it)->width();
		maxWidth = w < maxWidth ? maxWidth : w;
		++it;
	}
	return maxWidth;
}

void GroupBoxGL::pressed(double x, double y)
{
	double localX = x - this->x(), localY = y - this->y();
	if(m_focused != NULL) {
		m_focused->released(localX, localY);
		m_focused = NULL;
	}

	ContentT::const_iterator it = m_content.begin();
	while(it != m_content.end()) {
		QRectF itemRect( (*it)->rect() );
		if( itemRect.contains(localX, localY) ) {
			m_focused = *it;
			m_focused->pressed(localX, localY);
			return;
		}
		++it;
	}
}

void GroupBoxGL::released(double x, double y)
{
	double localX = x - this->x(), localY = y - this->y();
	if(m_focused != NULL) {
		m_focused->released(localX, localY);
		m_focused = NULL;
	}
}

void GroupBoxGL::moved(double x, double y)
{
	double localX = x - this->x(), localY = y - this->y();
	if(m_focused != NULL) {
		QRectF itemRect( m_focused->rect() );
		if(!itemRect.contains(localX, localY)) {
			m_focused->released(localX, localY);
			m_focused = NULL;
		} else {
			m_focused->moved(localX, localY);
		}
	}
}
