

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef NVIEWFORM_H
#define NVIEWFORM_H

#include <QMainWindow>

#include "ui_mainwindow.h"

namespace batch
{
	class Config;
}
class Selector;
class NViewList;
class Controller;
class GLWidget;
class DirTreeWidget;
class FeaturesView;
class QLabel;
class Splitter;
class AboutBox;

class MainWindow : public QMainWindow, private Ui::MainWindow
{
		Q_OBJECT

	public:
		enum Mode { NonDemo, Demo };
		MainWindow( Mode mode = NonDemo, QWidget* parent = 0 );
		~MainWindow();

		void setController( Controller* c );
		GLWidget* getGLWidget() { return glView; }
		FeaturesView* getFeaturesView() const { return fView; }
		NViewList* getSetsView();
		batch::Config getConfig()const;

		Mode getMode()const { return working_mode; }

	public slots:
		void open();
		void openImageDir();
		void save();
		void saveAs();
		void to3D();
		void to3D_2();
		void setSimpleSaving( bool );
		void setEmbedImages( bool );
		void setStoreDescrs( bool );
		void loadRecipe( std::string &r );
		void load( const QString& file );

	private slots:
		void slotLoadFromCamera();
		void slotStandard();
		void slotReset();
		void slotResetButViews();
		void slotChangeVisualizer();
		void slotShowAboutBox();

	private:
		Mode working_mode;

		Selector* selector;
		FeaturesView* fView;
		GLWidget* glView;
		NViewList* list;
		Controller* controller;
		Splitter * m_splitter3d_2d, * m_splitter_3d_conf, * m_splitter_recept_tree;
		AboutBox * aboutBox;

		void createActions();
		void createMenus();
		void createToolbars();
		
		QAction* aboutAct;//displays the about dialog
		QAction* changeVisualAct;
		QAction* toggleCamsAct;
		QLabel* visualNameLabel;
		QAction* resetAllButViewsAct;
		QAction* embedImagesAct;
		QAction* storeDescriptorsAct;
		QAction* m_showPointsAct, * m_showLinesAct, * m_showPcorrsAct, * m_showLcorrsAct;
		bool simpleSaving, embedImages, storeDescr;

		void readSettings();
		void writeSettings();

		bool downloadImageScript();
		void findGoodZoom();

		void setupNonDemo();
		void setupDemo();

		void closeEvent( QCloseEvent* event );

		QString lastCHE;
		QString lastDir;
		QString lastImgDir;

		DirTreeWidget* browser;
};

#endif
