

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <QGridLayout>
#include <QLabel>
#include <QLineEdit>
#include <QComboBox>
#include <QCheckBox>
#include <QIntValidator>
#include <QDoubleValidator>
#include <QDialogButtonBox>

#include "optionchooser.h"

OptionChooser::OptionChooser( const QString & ui, QWidget* parent )
		: QDialog( parent )
{
	batch::Parser parser;
	bool ok = parser.parseOptions( ui.toStdString(), optList );
	if ( !ok ) return;
	int cnt = optList.size();
	if ( !cnt ) return;

	QGridLayout* layout = new QGridLayout( this );
	int i;
	for ( i = 0; i < cnt; ++i )
	{
		QString type = QString::fromStdString( optList[i].type );
		int val_cnt = optList[i].values.size();
		layout->addWidget( new QLabel( QString::fromStdString( optList[i].name ) ), i, 0 );
		QVector < QObject* > controls;
		for ( int j = 0; j < val_cnt; ++j )
		{
			if ( "int" == type || "double" == type )
			{
				QLineEdit* ledit = new QLineEdit( QString::fromStdString( optList[i].values[j] ) );
				ledit->setMinimumWidth( 50 );
				ledit->setMaximumWidth( 100 );
				if ( "int" == type ) ledit->setValidator( new QIntValidator( 0, 1000000, ledit ) );
				else if ( "double" == type ) ledit->setValidator( new QDoubleValidator( 0, 1000000, 10, ledit ) );
				layout->addWidget( ledit, i, j + 1 );
				controls.append( ledit );
			}
			else if ( "enum" == type )
			{
				QComboBox* box = new QComboBox;
				std::vector< std::string > enums = parser.getEnums( optList[i].name );
				int enum_cnt = enums.size();
				int curidx = -1;
				for ( int k = 0; k < enum_cnt; ++k )
				{
					box->addItem( QString::fromStdString( enums[k] ) );
					if ( enums[k] == optList[i].values[j] ) curidx = k;
				}
				if ( -1 < curidx ) box->setCurrentIndex( curidx );
				layout->addWidget( box, i, j + 1 );
				controls.append( box );
			}
			else if ( "bool" == type )
			{
				QCheckBox* box = new QCheckBox;
				box->setChecked( "true" == optList[i].values[j] ? true : false );
				layout->addWidget( box, i, j + 1 );
				controls.append( box );
			}
		}
		values.append( controls );
	}

	QDialogButtonBox* box = new QDialogButtonBox( QDialogButtonBox::Ok | QDialogButtonBox::Cancel );
	connect( box, SIGNAL( accepted() ), this, SLOT( accept() ) );
	connect( box, SIGNAL( rejected() ), this, SLOT( reject() ) );
	layout->addWidget( box, i, 0, 1, -1 );
	setLayout( layout );
}

QString OptionChooser::options() const
{
	int cnt = optList.size();
	QString retval;
	for ( int i = 0; i < cnt; ++i )
	{
		retval += "<option>";
		retval += "<type>" + QString::fromStdString( optList[i].type ) + "</type>";
		retval += "<name>" + QString::fromStdString( optList[i].name ) + "</name>";
		int val_cnt = optList[i].values.size();
		for ( int j = 0; j < val_cnt; ++j )
		{
			if ( values[i][j]->inherits( "QLineEdit" ) )
			{
				QLineEdit* ledit = static_cast<QLineEdit*>( values[i][j] );
				retval += "<value>" + ledit->text() + "</value>";
			}
			else if ( values[i][j]->inherits( "QComboBox" ) )
			{
				retval += "<value>" + static_cast<QComboBox*>( values[i][j] )->currentText() + "</value>";
			}
			else if ( values[i][j]->inherits( "QCheckBox" ) )
			{
				bool checked = static_cast<QCheckBox*>( values[i][j] )->isChecked();
				QString t( "true" ), f( "false" );
				retval += "<value>" + ( checked ? t : f ) + "</value>";
			}
		}
		retval += "</option>\n";
	}
	return retval;
}
