

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef QGLHELPER_H
#define QGLHELPER_H

#include <vector>
#include <map>
#include <iostream>

#include "singletonholder.h"

#include <GL/gl.h>
#include <GL/glu.h>

class QGLWidget;

class QGLHelper {

	template<typename T> friend class SingletonHolder;
	typedef SingletonHolder<QGLHelper> Holder;

	typedef std::map<QGLWidget *, std::pair<GLuint, GLuint> > TexturesT;
	TexturesT m_textures;
	QGLWidget * m_current_context_holder;

	bool multisamplingAvailable;

	QGLHelper();
	QGLHelper(const QGLHelper&) {}
	QGLHelper& operator=(const QGLHelper&) { return *this; }

	static GLuint makeCircleTexture(double stop1, double stop2);

	double lineWidth;
	bool started;
	std::vector<double> coordsCache;

	static void aaGlLine( double x1, double y1, double x2, double y2 );

public:
	static void checkMultisampling();
	static bool isMultisamplingAvailable();

	static void prepareAATextures(QGLWidget * contextholder);
	static void setCurrentContext(QGLWidget * contextholder);

	static void aaGlLineWidth(double width);
	static void aaGlBeginLines();
	static void aaGlEndLines();
	static void aaGlVertex2(double x, double y);
};

class GLEnabler {
	GLenum feature;

	GLEnabler(const GLEnabler&) {}
	GLEnabler& operator=(const GLEnabler&) { return *this; }

public:
	GLEnabler(GLenum f) : feature(f) { glEnable(f); }
	~GLEnabler() { glDisable(feature); }
};

inline void glError(const char* str = NULL)
{
	GLenum err = glGetError();
	if(GL_NO_ERROR == err) return;
	const GLubyte* errString = gluErrorString(err);
	if(str) std::cerr << str << " : ";
	std::cerr << "glGetError(): " << errString << std::endl;
}

#endif //QGLHELPER_H
