

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <QtGui/QToolButton>
#include <QtGui/QVBoxLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QPainter>
#include <QtGui/QPaintEvent>
#include <QtGui/QMouseEvent>
#include <QtGui/QLinearGradient>

#include "splitter.h"

SplitterHandle::SplitterHandle(Qt::Orientation orientation, QSplitter * parent, int thickness) :
		QSplitterHandle(orientation, parent)
{
	m_btnl = new QToolButton;
	m_btnl->setIconSize(QSize(8, 8));
	m_btnl->setCursor(Qt::ArrowCursor);
	m_btnr = new QToolButton;
	m_btnr->setIconSize(QSize(8, 8));
	m_btnr->setCursor(Qt::ArrowCursor);

	thickness = std::max((int)GoodThickness, thickness);

	QBoxLayout * lay;
	QSpacerItem * spacer;
	if(orientation == Qt::Horizontal) {
		setMinimumWidth(thickness);
		lay = new QVBoxLayout;
		spacer = new QSpacerItem(0, 1, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);
		m_btnl->setIcon(QIcon(":/images/arrow_left.png"));
		m_btnr->setIcon(QIcon(":/images/arrow_right.png"));
		m_brush.setTextureImage(QImage(":/images/splitter_hhandle.png"));
		connect(m_btnl, SIGNAL(clicked()), this, SLOT(leftClicked()));
		connect(m_btnr, SIGNAL(clicked()), this, SLOT(rightClicked()));
	}
	else {
		setMinimumHeight(thickness);
		lay = new QHBoxLayout;
		spacer = new QSpacerItem(1, 0, QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
		m_btnl->setIcon(QIcon(":/images/arrow_down.png"));
		m_btnr->setIcon(QIcon(":/images/arrow_up.png"));
		m_brush.setTextureImage(QImage(":/images/splitter_vhandle.png"));
		connect(m_btnl, SIGNAL(clicked()), this, SLOT(downClicked()));
		connect(m_btnr, SIGNAL(clicked()), this, SLOT(upClicked()));
	}

	lay->setContentsMargins(0, 0, 0, 0);
	lay->setSpacing(8);
	lay->addWidget(m_btnl);
	lay->addWidget(m_btnr);
	lay->addSpacerItem(spacer);
	setLayout(lay);
}

QSize SplitterHandle::sizeHint () const
{
	if(orientation() == Qt::Horizontal)
		return QSize(minimumWidth(), 0);
	else
		return QSize(0, minimumHeight());

}

void SplitterHandle::paintEvent(QPaintEvent *event)
{
	QRect rect = event->rect();
	QPainter painter(this);
	if(orientation() == Qt::Horizontal) {
		int y = m_btnr->y() + m_btnr->height() + 4;
		rect.setTop(rect.top() + y);
	} else {
		int x = m_btnr->x() + m_btnr->width() + 4;
		rect.setLeft(rect.left() + x);
	}
	painter.fillRect(rect, m_brush);
}

void SplitterHandle::leftClicked()
{
	emit springLeft();
}

void SplitterHandle::rightClicked()
{
	emit springRight();
}

void SplitterHandle::downClicked()
{
	emit springDown();
}

void SplitterHandle::upClicked()
{
	emit springUp();
}

Splitter::Splitter(Qt::Orientation orientation, QWidget * parent) :
		QSplitter(orientation, parent)
{
}

QSplitterHandle * Splitter::createHandle()
{
	SplitterHandle * h = new SplitterHandle(orientation(), this);
	connect(h, SIGNAL(springLeft()), this, SLOT(leftClicked()));
	connect(h, SIGNAL(springRight()), this, SLOT(rightClicked()));
	connect(h, SIGNAL(springDown()), this, SLOT(downClicked()));
	connect(h, SIGNAL(springUp()), this, SLOT(upClicked()));
	return h;
}

void Splitter::leftClicked()
{
	upClicked();
}

void Splitter::rightClicked()
{
	downClicked();
}

void Splitter::downClicked()
{
	QList<int> szs = sizes();
	if(szs.size() != 2) return;
	if(szs[0] == 0) {
		szs[0] = szs[1] / 2;
		szs[1] = szs[1] - szs[0];
	} else {
		szs[0] += szs[1];
		szs[1] = 0;
	}
	setSizes(szs);
}

void Splitter::upClicked()
{
	QList<int> szs = sizes();
	if(szs.size() != 2) return;
	if(szs[1] == 0) {
		szs[1] = szs[0] / 2;
		szs[0] = szs[0] - szs[1];
	} else {
		szs[1] += szs[0];
		szs[0] = 0;
	}
	setSizes(szs);
}
